/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The Software and/or source code cannot be copied in whole and
sold without meaningful modification for a profit.

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with
the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/

/*------------------------------------------------------------------------
    File        : adrtb_createrelease.p
    Purpose     : Procedure using RTB API and ADRTB API to create release in RTB worksspace

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_createrelease.p -param rtbInstallPath,workspaceId,UserId,Password,ReleaseNote

    Description : appSolute Deployment Procedure for creating release in RTB using RTB and ADRTB API

    Created     : Fri Nov 14 09:53:22 CET 2008
    Notes       :
                11/2013 Thomas Hansen
                Restructured code to use ABL structured error handling better.
  ----------------------------------------------------------------------*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

/* ***************************  Definitions  ************************** */
DEFINE OUTPUT PARAMETER poReleaseNumber AS INTEGER NO-UNDO.
DEFINE OUTPUT PARAMETER pcReleaseReport AS CHARACTER NO-UNDO.

/* Standard variable definitions for RTB API usage */
{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

/* Specific variables for this process */
DEFINE VARIABLE gcReleaseNote    AS CHARACTER NO-UNDO.
DEFINE VARIABLE giEventCounter   AS INTEGER   NO-UNDO.
DEFINE VARIABLE glOnlyWhenNeeded AS LOGICAL   NO-UNDO.
DEFINE VARIABLE glCreateReport   AS LOGICAL   NO-UNDO.
DEFINE VARIABLE cReleaseNote     AS CHARACTER NO-UNDO.
DEFINE VARIABLE cReleaseReport   AS CHARACTER NO-UNDO.

ASSIGN
    giParamsExpected = 11
    glDebug          = TRUE
    gcLogFile        = "logs/adrtb_createrelease.log".

RUN initialize.

RUN processReleases.

RETURN "0". /* Success */

CATCH Err AS Progress.Lang.Error :
    ASSIGN
        cErrors = SUBSTITUTE(" *** &1":U, Err:GetMessage(1)).

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    /*    RUN shutdown.*/

    RETURN "1". /* Failure */

END CATCH.

FINALLY:
    RUN shutdown.
END FINALLY.

/* **********************  Internal Procedures  *********************** */

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

    RETURN.

    CATCH err AS Progress.Lang.Error:
        /***DIAG***/
        MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
            "error: " err:GetMessage(1)
            VIEW-AS ALERT-BOX INFO BUTTONS OK.

        UNDO, THROW err.
    END CATCH.

    FINALLY:
        DELETE OBJECT hPropertiesTT NO-ERROR.
    END FINALLY.
END PROCEDURE.

PROCEDURE processReleases:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cWorkspace      AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cError          AS CHARACTER NO-UNDO.
    DEFINE VARIABLE i               AS INTEGER   NO-UNDO.
    DEFINE VARIABLE cReleaseReport  AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lReleaseCreated AS LOGICAL   NO-UNDO INITIAL FALSE.

    IF NUM-ENTRIES(gcWorkspaceList) > 1 THEN
    DO:
        ASSIGN
            gcWorkspaceList = TRIM(gcWorkspaceList)
            gcWorkspaceList = TRIM(gcWorkspaceList, ",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, ",,",",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, "~n","":U) NO-ERROR.

        ASSIGN
            cMessage = SUBSTITUTE("  Processing workspaces : &1...",  gcWorkspaceList).
        oRTBAutomationApi:displayMessage(cMessage, YES, YES).
    END.

    DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, NEXT:
        ASSIGN
            cWorkspace = TRIM(ENTRY(i, gcWorkspaceList))
            cWorkspace = TRIM(cWorkspace, ",":U).

        IF NUM-ENTRIES(gcWorkspaceList) > 1 THEN
            ASSIGN
                cMessage = "  /*****************************************/".

        IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN
        DO ON ERROR UNDO:
            ASSIGN
                cMessage = cMessage + SUBSTITUTE("~n   Creating release in workspace: &1", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).

            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).

            lReleaseCreated = oRTBAutomationApi:createRelease (
                INPUT  cWorkspace,
                INPUT  gcUser,
                INPUT  gcReleaseNote,
                INPUT  glOnlyWhenNeeded,  /* Only create a release if needed */
                OUTPUT poReleaseNumber,
                OUTPUT giEventCounter).

            IF lReleaseCreated THEN
            DO:
                ASSIGN
                    cMessage = "   Release created in workspace: " + cWorkspace + "~n":U +
                               "    Release number              : " + STRING(poReleaseNumber, ">>>,>>>,999") + "~n":U +
                               "    Event counter               : " + STRING(giEventCounter, ">>>,>>>,999").
                oRTBAutomationApi:displayMessage(cMessage, YES, YES).

                oRTBAutomationApi:getRtbReleaseInfo(
                    INPUT  cWorkspace,
                    INPUT  poReleaseNumber,
                    OUTPUT giEventCounter,
                    OUTPUT cReleaseNote).

                IF oRTBAutomationApi:RtbReportBaseDir = "":U OR
                    oRTBAutomationApi:RtbReportBaseDir = ? THEN
                DO:
                    ASSIGN
                        cMessage = "~n*** Release report root directory not set. Cannot create release report!":U.
                    oRTBAutomationApi:displayMessage(cMessage, YES, YES).
                    RETURN.
                END.

                /* Check if we should be creating a release report - if so do it */
                IF glCreateReport AND
                    poReleaseNumber NE 0 THEN
                DO ON ERROR UNDO:
                    ASSIGN
                        cMessage = "   Creating release report, please wait...":U.
                    oRTBAutomationApi:displayMessage(cMessage, YES, YES).

                    oRTBAutomationApi:createReleaseReport (
                        INPUT  cWorkspace,
                        INPUT  poReleaseNumber - 1,
                        INPUT  poReleaseNumber,
                        INPUT  SUBSTITUTE("&1/&2/&3", oRTBAutomationApi:RtbReportBaseDir, "Releases":U, oRTBAutomationApi:RtbWorkspaceId),
                        OUTPUT cReleaseReport).

                    ASSIGN
                        cMessage        = "   Release report created and written to file : ":U + cReleaseReport
                        pcReleaseReport = pcReleaseReport + ",":U + cReleaseReport
                        pcReleaseReport = REPLACE(pcReleaseReport, "~\":U, "/":U)
                        pcReleaseReport = TRIM(pcReleaseReport, ",":U).

                    oRTBAutomationApi:displayMessage(cMessage, YES, YES).

                    CATCH err AS Progress.Lang.Error :
                        IF oRTBAutomationApi:DebugOn THEN
                            oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).

                        UNDO, THROW err.
                    END CATCH.
                END.
            END.

            CATCH err AS Progress.Lang.Error :
                IF oRTBAutomationApi:DebugOn THEN
                    oRTBAutomationApi:DisplayMessage(SUBSTITUTE("   ** &1 ", err:GetMessage(1)), YES, YES).

                UNDO, THROW err.
            END CATCH.
        END.
        ELSE
        DO:
            ASSIGN
                cMessage = SUBSTITUTE("***   Invalid workspace '&1'. Release processing cancelled for this workspace!", cWorkspace).
            UNDO, THROW NEW Progress.Lang.AppError(cMessage, 0).
        END.

        CATCH err AS Progress.Lang.Error :
            oRTBAutomationApi:displayMessage(err:GetMessage(1), YES, YES).

            ASSIGN
                glProcessFailed = TRUE.
        END CATCH.
    END.

    IF glProcessFailed THEN
        UNDO, THROW NEW Progress.Lang.AppError("Workspace release processing failures occured. Overall processing failed.", 0).

    ASSIGN
        cMessage = "   Release processing complete.".
    oRTBAutomationApi:displayMessage(cMessage, TRUE, TRUE).

    RETURN.

    CATCH err AS Progress.Lang.Error :
        IF oRTBAutomationApi:DebugOn THEN
            oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).

        UNDO, THROW err.
    END CATCH.
END PROCEDURE.

PROCEDURE readparams:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    IF ghBuf:AVAILABLE THEN
    DO:
        ASSIGN
            gcRTBInstallPath         = ghBuf::RtbInstallPath
            gcUser                   = ghBuf::RtbUser
            gcPassword               = ghBuf::RtbPassword
            gcWorkspaceList          = ghBuf::RtbReleaseWspaceId
            gcReleaseNote            = ghBuf::RtbReleaseReleaseNote
            glOnlyWhenNeeded         = ghBuf::RtBReleaseOnlyWhenNeeded
            glCreateReport           = ghBuf::RtBReleaseCreateReport
            gcAppServerConnectString = ghBuf::RtbAppServerConnection
            glUseAppServer           = ghBuf::RtbUseAppServer
            gcReportBaseDir          = ghBuf::RtbReportBaseDir.

    END.

    RETURN.

END PROCEDURE.

PROCEDURE shutdown:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    /***DIAG***/
    MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
        "Shutting down..."
        VIEW-AS ALERT-BOX INFO BUTTONS OK.

    oRTBAutomationApi:stopAPIs().

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        DELETE OBJECT oRTBAutomationApi.

    RETURN.

END PROCEDURE.


