/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The Software and/or source code cannot be copied in whole and
sold without meaningful modification for a profit.

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with
the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/

/*------------------------------------------------------------------------
    File        : adrtb_makedeployment.p
    Purpose     : Procedure using RTB API and RTBAutomationApi to create deployment in RTB workspace

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_makedeployment.p -param rtbInstallPath,workdpaceId,UserId,Password,deploymentSite

    Description : appSolute Deployment Procedure for making a deployment from RTB using RTB and ADRTB API

    Created     : Oct 21 2012
    Notes       :
                11/2013 Thomas Hansen
                Restructured code to use ABL structured error handling better.
  ----------------------------------------------------------------------*/
USING scm.custom.rtb.base.* FROM PROPATH.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

/* ***************************  Definitions  ************************** */
DEFINE OUTPUT PARAMETER pcReleaseReport    AS CHARACTER NO-UNDO.

/* Standard variable definitions for RTB API usage */
{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

DEFINE VARIABLE gcDeploymentSiteCode        AS CHARACTER            NO-UNDO.
DEFINE VARIABLE giDeploymentSequence        AS INTEGER              NO-UNDO.
DEFINE VARIABLE giDeploymentReleaseNum      AS INTEGER              NO-UNDO.
DEFINE VARIABLE gcDeploymentDir             AS CHARACTER            NO-UNDO.
DEFINE VARIABLE glIncludeSchema             AS LOGICAL              NO-UNDO.
DEFINE VARIABLE glMarkCompleteAfterMake     AS LOGICAL              NO-UNDO INITIAL FALSE.
DEFINE VARIABLE cRowIdent                   AS CHARACTER            NO-UNDO.
DEFINE VARIABLE hDeploymentData             AS HANDLE               NO-UNDO.
DEFINE VARIABLE gcDeploymentBaseDir         AS CHARACTER            NO-UNDO.
DEFINE VARIABLE glCreateReleaseReport       AS LOGICAL              NO-UNDO.

ASSIGN
    giParamsExpected = 16
    gcLogFile        = "logs/adrtb_deployment.log".

RUN initialize.

RUN processDeployment.

RETURN "0". /* Success */

CATCH Err AS Progress.Lang.Error :
    ASSIGN
        cErrors = SUBSTITUTE(" *** &1", Err:GetMessage(1)).

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    RUN shutdown.

    RETURN "1". /* Failure */
END CATCH.

FINALLY:
    RUN shutdown.
END FINALLY.
/* **********************  Internal Procedures  *********************** */

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

   RETURN.

   CATCH LangError AS Progress.Lang.Error:
      UNDO, THROW LangError.
   END CATCH.

END PROCEDURE.

PROCEDURE readParams:
   /*------------------------------------------------------------------------------
    Purpose:
    Notes:
   ------------------------------------------------------------------------------*/

   IF ghBuf:AVAILABLE THEN
   DO:
      ASSIGN
         glDebug                  = ghBuf::BuildDebug
         gcRTBInstallPath         = ghBuf::RtbInstallPath
         gcUser                   = ghBuf::RtbUser
         gcPassword               = ghBuf::RtbPassword
         gcWorkspaceList          = ghBuf::RtbDeploymentWspaceId
         gcDeploymentSiteCode     = ghBuf::RtbDeploymentDeploymentSiteCode
         giDeploymentSequence     = ghBuf::RtbDeploymentDeploymentSequence
         giDeploymentReleaseNum   = ghBuf::RtbDeploymentDeploymentReleaseNum
         gcDeploymentDir          = ghBuf::RtbDeploymentDeploymentDir
         glIncludeSchema          = ghBuf::RtbDeploymentIncludeSchema
         glMarkCompleteAfterMake  = ghBuf::RtbDeploymentMarkCompleteAfterMake
         gcDeploymentBaseDir      = ghBuf::RtbDeploymentBaseDir
         glCreateReleaseReport    = ghBuf::RtbDeploymentCreateReport
         gcAppServerConnectString = ghBuf::RtbAppServerConnection
         glUseAppServer           = ghBuf::RtbUseAppServer
         gcReportBaseDir          = ghBuf::RtbReportBaseDir
        NO-ERROR.
   END.

   RETURN.

END PROCEDURE.

PROCEDURE shutdown:
   /*------------------------------------------------------------------------------
           Purpose:
           Notes:
   ------------------------------------------------------------------------------*/

   oRTBAutomationApi:stopAPIs().

   IF VALID-OBJECT(oRTBAutomationApi) THEN
      DELETE OBJECT oRTBAutomationApi.

   RETURN.

END PROCEDURE.

PROCEDURE processDeployment:
   /*------------------------------------------------------------------------------
           Purpose:
           Notes:
   ------------------------------------------------------------------------------*/
   DEFINE VARIABLE cWorkspace          AS CHARACTER NO-UNDO.
   DEFINE VARIABLE i                   AS INTEGER   NO-UNDO.
   DEFINE VARIABLE cError              AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cMessage            AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cErrorMessages      AS CHARACTER NO-UNDO.
   DEFINE VARIABLE iDeploymentSequence AS INTEGER NO-UNDO.

   IF NUM-ENTRIES(gcWorkspaceList) > 1 THEN
   DO:
      ASSIGN
         gcWorkspaceList = TRIM(gcWorkspaceList)
         gcWorkspaceList = TRIM(gcWorkspaceList, ",":U)
         gcWorkspaceList = REPLACE(gcWorkspaceList, ",,",",":U)
         gcWorkspaceList = REPLACE(gcWorkspaceList, "~n","":U) NO-ERROR.

      ASSIGN
         cMessage = SUBSTITUTE("Processing deployments in workspace(s) '&1' for site '&2'~n~n",gcWorkspaceList, gcDeploymentSiteCode).
      oRTBAutomationApi:displayMessage(cMessage, YES, glDebug).
   END.
   ELSE
      iDeploymentSequence = giDeploymentSequence.

   DeploymentLoop:
   DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, THROW:
      ASSIGN
         cWorkspace = ENTRY(i, gcWorkspaceList)
         cWorkspace = TRIM(cWorkspace, ",":U)
         iDeploymentSequence= 0.

      IF i > 1 THEN
         ASSIGN
            cMessage = "  /*****************************************/" + "~n~n".

      ASSIGN
         cMessage = cMessage + SUBSTITUTE("  Processing workspace: &1...",  cWorkspace) + "~n  /*****************************************/".
      oRTBAutomationApi:displayMessage(cMessage, YES, YES).

      IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN
      DO ON ERROR UNDO, THROW:
         oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).

         IF cError NE "":U THEN
            UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

         ASSIGN
            cMessage = SUBSTITUTE("  Processing deployment in workspace: &1...",  cWorkspace).
         oRTBAutomationApi:displayMessage(cMessage, YES, YES).

         oRTBAutomationApi:processDeployment (
            INPUT         cWorkspace,
            INPUT         gcDeploymentSiteCode,
            INPUT         giDeploymentReleaseNum,
            INPUT         gcDeploymentDir,
            INPUT         glIncludeSchema,
            INPUT         glCreateReleaseReport,
            INPUT-OUTPUT  iDeploymentSequence, /* set to 0 to create new deployment record first*/
            INPUT glMarkCompleteAfterMake).

         IF glCreateReleaseReport THEN
            ASSIGN
               pcReleaseReport = pcReleaseReport + ",":U + oRTBAutomationApi:RtbReleaseReport.

         CATCH err AS Progress.Lang.Error :
            ASSIGN
               cMessage = SUBSTITUTE(" *** Deployment processing error for workspace '&1':~n    &2", cWorkspace, err:GetMessage(1)).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).
         END CATCH.
      END.
    END. /* DO i = 1 TO NUM-ENTRIES(gcWorkspaceList)...*/

   IF glProcessFailed THEN
      UNDO, THROW NEW Progress.Lang.AppError("Workspace deployment processing failures occured. Overall processing failed.", 0).

   ASSIGN
      pcReleaseReport = TRIM(pcReleaseReport, ",":U).

   ASSIGN
      cMessage = "   Deployment processing complete.~n~n".
   oRTBAutomationApi:displayMessage(cMessage, TRUE, TRUE).

   RETURN.

   CATCH Err AS Progress.Lang.Error :
      IF oRTBAutomationApi:DebugOn THEN
         oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
      UNDO, THROW Err.
   END CATCH.

END PROCEDURE.

