/*********************************************************************
Copyright (C) 2012-2016 by nodeable limited ("nodeable") -
 www.nodeable.io and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The Software and/or source code cannot be copied in whole and
sold without meaningful modification for a profit.

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with
the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/

/*------------------------------------------------------------------------
    File        : adrtb_processimport.p
    Purpose     : Procedure using RTB API and ADRTB API to process imports in RTB worksspaces

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_processimport.p -param rtbInstallPath,workdpaceId,UserId

    Description : appSolute Deployment Procedure for processing imports in RTB using ARAT

  ----------------------------------------------------------------------*/
USING scm.custom.rtb.base.* FROM PROPATH.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

/* ***************************  Definitions  ************************** */
DEFINE OUTPUT PARAMETER pcImportAnalaysisReport      AS CHARACTER NO-UNDO.

/* Standard variable definitions for RTB API usage */
{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

DEFINE VARIABLE glReportAfterApply           AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glDeleteAfterApply           AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE lImportListExists            AS LOGICAL   NO-UNDO.
DEFINE VARIABLE cImportReport                AS CHARACTER NO-UNDO.
DEFINE VARIABLE iNumRecords                  AS INTEGER   NO-UNDO.
DEFINE VARIABLE glCompileAfterImport         AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glCreateRtbTaskForImport     AS LOGICAL   NO-UNDO INITIAL TRUE.
DEFINE VARIABLE glCompleteRtbTaskAfterImport AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glProcessImportTableTasks    AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glFailOnImportCompileErrors   AS LOGICAL              NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glCompiledSuccessfully        AS LOGICAL              NO-UNDO INITIAL TRUE.
DEFINE VARIABLE gcWorkspacesWithCompileErrors AS CHARACTER            NO-UNDO INITIAL "".
DEFINE VARIABLE giNumEntries                  AS INTEGER              NO-UNDO.
DEFINE VARIABLE giEntry                       AS INTEGER              NO-UNDO.

ASSIGN
    giParamsExpected = 16
    glDebug          = FALSE
    gcLogFile        = "logs/adrtb_processsimport.log".

RUN initialize.

RUN processImport.

IF glCompiledSuccessfully THEN
DO:
    RETURN "0". /* Success */
END.
ELSE
DO:
    IF gcWorkspacesWithCompileErrors NE "":U THEN
    DO:
        ASSIGN
            giNumEntries = NUM-ENTRIES(gcWorkspacesWithCompileErrors).

        ASSIGN
            cMessage = "    Workspaces with compilation errors:".
        oRTBAutomationApi:displayMessage(cMessage, ?, YES, YES).

        ASSIGN
            cMessage = "    =========================".
        oRTBAutomationApi:displayMessage(cMessage, ?, YES, YES).

        REPEAT giEntry = 1 TO giNumEntries:
            ASSIGN
                cMessage = SUBSTITUTE("    &1", ENTRY(giEntry, gcWorkspacesWithCompileErrors)).
            oRTBAutomationApi:displayMessage(cMessage, ?, YES, YES).
        END.
    END.

    RUN shutdown.

    RETURN "1". /* Failure */
END.

CATCH Err AS Progress.Lang.Error :
    ASSIGN
        cErrors = " ***":U + Err:GetMessage(1).

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    RUN shutdown.

END CATCH.

FINALLY:
    RUN shutdown.
END FINALLY.


/* **********************  Internal Procedures  *********************** */

PROCEDURE applyImportTable:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE INPUT  PARAMETER pcWorkspace AS CHARACTER NO-UNDO.
    DEFINE INPUT  PARAMETER piTaskNum   AS INTEGER   NO-UNDO.

    DEFINE VARIABLE lcError           AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE cMessage          AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lImportListExists AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE iNumRecords       AS INTEGER   NO-UNDO.
    DEFINE VARIABLE lcCompileResults  AS LONGCHAR  NO-UNDO.

    ASSIGN
        cMessage = SUBSTITUTE("   Checking import table for workspace: &1...", pcWorkspace).
    oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).

    lImportListExists = oRTBAutomationApi:checkImportTableExists(pcWorkspace, OUTPUT iNumRecords).

    IF lImportListExists THEN
    DO ON ERROR UNDO:
        ASSIGN
            cMessage = SUBSTITUTE("   Applying workspace import table to workspace : &1", pcWorkspace).
        oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).

        oRTBAutomationApi:applyWorkspaceImportList(
            INPUT  pcWorkspace,
            INPUT  piTaskNum,
            INPUT  oRTBAutomationApi:RtbCompileAfterImport,
            OUTPUT lcCompileResults,
            OUTPUT lcError).

        IF lcCompileResults NE "":U AND
            glFailOnImportCompileErrors THEN
        DO:
            ASSIGN
                glCompiledSuccessfully        = FALSE
                gcWorkspacesWithCompileErrors = gcWorkspacesWithCompileErrors + pcWorkspace + ",".
        END.

        IF lcError NE "":U THEN
        DO:
            /* The following is to create a line in the output, as PCT seems to be stripping blank lines from any output */
            oRTBAutomationApi:displayMessage("***", YES, YES).
            ASSIGN
                lcError = SUBSTITUTE("   RTB import table processing error:~n&1~n~n", lcError).
            oRTBAutomationApi:displayMessage(lcError, YES, YES).
            oRTBAutomationApi:displayMessage("***", YES, YES).
        END.

        ASSIGN
            cMessage = "   Application of workspace import table complete.".
        oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).

        CATCH err AS Progress.Lang.Error :
            oRTBAutomationApi:displayMessage(err:GetMessage(1), YES, YES).
        END CATCH.
    END.
    ELSE
    DO:
        ASSIGN
            cMessage = "   Nothing to import. Processing cancelled.".
        oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).

        RETURN.
    END.

    RETURN.

    CATCH AppError AS Progress.Lang.AppError :
        oRTBAutomationApi:DisplayMessage("Error : " + AppError:GetMessage(1), TRUE, TRUE).
    END CATCH.

END PROCEDURE.


PROCEDURE deleteWorkspaceImportList:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE INPUT  PARAMETER pcWorkspace AS CHARACTER NO-UNDO.

    DEFINE VARIABLE cError   AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.

    ASSIGN
        cMessage = SUBSTITUTE("   Deleting workspace import table from workspace : &1", pcWorkspace).
    oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).

    oRTBAutomationApi:deleteWorkspaceImportList(
        INPUT pcWorkspace,
        OUTPUT cError).

    IF cError NE "":U THEN
        UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("RTB import table deletion error: ", cError), 0).

    ASSIGN
        cMessage = "   Deletion of workspace import table complete.".
    oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).

    RETURN.

END PROCEDURE.

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

    RETURN.

    CATCH LangError AS Progress.Lang.Error:
        UNDO, THROW LangError.
    END CATCH.

END PROCEDURE.

PROCEDURE readParams:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    IF ghBuf:AVAILABLE THEN
    DO:
        ASSIGN
            glDebug                      = ghBuf::BuildDebug
            gcRTBInstallPath             = ghBuf::RtbInstallPath
            gcUser                       = ghBuf::RtbUser
            gcPassword                   = ghBuf::RtbPassword
            gcWorkspaceList              = ghBuf::RtbImportWspaceId
            giMaxWorkspaces              = ghBuf::RtbImportMaxWorkspaces
            glReportAfterApply           = ghBuf::RtbImportReportAfterApply
            glDeleteAfterApply           = ghBuf::RtbImportDeleteAfterApply
            glCompileAfterImport         = ghBuf::RtbImportCompileAfterImport
            glCreateRtbTaskForImport     = ghBuf::RtbImportCreateTaskForImport
            glCompleteRtbTaskAfterImport = ghBuf::RtbImportCompleteImportTask
            glProcessImportTableTasks    = ghBuf::RtbImportProcessImportTableTasks
            gcAppServerConnectString     = ghBuf::RtbAppServerConnection
            glUseAppServer               = ghBuf::RtbUseAppServer
            gcReportBaseDir              = ghBuf::RtbReportBaseDir
            glFailOnImportCompileErrors  = ghBuf::RtbImportFailOnImportCompileErrors.
    END.

    /* Set additional properties in API */
    ASSIGN
        oRTBAutomationApi:CompleteRtbTaskAfterImport = glCompleteRtbTaskAfterImport
        oRTBAutomationApi:ProcessImportTableTasks    = glProcessImportTableTasks
        oRTBAutomationApi:CreateRtbTaskForImport     = glCreateRtbTaskForImport
        oRTBAutomationApi:RtbCompileAfterImport      = glCompileAfterImport.

    RETURN.

END PROCEDURE.

PROCEDURE shutdown:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    oRTBAutomationApi:stopAPIs().

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        DELETE OBJECT oRTBAutomationApi.

    RETURN.

END PROCEDURE.

PROCEDURE processImport:
    /*------------------------------------------------------------------------------
    Purpose: Process importing for a workspace
    Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cWorkspace             AS CHARACTER NO-UNDO.
    DEFINE VARIABLE i                      AS INTEGER   NO-UNDO.
    DEFINE VARIABLE cError                 AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cMessage               AS CHARACTER NO-UNDO.

    DEFINE VARIABLE lWorkspaceTaskRequired AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE cTaskSummary           AS CHARACTER NO-UNDO.
    DEFINE VARIABLE iTaskNum               AS INTEGER   NO-UNDO.

    IF NUM-ENTRIES(gcWorkspaceList) > giMaxWorkspaces THEN
    DO:
        ASSIGN
            cMessage = SUBSTITUTE(" From process API: Number of workspaces specified exceeds the maximum of &1.", giMaxWorkspaces).

        UNDO, THROW NEW Progress.Lang.AppError(cMessage, 0).
    END.

    IF NUM-ENTRIES(gcWorkspaceList) > 1 THEN
    DO:
        ASSIGN
            gcWorkspaceList = TRIM(gcWorkspaceList)
            gcWorkspaceList = TRIM(gcWorkspaceList, ",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, ",,",",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, "~n","":U) NO-ERROR.

        ASSIGN
            cMessage = SUBSTITUTE("  Processing workspaces : &1...",  gcWorkspaceList).
        oRTBAutomationApi:displayMessage(cMessage, YES, YES).
    END.

    ImportLoop:
    DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, NEXT :
        ASSIGN
            cMessage   = "":U
            cWorkspace = TRIM(ENTRY(i, gcWorkspaceList))
            cWorkspace = TRIM(cWorkspace, ",":U).

        IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN
        DO ON ERROR UNDO:
            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

            IF i > 1 THEN
                ASSIGN
                    cMessage = "   /*****************************************/".

            ASSIGN
                cMessage = cMessage + SUBSTITUTE("~n   Processing workspace: &1...",  cWorkspace) + "~n  /*****************************************/".
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).

            lImportListExists = oRTBAutomationApi:checkImportTableExists(cWorkspace, OUTPUT iNumRecords).

            IF lImportListExists THEN
            DO:
                lWorkspaceTaskRequired = oRTBAutomationApi:rtbCheckWorkspaceTaskRequired(cWorkspace).

                IF lWorkspaceTaskRequired = TRUE AND
                    oRTBAutomationApi:CreateRtbTaskForImport = TRUE THEN
                DO ON ERROR UNDO :
                    ASSIGN
                        cTaskSummary = SUBSTITUTE("   &1: Task automatically created for import processing.", cWorkspace).

                    oRTBAutomationApi:RTBProxyHelper:AppServerHandle = oRTBAutomationApi:AppServerHandle.

                    /* Use a custom proxy helper method that also sets various fields on the task */
                    iTaskNum = oRTBAutomationApi:RTBProxyHelper:custRtbCreateTask(
                        INPUT  cWorkspace,
                        INPUT  cTaskSummary,
                        INPUT  "":U, /* pcTaskDescription */
                        INPUT  "RTB Automation":U, /*pcTaskUserTaskref */
                        INPUT  oRTBAutomationApi:RTBUser, /* Programmer */
                        INPUT  oRTBAutomationApi:RTBUser /* pcManager */).

                    IF cError NE "":U THEN
                        UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

                    IF iTaskNum NE 0 THEN
                    DO:
                        oRTBAutomationApi:DisplayMessage(SUBSTITUTE("   Created task : &1 - &2.", iTaskNum, cTaskSummary), TRUE, TRUE).

                        /* set the 'current' task in RTB on the AppServer if we are running with a remote AppServer */
                        oRTBAutomationApi:RTBProxyHelper:rtbSetSessionProperty(
                            INPUT "task":U,
                            INPUT STRING(iTaskNum)).
                    END.

                    CATCH AppErr AS Progress.Lang.AppError :
                        UNDO, THROW AppErr.
                    END CATCH.
                END.

                RUN applyImportTable (
                    INPUT cWorkspace,
                    INPUT iTaskNum ).

                IF glReportAfterApply THEN
                DO:
                    oRTBAutomationApi:createImportAnalysisReport(
                        INPUT  cWorkspace,
                        INPUT  SUBSTITUTE("&1/&2/&3", oRTBAutomationApi:RtbReportBaseDir,"Imports":U, cWorkspace),
                        INPUT  "":U,
                        OUTPUT cImportReport,
                        OUTPUT cError).

                    IF cError NE "":U THEN
                        UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

                    ASSIGN
                        cMessage = "   Import analysis report created and written to file :~n    ":U + cImportReport + "...".
                    oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).

                    ASSIGN
                        pcImportAnalaysisReport = pcImportAnalaysisReport + ",":U + REPLACE(cImportReport, "~\":U, "/":U)
                        pcImportAnalaysisReport = REPLACE(pcImportAnalaysisReport, "~\", "/":U)
                        pcImportAnalaysisReport = TRIM(pcImportAnalaysisReport, ",":U).
                END.

                IF oRTBAutomationApi:IsImportComplete(cWorkspace) THEN
                DO:
                    IF glDeleteAfterApply THEN
                        RUN deleteWorkspaceImportList(cWorkspace).

                    /* If we are to complete the task after a successful importm then complete the task now */
                    IF oRTBAutomationApi:CompleteRtbTaskAfterImport = TRUE AND
                        iTaskNum NE 0 THEN
                    DO ON ERROR UNDO:
                        oRTBAutomationApi:RTBProxyHelper:rtbCompleteTask(iTaskNum).

                        oRTBAutomationApi:DisplayMessage(SUBSTITUTE("   Task &1 completed.", iTaskNum), TRUE, TRUE).

                        CATCH err AS Progress.Lang.Error :
                            UNDO, THROW err.
                        END CATCH.
                    END.
                END.
                ELSE
                DO:
                    ASSIGN
                        cMessage = SUBSTITUTE(" *** Import not completed successfully in workspace '&1'. Import table has not been deleted. Please review the import errors manually!~n    NB: If task '&2' has been created automatically for this import, the task will also be WIP and will have to be completed manually.":U, cWorkspace, iTaskNum).
                    oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).
                END.
            END. /* IF lImportListExists THEN */
            ELSE
            DO:
                ASSIGN
                    cMessage = "   No import table to process.".
                oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).
            END.

            CATCH err AS Progress.Lang.Error :
                ASSIGN
                    cMessage = SUBSTITUTE("   ***Import processing error for workspace '&1': &2", cWorkspace, err:GetMessage(1)).
                oRTBAutomationApi:displayMessage(cMessage, YES, YES).

                ASSIGN
                    glProcessFailed = TRUE.
            END CATCH.
        END.
    END.

    IF glProcessFailed THEN
        UNDO, THROW NEW Progress.Lang.AppError("Workspace deployment processing failures occured. Overall processing failed.", 0).

    RETURN.

    CATCH Err AS Progress.Lang.Error :
        IF oRTBAutomationApi:DebugOn THEN
            oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
        UNDO, THROW Err.
    END CATCH.

END PROCEDURE.

