/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : adrtb_processschema.p
    Purpose     : Procedure using RTB API and ADRTB API to process schema updates in RTB workspaces

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_processimport.p -param rtbInstallPath,UserId,Passsword,workspaceId,SchemaList,ProcessSchema,DeleteAfterApply,DeleteExistingSchupd

    Description : appSolute Deployment Procedure for creating release in RTB using RTB and ADRTB API

    Created     : 29/10/12
    Notes       :
  ----------------------------------------------------------------------*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

DEFINE OUTPUT PARAMETER pcSchemaUpdateLogFiles AS CHARACTER NO-UNDO.

/* ***************************  Definitions  ************************** */

/* Standard variable definitions for RTB API usage */
{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

DEFINE VARIABLE gcSchemaList           AS CHARACTER NO-UNDO.
DEFINE VARIABLE glDeleteExistingSchupd AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glProcessSchema        AS LOGICAL   NO-UNDO INITIAL TRUE.
DEFINE VARIABLE glSkipSchemaAllUpdates AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glDeleteAfterApply     AS LOGICAL   NO-UNDO INITIAL TRUE.

ASSIGN
    giParamsExpected = 13
    glDebug          = FALSE
    gcLogFile        = "logs/adrtb_processschema.log".

RUN INITIALIZE.

ASSIGN
    cMessage = SUBSTITUTE("  Number of workspaces specified: &2, Minimum allowed workspaces: 1, Maximum allowed workspaces: &1", giMaxWorkspaces, NUM-ENTRIES(gcWorkspaceList)).
oRTBAutomationApi:displayMessage(cMessage, TRUE, TRUE).

RUN processSchema.

RETURN "0". /* Success */

CATCH Err AS Progress.Lang.Error :
    ASSIGN
        cErrors = " ***":U + Err:GetMessage(1).

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    RUN shutdown.

    RETURN "1".
END CATCH.

FINALLY:
    RUN shutdown.
END FINALLY.

/* **********************  Internal Procedures  *********************** */

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

    RETURN.

    CATCH Err AS Progress.Lang.Error:
        IF oRTBAutomationApi:DebugOn THEN
            oRTBAutomationApi:DisplayMessage(Err:GetMessage(1), TRUE, TRUE).

        UNDO, THROW Err.
    END CATCH.

END PROCEDURE.

PROCEDURE readParams:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    IF ghBuf:AVAILABLE THEN
    DO:
        ASSIGN
            glDebug                  = ghBuf::BuildDebug
            gcRTBInstallPath         = ghBuf::RtbInstallPath
            gcUser                   = ghBuf::RtbUser
            gcPassword               = ghBuf::RtbPassword
            gcWorkspaceList          = ghBuf::RtbSchemaWspaceId
            giMaxWorkspaces          = ghBuf::RtbSchemaMaxWorkspaces
            gcSchemaList             = ghBuf::RtbSchemaDbList
            glDeleteAfterApply       = ghBuf::RtbSchemaDeleteAfterProcess
            glDeleteExistingSchupd   = ghBuf::RtbSchemaDeleteExistingSchupd
            glSkipSchemaAllUpdates   = ghBuf::RtbSchemaSkipAllSchemaUpdates
            gcAppServerConnectString = ghBuf::RtbAppServerConnection
            glUseAppServer           = ghBuf::RtbUseAppServer
            gcReportBaseDir          = ghBuf::RtbReportBaseDir
        NO-ERROR.
    END.

    RETURN.

END PROCEDURE.

PROCEDURE shutdown:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    oRTBAutomationApi:stopAPIs().

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        DELETE OBJECT oRTBAutomationApi.
END PROCEDURE.

PROCEDURE processSchema:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cWorkspace              AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cMessage                AS CHARACTER NO-UNDO.
    DEFINE VARIABLE i                       AS INTEGER   NO-UNDO.
    DEFINE VARIABLE cError                  AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lSchemaUpdateListExists AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE cWorkspaceRoot          AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cSchemaUpdateLogFile    AS CHARACTER NO-UNDO.

    IF NUM-ENTRIES(gcWorkspaceList) > giMaxWorkspaces THEN
    DO:
        ASSIGN
            cMessage = SUBSTITUTE("  Number of workspaces specified exceeds the maximum of &1.", giMaxWorkspaces).

        UNDO, THROW NEW Progress.Lang.AppError(cMessage, 0).
    END.

    workspaceLoop:
    DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, THROW:
        ASSIGN
            cWorkspace              = ENTRY(i, gcWorkspaceList)
            cSchemaUpdateLogFile    = "":U
            lSchemaUpdateListExists = FALSE.

        IF i > 1 THEN
            ASSIGN
                cMessage = "  /*****************************************/".

        ASSIGN
            cMessage = cMessage + SUBSTITUTE("~n  Processing workspace: &1...",  cWorkspace) + "~n  /*****************************************/".
        oRTBAutomationApi:displayMessage(cMessage, YES, YES).

        IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN
        DO ON ERROR UNDO:
            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).

            ASSIGN
                cMessage = SUBSTITUTE("   Processing schema for workspace: &1", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, TRUE, TRUE).

            /* First, check if we have an existing schema update table */
            lSchemaUpdateListExists = oRTBAutomationApi:checkRtbSchemaUpdateTableExists(cWorkspace, "":U).

            IF lSchemaUpdateListExists THEN
            DO:
                /* If we are supposed to delete the existing schema update table, first, then do this... */
                IF glDeleteExistingSchupd THEN
                DO:
                    ASSIGN
                        cMessage = "   Deleting existing schema update table...".
                    oRTBAutomationApi:displayMessage(cMessage, ?, YES, TRUE).

                    oRTBAutomationApi:deleteSchemaUpdateList(
                        INPUT  cWorkspace,
                        INPUT  "",
                        OUTPUT cError) NO-ERROR.

                    IF cError NE "":U THEN
                    DO:
                        UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
                    END.
                END.
                ELSE
                DO:
                    ASSIGN
                        cError = "Schema update table exists. Delete the existing table and try again...".
                    oRTBAutomationApi:displayMessage(cError, YES, YES).
                    NEXT workspaceLoop.
                /*                    UNDO, THROW NEW Progress.Lang.AppError(cError, 0).*/
                END.
            END.

            CATCH Err AS Progress.Lang.Error :
                UNDO, THROW Err.
            END CATCH.
        END.
        ELSE
        DO:
            ASSIGN
                cMessage = SUBSTITUTE("***   Invalid workspace '&1'. Schema update cancelled for this workspace!", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).
        END.

        oRTBAutomationApi:buildWorkspaceSchemaUpdateList(
            INPUT  cWorkspace,
            INPUT  gcSchemaList,
            OUTPUT lSchemaUpdateListExists,
            OUTPUT cError).

        IF cError NE "":U THEN
            UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Schema update build error: &1", cError), 0).

        IF lSchemaUpdateListExists AND
            glProcessSchema THEN
        DO:
            ASSIGN
                cMessage = "   Processing schema update table...".
            oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).

            ASSIGN
                cError = "":U.  /* Reset to blank */

            oRTBAutomationApi:applySchemaUpdateList(
                INPUT  cWorkspace,
                OUTPUT cSchemaUpdateLogFile,
                OUTPUT cError).

            IF cSchemaUpdateLogFile NE "":U THEN
            DO:
                ASSIGN
                    pcSchemaUpdateLogFiles = pcSchemaUpdateLogFiles + ",":U + REPLACE(cSchemaUpdateLogFile, "~\":U, "/":U)
                    pcSchemaUpdateLogFiles = REPLACE(pcSchemaUpdateLogFiles, "~\", "/":U)
                    pcSchemaUpdateLogFiles = TRIM(pcSchemaUpdateLogFiles, ",":U).
            END.

            IF cError NE "":U THEN
            DO:
                oRTBAutomationApi:displayMessage(cError, YES, TRUE).
                UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Schema update processing error: &1", cError), 0).
            END.

            /* If we have a property set to delete the schema update table after it has been processed, then do this now... */
            IF glDeleteAfterApply THEN
            DO:
                oRTBAutomationApi:deleteSchemaUpdateList(
                    INPUT  cWorkspace,
                    INPUT  "":U,
                    OUTPUT cError).

                IF cError NE "":U THEN
                    UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Schema update list delete error: &1", cError), 0).
            END.
        END.

        CATCH err AS Progress.Lang.Error :
            ASSIGN
                cMessage = SUBSTITUTE(" *** RTB schema processing error for workspace '&1': &2", cWorkspace, err:GetMessage(1)).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).

            ASSIGN
                glProcessFailed = TRUE.
        END CATCH.
    END.

    IF glProcessFailed THEN
        UNDO, THROW NEW Progress.Lang.AppError("Workspace deployment processing failures occured. Overall processing failed.", 0).

    ASSIGN
        cMessage = "   Schema processing complete.".
    oRTBAutomationApi:displayMessage(cMessage, TRUE, TRUE).

    RETURN.

    CATCH Err AS Progress.Lang.Error :
        IF oRTBAutomationApi:DebugOn THEN
            oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
        UNDO, THROW Err.
    END CATCH.

END PROCEDURE.
