/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : adrtb_selectivecompile.p
    Purpose     : Procedure using RTB API and ADRTB API to do a selective compile of a workspace

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_selectivecompile.p
                  -param rtbInstallPath,workdpaceId,UserId,Password,

    Description : appSolute Deployment Procedure for creating release in RTB using RTB and ADRTB API

    Created     : Fri Nov 14 09:53:22 CET 2008
    Notes       :
                11/2013 Thomas Hansen
                Restructured code to use ABL structured error handling better.
  ----------------------------------------------------------------------*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING appSolutions.appSolute.Utils.FileHelper FROM PROPATH.

/* ***************************  Definitions  ************************** */
DEFINE OUTPUT PARAMETER pcCompileLogFile AS CHARACTER NO-UNDO.

{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

/* Specific variables for this process */
DEFINE VARIABLE lImportListExists  AS LOGICAL   NO-UNDO.
DEFINE VARIABLE giTaskNum          AS INTEGER   NO-UNDO.
DEFINE VARIABLE gcModule           AS CHARACTER NO-UNDO.
DEFINE VARIABLE gcGroup            AS CHARACTER NO-UNDO.
DEFINE VARIABLE gcObjectType       AS CHARACTER NO-UNDO.
DEFINE VARIABLE gcObject           AS CHARACTER NO-UNDO.
DEFINE VARIABLE glForceCompile     AS LOGICAL   NO-UNDO.
DEFINE VARIABLE glCompileListing   AS LOGICAL   NO-UNDO.
DEFINE VARIABLE glXref             AS LOGICAL   NO-UNDO.
DEFINE VARIABLE glFailOnError      AS LOGICAL   NO-UNDO.
DEFINE VARIABLE glIgnoreWarnings   AS LOGICAL   NO-UNDO.
DEFINE VARIABLE glTaskRcode        AS LOGICAL   NO-UNDO.
DEFINE VARIABLE glCreateReport     AS LOGICAL   NO-UNDO.
DEFINE VARIABLE gcBuildCompLogFile AS CHARACTER NO-UNDO.
DEFINE VARIABLE glReportAfterApply AS LOGICAL   NO-UNDO INITIAL FALSE.
DEFINE VARIABLE glDeleteAfterApply AS LOGICAL   NO-UNDO INITIAL FALSE.

{rtb/i/rtb_super.i}

ASSIGN
    giParamsExpected = 20
    gcLogFile        = "logs/adrtb_selcomp.log".

RUN initialize.

RUN processCompile.

RETURN "0". /* Success */

CATCH Err AS Progress.Lang.Error :
    ASSIGN
        cErrors = " ***":U + Err:GetMessage(1).

    /***DIAG***/
    MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
        cErrors
        VIEW-AS ALERT-BOX INFO BUTTONS OK.

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    RETURN "1". /* Failure */
END CATCH.

FINALLY:
    RUN shutdown.
END FINALLY.

/* **********************  Internal Procedures  *********************** */

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

    RETURN.

    CATCH LangError AS Progress.Lang.Error:
        UNDO, THROW LangError.
    END CATCH.

    FINALLY:
        DELETE OBJECT hPropertiesTT NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE processCompile:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cWorkspace          AS CHARACTER                                    NO-UNDO.
    DEFINE VARIABLE cError              AS CHARACTER                                    NO-UNDO.
    DEFINE VARIABLE i                   AS INTEGER                                      NO-UNDO.
    DEFINE VARIABLE cCompileLogsDir     AS CHARACTER                                    NO-UNDO.
    DEFINE VARIABLE oDirectoryHelper    AS appSolutions.appSolute.Utils.DirectoryHelper NO-UNDO.
    DEFINE VARIABLE oFileHelper         AS appSolutions.appSolute.Utils.FileHelper      NO-UNDO.
    DEFINE VARIABLE cTempCompileLogFile AS CHARACTER                                    NO-UNDO.
    DEFINE VARIABLE cCompileLogFile     AS CHARACTER                                    NO-UNDO.
    DEFINE VARIABLE lCompileErrors      AS LOGICAL                                      NO-UNDO INITIAL FALSE.

    IF NUM-ENTRIES(gcWorkspaceList) > 1 THEN
    DO:
        ASSIGN
            gcWorkspaceList = TRIM(gcWorkspaceList)
            gcWorkspaceList = TRIM(gcWorkspaceList, ",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, ",,",",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, "~n","":U) NO-ERROR.

        ASSIGN
            cMessage = SUBSTITUTE("  Processing workspaces : &1...",  gcWorkspaceList).
        oRTBAutomationApi:displayMessage(cMessage, YES, YES).
    END.

    DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, NEXT:
        ASSIGN
            cWorkspace = ENTRY(i, gcWorkspaceList)
            cWorkspace = TRIM(cWorkspace, ",":U).

        IF i > 1 THEN
            ASSIGN
                cMessage = "  /*****************************************/".

        ASSIGN
            cMessage = cMessage + SUBSTITUTE("~n  Processing workspace: &1...",  cWorkspace) + "~n  /*****************************************/".
        oRTBAutomationApi:displayMessage(cMessage, YES, YES).

        IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN
        DO ON ERROR UNDO:
            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).

            ASSIGN
                cMessage = "   Compile settings " + "~n":U +
                    "   =================" + "~n":U +
                    SUBSTITUTE("   Workspace       : &1~n", cWorkspace) +
                    SUBSTITUTE("   Task number     : &1~n", STRING(giTaskNum, "999999999")) +
                    SUBSTITUTE("   Module          : &1~n", gcModule) +
                    SUBSTITUTE("   Group           : &1~n", gcGroup) +
                    SUBSTITUTE("   Object type     : &1~n", gcObjectType) +
                    SUBSTITUTE("   Object(s)       : &1~n", gcObject) +
                    SUBSTITUTE("   Force compile   : &1~n", glForceCompile) +
                    SUBSTITUTE("   Listings        : &1~n", glCompileListing) +
                    SUBSTITUTE("   Xref            : &1~n", glXref) +
                    SUBSTITUTE("   Ignore warnings : &1~n", glIgnoreWarnings) +
                    SUBSTITUTE("   Stop on error   : &1~n", glFailOnError).

            oRTBAutomationApi:displayMessage(cMessage, YES, YES).

            /* Make sure we have the compile logs directory we need in place. */
            IF oRTBAutomationApi:RtbReportBaseDir = "":U THEN
                ASSIGN
                    oRTBAutomationApi:RtbReportBaseDir = ".":U.

            IF NOT oRTBAutomationApi:RtbReportBaseDir MATCHES "*/Compiles*":U THEN
                ASSIGN
                    cCompileLogsDir = SUBSTITUTE("&1/&2/&3/":U, oRTBAutomationApi:RtbReportBaseDir, "Compiles":U, cWorkspace).
            ELSE
                ASSIGN
                    cCompileLogsDir = oRTBAutomationApi:RtbReportBaseDir + "/":U.

            /* Make sure the dir for the report exists */
            oDirectoryHelper = NEW appSolutions.appSolute.Utils.DirectoryHelper().
            IF NOT oDirectoryHelper:DirectoryExists(cCompileLogsDir) THEN
                oDirectoryHelper:makeDirectory(cCompileLogsDir).

            ASSIGN
                cCompileLogFile = oRTBAutomationApi:buildOutputFilename (cWorkspace, "CompileReport":U).

            oRTBAutomationApi:selectiveCompile(
                INPUT  cWorkspace,
                INPUT  giTaskNum,
                INPUT  gcModule,
                INPUT  gcGroup,
                INPUT  gcObjectType,
                INPUT  gcObject,
                INPUT  glForceCompile,
                INPUT  glCompileListing,
                INPUT  glXref,
                INPUT  glIgnoreWarnings,
                INPUT  glFailOnError,
                INPUT  glTaskRcode,
                OUTPUT cError,
                OUTPUT cTempCompileLogFile).

            IF cError NE "":U AND NOT cError BEGINS "Compile Results" THEN
            DO:
                ASSIGN
                    lCompileErrors = TRUE.
                oRTBAutomationApi:displayMessage(cError, YES, YES).
            END.
            ELSE
            DO:
                ASSIGN
                    cMessage = "   Selective compile complete.".
                oRTBAutomationApi:displayMessage(cMessage, YES, YES).
            END.

            /* Work with the compile log file */
            IF cTempCompileLogFile NE "":U THEN
            DO:
                oFileHelper = NEW appSolutions.appSolute.Utils.FileHelper().
                IF oFileHelper:FileExists(cTempCompileLogFile) THEN
                DO:
                    ASSIGN
                        cCompileLogFile = SUBSTITUTE("&1/&2", cCompileLogsDir, cCompileLogFile).

                    OS-COPY VALUE(cTempCompileLogFile) VALUE(cCompileLogFile).
                    FILE-INFORMATION:FILE-NAME = cCompileLogFile.
                    IF FILE-INFORMATION:FULL-PATHNAME NE "":U AND
                        FILE-INFORMATION:FULL-PATHNAME NE ? THEN
                    DO:
                        ASSIGN
                            cMessage = "   Compile log copied to : " + FILE-INFORMATION:FULL-PATHNAME.
                        oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).

                        ASSIGN
                            pcCompileLogFile = pcCompileLogFile + ",":U + REPLACE(cCompileLogFile, "~\":U, "/":U)
                            pcCompileLogFile = TRIM(pcCompileLogFile, ",":U)
                            pcCompileLogFile = REPLACE(pcCompileLogFile, "~\", "/":U).

                        OS-DELETE VALUE(cTempCompileLogFile).
                    END.
                END.
            END.

            CATCH err AS Progress.Lang.Error :
                ASSIGN
                    cMessage = SUBSTITUTE(" *** Compile processing error for workspace '&1': &2", cWorkspace, err:GetMessage(1)).
                oRTBAutomationApi:displayMessage(cMessage, YES, YES).

                ASSIGN
                    glProcessFailed = TRUE.
            END CATCH.
        END.
        ELSE
        DO:
            ASSIGN
                cMessage = SUBSTITUTE("***   Invalid workspace '&1'. Compile cancelled for this workspace!", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).
        END.
    END.

    IF glProcessFailed THEN
        UNDO, THROW NEW Progress.Lang.AppError("Workspace compile failures occured. Overall processing failed.", 0).

    IF lCompileErrors THEN
        UNDO, THROW NEW Progress.Lang.AppError("Workspace compile errors occured. Overall processing failed.", 0).

    RETURN.

    CATCH Err AS Progress.Lang.Error :
        IF oRTBAutomationApi:DebugOn THEN
            oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
        UNDO, THROW Err.
    END CATCH.

    FINALLY:
        IF VALID-OBJECT (oDirectoryHelper) THEN
            DELETE OBJECT oDirectoryHelper.

        IF VALID-OBJECT (oFileHelper) THEN
            DELETE OBJECT oFileHelper.
    END FINALLY.

END PROCEDURE.

PROCEDURE readParams:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    IF ghBuf:AVAILABLE THEN
    DO:
        ASSIGN
            glDebug                  = ghBuf::BuildDebug
            gcRTBInstallPath         = ghBuf::RtbInstallPath
            gcUser                   = ghBuf::RtbUser
            gcPassword               = ghBuf::RtbPassword
            gcWorkspaceList          = ghBuf::RtbCompileWspaceId
            giTaskNum                = ghBuf::RtbCompileTask
            gcModule                 = ghBuf::RtbCompileModule
            gcGroup                  = ghBuf::RtbCompileGroup
            gcObjectType             = ghBuf::RtbCompileObjectType
            gcObject                 = ghBuf::RtbCompileObject
            glForceCompile           = ghBuf::RtbCompileForceCompile
            glCompileListing         = ghBuf::RtbCompileListing
            glXref                   = ghBuf::RtbCompileWithXref
            glFailOnError            = ghBuf::RtbCompileFailOnError
            glIgnoreWarnings         = ghBuf::RtbCompileIgnoreWarnings
            glTaskRcode              = ghBuf::RtbCompileTaskRcode
            gcBuildCompLogFile       = ghBuf::RtbCompileLogFile
            gcAppServerConnectString = ghBuf::RtbAppServerConnection
            glUseAppServer           = ghBuf::RtbUseAppServer
            gcReportBaseDir          = ghBuf::RtbReportBaseDir.
    END.

    RETURN.

END PROCEDURE.

PROCEDURE shutdown:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        DELETE OBJECT oRTBAutomationApi.

    RETURN.

END PROCEDURE.


