/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : adrtb_template.p
    Purpose     : Template Procedure using RTB API and ADRTB dAPI

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_template.p -param rtbInstallPath,workspaceId,UserId,Password

    Description : Template procedure to RTB API Automation wrapper procedure

    Created     : Fri Nov 2 2012
    Notes       :
                11/2013 Thomas Hansen
                Restructured code to use ABL structured error handling better.

                07/2014 Thomas Hansen
                Template updated with new structure using JSON input from Ant XML files.
  ----------------------------------------------------------------------*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

/* ***************************  Definitions  ************************** */

/* Standard variable definitions for RTB API usage */
{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

/* Specific variables for this process */

{rtb/i/rtb_super.i}

/* ********************  Preprocessor Definitions  ******************** */

/* ***************************  Main Block  *************************** */
ASSIGN
    giParamsExpected = 7
    gcLogFile        = "logs/adrtb_template.log".

RUN initialize.

RUN processSomething.

RETURN "0". /* Success */

CATCH Err AS Progress.Lang.Error :
    ASSIGN
        cErrors = " ***":U + Err:GetMessage(1).

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    RUN shutdown.

END CATCH.

FINALLY:
    RUN shutdown.
END FINALLY.
/* **********************  Internal Procedures  *********************** */

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

    RETURN.

    CATCH LangError AS Progress.Lang.Error:
        UNDO, THROW LangError.
    END CATCH.

END PROCEDURE.

PROCEDURE processSomething:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cWorkspace     AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cError         AS CHARACTER NO-UNDO.
    DEFINE VARIABLE i              AS INTEGER   NO-UNDO.
    DEFINE VARIABLE cReleaseReport AS CHARACTER NO-UNDO.

    DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, NEXT:
        ASSIGN
            cWorkspace = ENTRY(i, gcWorkspaceList).

        IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN
        DO ON ERROR UNDO:
            ASSIGN
                cMessage = SUBSTITUTE("/***/~n   Creating release in workspace: &1", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).

            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).


            /* The guts of the per workspace processing code goes in here... */

            IF oRTBAutomationApi:RtbReportBaseDir = "":U OR
                oRTBAutomationApi:RtbReportBaseDir = ? THEN
            DO:
                ASSIGN
                    cMessage = "~n*** Release report root directory not set. Cannot create release report!":U.
                oRTBAutomationApi:displayMessage(cMessage, YES, YES).
                RETURN.
            END.
        END.
        ELSE
        DO:
            ASSIGN
                cMessage = SUBSTITUTE("***   Invalid workspace '&1'. Processing cancelled for this workspace!", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).
        END.

        CATCH err AS Progress.Lang.Error :
            oRTBAutomationApi:displayMessage(err:GetMessage(1), YES, YES).
        END CATCH.
    END.

    RETURN.

    CATCH LangErr AS Progress.Lang.Error :
        oRTBAutomationApi:displayMessage(LangErr:GetMessage(1), YES, YES).

        UNDO, THROW LangErr.
    END CATCH.

END PROCEDURE.

PROCEDURE readParams:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/



    IF ghBuf:AVAILABLE THEN
    DO:
        ASSIGN
            glDebug                  = ghBuf::BuildDebug
            gcRTBInstallPath         = ghBuf::RtbInstallPath
            gcUser                   = ghBuf::RtbUser
            gcPassword               = ghBuf::RtbPassword
            gcWorkspaceList          = ghBuf::RtbCompileWspaceId
            /* Add setting of additonal paremeters here */
            gcAppServerConnectString = ghBuf::RtbAppServerConnection
            glUseAppServer           = ghBuf::RtbUseAppServer
            gcReportBaseDir          = ghBuf::RtbReportBaseDir
        NO-ERROR.
    END.

    RETURN.

END PROCEDURE.

PROCEDURE shutdown:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    oRTBAutomationApi:stopAPIs().

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        DELETE OBJECT oRTBAutomationApi.

    RETURN.

END PROCEDURE.

