/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
   File        : appSolutions.appSolute.RTB.Base.RTBAutomationApi - appSolute RTB Automation Toolkit API - Base Class
   Purpose     : Provide a class wrapper for the standard API and proxy procedures in RTB that wil allow for external access to RTB functionality.
   Syntax      :
   Description :
   Notes       :
 ----------------------------------------------------------------------*/

USING Progress.Lang.*.
USING appSolutions.appSolute.RTB.*   FROM PROPATH.
USING appSolutions.appSolute.Utils.* FROM PROPATH.
USING scm.custom.rtb.utils.*         FROM PROPATH.
USING scm.custom.rtb.utils.RTBProxyHelper FROM PROPATH.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS appSolutions.appSolute.RTB.Base.RTBAutomationApi:

    DEFINE STREAM sLogStream.
    DEFINE STREAM sOutStream.

    DEFINE PRIVATE TEMP-TABLE ttFileSpec
        FIELD SourceDir AS CHARACTER
        FIELD fileName  AS CHARACTER.

    DEFINE PUBLIC PROPERTY CompleteRtbTaskAfterImport AS LOGICAL INITIAL FALSE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY ProcessImportTableTasks AS LOGICAL INITIAL FALSE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY CreateRtbTaskForImport AS LOGICAL INITIAL FALSE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RtbUseAppServer AS LOGICAL INITIAL FALSE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY TempTableHelper AS TempTableHelper NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RtbAppServerConnected AS LOGICAL INITIAL FALSE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY AppServerConnectString AS CHARACTER NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY AppServerHandle AS HANDLE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY DebugOn AS LOGICAL NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY LogFileName AS CHARACTER NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RTBAPI AS HANDLE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RtbCompileAfterImport AS LOGICAL INITIAL FALSE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RtbCompilerHandle AS HANDLE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RTBInstallPath AS CHARACTER NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RtbLoadWriteHandle AS HANDLE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RTBProxyHelper AS RTBProxyHelper NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RtbReleaseReport AS CHARACTER NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RtbReportBaseDir AS CHARACTER NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RTBSessionID AS INTEGER NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RTBUser AS CHARACTER NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RTBUserPassword AS CHARACTER NO-UNDO
        GET.
        SET.

    DEFINE PROTECTED PROPERTY hImportTaskData AS HANDLE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY RTBVersion AS CHARACTER NO-UNDO
        GET():
            DEFINE VARIABLE cRtbVerFile AS CHARACTER NO-UNDO.
            DEFINE VARIABLE cRTBVersion AS CHARACTER NO-UNDO.

            IF THIS-OBJECT:RTBVersion = "":U THEN
            DO:
                /* Check the version of RTB that we are using. This is needed due to difference in some
                  of the routines to start SDOs and super procedures.
                */
                ASSIGN
                    cRtbVerFile = SEARCH("rtbver").

                IF cRtbVerFile NE ? THEN
                DO:
                    /* Read and parse the rtbver file. This is the actual file for the RTB version we are
                       using and not the one we use at compilation time - as they may be different */
                    INPUT FROM VALUE(cRtbVerFile).
                    IMPORT UNFORMATTED cRtbVersion.
                    INPUT CLOSE.

                    IF cRtbVersion NE "":U THEN
                        ASSIGN
                            THIS-OBJECT:RTBVersion = TRIM(REPLACE(cRTBVersion, "Roundtable TSMS release ":U, "":U)).
                END.
            END.

            RETURN THIS-OBJECT:RTBVersion.
        END GET.

        SET.

    DEFINE PUBLIC PROPERTY RtbWorkspaceId AS CHARACTER NO-UNDO
        GET.
        SET.

    CONSTRUCTOR PUBLIC RTBAutomationApi (  ):

    END CONSTRUCTOR.

    /*------------------------------------------------------------------------
        File        : applySchemaUpdateList
        Description : Proxy method to update schema from schema update
                      list.

        Parameters  : Input  pcWorkspace - Configuration ID.
                      Output pcError     - Non-blank on error.
        Created     :
        Notes       :
      ----------------------------------------------------------------------*/
    METHOD PUBLIC VOID applySchemaUpdateList(
        INPUT  pcWorkspace           AS CHARACTER,
        OUTPUT pcSchemaUpdateLogFile AS CHARACTER,
        OUTPUT pcError               AS CHARACTER):

        DEFINE VARIABLE hDS               AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cWorkspaceRoot    AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lcSchemaUpdateLog AS LONGCHAR  NO-UNDO.
        DEFINE VARIABLE cSchemaLogsDir    AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cMessage          AS CHARACTER NO-UNDO.

        hDS = RTBProxyHelper:rtbApplySchemaUpdateList (
            INPUT  pcWorkspace,
            INPUT  TRUE /* plDelta */).

        ASSIGN
            pcSchemaUpdateLogFile = buildOutputFilename (pcWorkspace, "SchemaUpdateLog":U).

        checkLogsDir(
            INPUT  "Schema",
            INPUT  RtbWorkspaceId,
            INPUT  TRUE,
            OUTPUT cSchemaLogsDir ).

        pcSchemaUpdateLogFile = SUBSTITUTE("&1/&2":U, cSchemaLogsDir, pcSchemaUpdateLogFile).

        lcSchemaUpdateLog = RTBProxyHelper:custRtbGetSchemaUpdateLog(pcWorkspace).

        IF lcSchemaUpdateLog NE ? AND
            lcSchemaUpdateLog NE "":U THEN
        DO:
            /* Unless an error has been thrown in the checkLogsDir above, then write the contents of the file to disk */
            COPY-LOB FROM lcSchemaUpdateLog TO FILE pcSchemaUpdateLogFile.

            ASSIGN
                cMessage = SUBSTITUTE("   Schema Update log written to : &1", pcSchemaUpdateLogFile).
            DisplayMessage(cMessage, TRUE, TRUE).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hDS NO-ERROR.
        END FINALLY.


    END METHOD.

    /*------------------------------------------------------------------------------
               Purpose: Perform an import in the given workspace
               Notes:
       ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID applyWorkspaceImportList(
        INPUT  pcWorkspace        AS CHARACTER,
        INPUT  piTaskNum          AS INTEGER,
        OUTPUT plcErrorLog        AS LONGCHAR):

        DEFINE VARIABLE hTTImportLog           AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE i                      AS INTEGER                                 NO-UNDO.
        DEFINE VARIABLE cImportErrorLog        AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE cError                 AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE lWorkspaceTaskRequired AS LOGICAL                                 NO-UNDO.
        DEFINE VARIABLE lcJsonData             AS LONGCHAR                                NO-UNDO.
        DEFINE VARIABLE oFileHelper            AS appSolutions.appSolute.Utils.FileHelper NO-UNDO.

        IF validWorkspace(pcWorkspace) THEN
        DO:
            oFileHelper = NEW appSolutions.appSolute.Utils.FileHelper().
            lWorkspaceTaskRequired = rtbCheckWorkspaceTaskRequired(pcWorkspace).

            IF lWorkspaceTaskRequired = TRUE AND
                piTaskNum = 0 THEN
            DO ON ERROR UNDO :
                piTaskNum = RTBProxyHelper:custRtbCreateTask(
                    INPUT  pcWorkspace,
                    INPUT  SUBSTITUTE("Automatically created task for import of task : &1", piTaskNum),
                    INPUT  "":U, /* pcTaskDescription */
                    INPUT  "":U, /*pcTaskUserTaskref */
                    INPUT  RTBUser, /*pcProgrammer */
                    INPUT  RTBUser). /*pcManager */

                IF piTaskNum NE 0 THEN
                DO:
                    /* set the 'current' task in RTB on the AppServer if we are running with a remote AppServer */
                    RTBProxyHelper:rtbSetSessionProperty(
                        INPUT "task":U,
                        INPUT STRING(piTaskNum)).
                END.

                CATCH AppErr AS Progress.Lang.AppError :
                    UNDO, THROW AppErr.
                END CATCH.
            END.

            lcJsonData = RTBProxyHelper:rtbApplyWorkspaceImportList(pcWorkspace).

            IF lcJsonData NE "":U THEN
                hTTImportLog = TempTableHelper:ConvertJsonToTtHandle(lcJsonData).

            IF VALID-HANDLE(hTTImportLog) THEN
            DO:
                ASSIGN
                    cImportErrorLog = SUBSTITUTE("&1/&2_Import_Error.log", SESSION:TEMP-DIR, pcWorkspace).
                IF oFileHelper:FileExists(cImportErrorLog) THEN
                    OS-DELETE value(cImportErrorLog).

                /* Start the rtb file load write super to handle files in temp-table */
                IF NOT VALID-HANDLE(RtbLoadWriteHandle) AND
                    oFileHelper:FileExists("rtb/p/rtb_fileloadwrite.p") THEN
                    RUN rtb/p/rtb_fileloadwrite.p PERSISTENT SET RtbLoadWriteHandle NO-ERROR.

                IF VALID-HANDLE(RtbLoadWriteHandle) THEN
                DO ON ERROR UNDO:
                    /* If we have  */
                    RUN write_file_from_tt IN RtbLoadWriteHandle(
                        INPUT TABLE-HANDLE hTTImportLog,
                        INPUT  cImportErrorLog,
                        INPUT  FALSE, /* Binary */
                        INPUT  TRUE, /* Overwrite */
                        OUTPUT cError).

                    IF cError = "":U AND
                        oFileHelper:FileExists(cImportErrorLog) THEN
                    DO:
                        COPY-LOB FROM FILE cImportErrorLog TO plcErrorLog.
                        IF plcErrorLog NE ? AND
                            plcErrorLog NE "":U THEN
                            OS-DELETE value(cImportErrorLog).
                    END.
                    ELSE
                        UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("*** Import log file error: &1", cError), 0).
                END.
                ELSE
                    UNDO, THROW NEW Progress.Lang.AppError("RTB procedure 'rtb/p/rtb_fileloadwrite.p' not running. Cannot process import error log!", 0).
            END.
        END.
        ELSE
            UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("*** Workspace '&1' not valid! Cannot apply workspace import list!", pcWorkspace ), 0).

        RETURN.

        CATCH Err AS Progress.Lang.Error :
            DisplayMessage("ERROR : " + Err:GetMessage(1), TRUE, TRUE).
        END CATCH.

        FINALLY:
            DELETE OBJECT hTTImportLog NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
                   Purpose: Perform an import in the given workspace, and optionally compile the objects in the import list
                   Notes:
           ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID applyWorkspaceImportList(
        INPUT  pcWorkspace             AS CHARACTER,
        INPUT  piTaskNum               AS INTEGER,
        INPUT  plCompileAfterImport    AS LOGICAL,
        OUTPUT plcImportCompileResults AS LONGCHAR,
        OUTPUT plcErrorLog             AS LONGCHAR):

        applyWorkspaceImportList(
            INPUT  pcWorkspace,
            INPUT  piTaskNum,
            OUTPUT plcErrorLog).

        IF plcErrorLog = "":U AND
            plCompileAfterImport THEN
        DO:
            compileWorkspaceImportsList(
                INPUT  pcWorkspace,
                OUTPUT plcImportCompileResults).
        END.

        RETURN.

        CATCH Err AS Progress.Lang.Error :
            DisplayMessage("Import error : " + Err:GetMessage(1), TRUE, TRUE).
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
               Purpose: Perform an import in the given workspace, and optionally compile the objects in the import list
               Notes:
       ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID applyWorkspaceImportList(
        INPUT  pcWorkspace             AS CHARACTER,
        INPUT  plCompileAfterImport    AS LOGICAL,
        OUTPUT plcImportCompileResults AS LONGCHAR,
        OUTPUT plcError                AS LONGCHAR):

        applyWorkspaceImportList(
            INPUT  pcWorkspace,
            INPUT  0,
            OUTPUT plcError).

        IF plcError = "":U AND
            plCompileAfterImport THEN
        DO:
            compileWorkspaceImportsList(
                INPUT  pcWorkspace,
                OUTPUT plcImportCompileResults).
        END.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID buildDeploymentSchemaUpdateList(
        INPUT  pcWorkspace  AS CHARACTER,
        INPUT  pcSiteCode   AS CHARACTER,
        INPUT  piSequence   AS INTEGER,
        OUTPUT pcError      AS CHARACTER):

        DEFINE VARIABLE cMessage   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hDS        AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonData AS LONGCHAR  NO-UNDO.

        IF piSequence = 0 THEN
        DO:
            ASSIGN
                pcError = "*** Cannot build deployment schema update list for deployment sequence number 0. Cannot continue.".
            RETURN.
        END.

        ASSIGN
            cMessage = SUBSTITUTE("    Building deployment schema update list for deployment site '&1', deployment number: &2...":U, pcSiteCode, piSequence).
        DisplayMessage(cMessage, YES, YES).

        hDS = RTBProxyHelper:rtbBuildDeploymentSchemaUpdateList (
            INPUT  pcWorkspace,
            INPUT  pcSiteCode ,
            INPUT  piSequence).

        ASSIGN
            cMessage = "    Deployment schema update list build complete.".
        DisplayMessage(cMessage, YES, YES).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hDS NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER buildOutputFilename(
        INPUT pcReportParamString1      AS CHARACTER,
        INPUT pcReportParamString2 AS CHARACTER):

        DEFINE VARIABLE cOutputFilename  AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cWorkspaceString AS CHARACTER NO-UNDO.

        /* Create a string from the workspace without "-", "_" and spaces*/
        ASSIGN
            cWorkspaceString = REPLACE(pcReportParamString1, "-":U, "":U)
            cWorkspaceString = REPLACE(cWorkspaceString, "_":U, "":U)
            cWorkspaceString = REPLACE(cWorkspaceString, " ":U, "":U).

        ASSIGN
            cOutputFilename = SUBSTITUTE("&1_&2_&3&4&5_&6.txt", pcReportParamString1, pcReportParamString2, STRING(YEAR(TODAY), "9999":U), STRING(MONTH(TODAY), "99":U), STRING(DAY(TODAY), "99":U), STRING(TIME, "HH:MM:SS"))
            cOutputFilename = REPLACE(cOutputFilename, ":":U, "":U)
            cOutputFilename = REPLACE(cOutputFilename, "/":U, "":U).

        RETURN cOutputFilename.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID buildWorkspaceImportList(
        INPUT  pcWorkspaces        AS CHARACTER,
        INPUT  pcSourceWorkspaces  AS CHARACTER,
        INPUT  pcPmods             AS CHARACTER,
        INPUT  pcTasks             AS CHARACTER,
        INPUT  piReleaseNum        AS INTEGER,
        INPUT  plHigherVerOnly     AS LOGICAL,
        INPUT  plCurrReleaseOnly   AS LOGICAL,
        INPUT  plCompleteTasksOnly AS LOGICAL,
        OUTPUT pcError             AS CHARACTER ):

        DEFINE VARIABLE i            AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cWorkspaceId AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cError       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cMessage     AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lcJsonData   AS LONGCHAR  NO-UNDO.

        IF pcWorkspaces NE "":U THEN
        DO:
            DO i = 1 TO NUM-ENTRIES(pcWorkspaces):
                ASSIGN
                    cWorkspaceId = ENTRY(i, pcWorkspaces).

                IF RTBProxyHelper:rtbWorkspaceExists(cWorkspaceId) = TRUE  THEN
                DO:
                    IF pcTasks NE "":U AND
                        pcTasks NE "0":U THEN
                    DO:
                        pcTasks = validateCompletedTasks(
                            INPUT  pcTasks,
                            OUTPUT cError).

                        IF cError NE "":U THEN
                        DO:
                            ASSIGN
                                pcError = SUBSTITUTE("*** Task validation failed!: &1", cError).
                            RETURN.
                        END.
                        ELSE
                        DO:
                            ASSIGN
                                cMessage = "Task list validated.".
                            DisplayMessage(cMessage, YES, YES).
                        END.

                        IF pcTasks = "":U THEN
                        DO:
                            ASSIGN
                                pcError = "No tasks to process. Import processing cancelled.".
                            RETURN.
                        END.
                    END.

                    /* Run the RTB Proxy procedure to build the import table  */
                    lcJsonData = RTBProxyHelper:rtbBuildWorkspaceImportList(
                        INPUT  cWorkspaceId,
                        INPUT  pcSourceWorkspaces,
                        INPUT  pcPmods,
                        INPUT  pcTasks,
                        INPUT  piReleaseNum,
                        INPUT  plHigherVerOnly,
                        INPUT  plCurrReleaseOnly,
                        INPUT  plCompleteTasksOnly).
                END.
                ELSE
                DO:
                    ASSIGN
                        pcError = SUBSTITUTE("*** Workspace '&1' is not valid. Import processing cannot continue!", cWorkspaceId).

                    RETURN.
                END.
            END.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error:
            ASSIGN
                pcError = pcError + (IF pcError NE "":U THEN "~n":U ELSE "":U) + err:GetMessage(1).
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID buildWorkspaceSchemaUpdateList(
        INPUT  pcWorkspace              AS CHARACTER,
        INPUT  pcSchemaList             AS CHARACTER,
        OUTPUT plSchemaUpdateListExists AS LOGICAL ,
        OUTPUT pcError                  AS CHARACTER):

        DEFINE VARIABLE cMessage      AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cError        AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hDS           AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonData    AS LONGCHAR  NO-UNDO.
        DEFINE VARIABLE iNumDbUpdates AS INTEGER   NO-UNDO.

        IF pcSchemaList = "All":U THEN
            ASSIGN
                pcSchemaList = "*":U.

        ASSIGN
            cMessage = SUBSTITUTE("   Building schema update list for workspace '&1'...", pcWorkspace).
        DisplayMessage(cMessage, YES, YES).

        hDS = RTBProxyHelper:rtbBuildWorkspaceSchemaUpdateList(
            INPUT  pcWorkspace,
            INPUT  "*" /*"pcSchemaList, */).

        /* First, Check if we have an existing schema update table */
        plSchemaUpdateListExists = checkRtbSchemaUpdateTableExists(pcWorkspace, "":U, OUTPUT iNumDbUpdates).

        ASSIGN
            cMessage = "   Building schema update complete!" + SUBSTITUTE(" Schema updates to process : &1", iNumDbUpdates).
        DisplayMessage(cMessage, YES, YES).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            ASSIGN
                cError = err:GetMessage(1).
            IF cError <> "" AND (ENTRY(1, cError, CHR(1)) <> "WARNING" AND ENTRY(1, cError, CHR(1)) <> "INFO") THEN
                ASSIGN
                    pcError = cError.
        END CATCH.

        FINALLY:
            IF VALID-HANDLE(hDS) THEN
                DELETE OBJECT hDS.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER calculateDeploymentDirectory(
        INPUT pcWorkspace  AS CHARACTER,
        INPUT pcSiteCode   AS CHARACTER,
        INPUT pcDir        AS CHARACTER ,
        INPUT piSequence   AS INTEGER,
        INPUT piReleaseNum AS INTEGER ):

        IF pcDir NE "":U AND
            pcDir MATCHES "*<release-num>"THEN
            ASSIGN
                pcDir = REPLACE(pcDir, "<release-num>", STRING(piReleaseNum, "9999":U)).

        IF pcDir NE "":U AND
            pcDir MATCHES "*<deploy-sequence>"THEN
            ASSIGN
                pcDir = REPLACE(pcDir, "<deploy-sequence>", STRING(piSequence, "9999":U)).

        IF pcDir NE "":U AND
            pcDir MATCHES "*<workspace-id>*"THEN
            ASSIGN
                pcDir = REPLACE(pcDir, "<workspace-id>", pcWorkspace).

        IF pcDir NE "":U AND
            pcDir MATCHES "*<site-code>*"THEN
            ASSIGN
                pcDir = REPLACE(pcDir, "<site-code>", pcSiteCode).

        RETURN pcDir.

    END METHOD.

    /*------------------------------------------------------------------------------
          Purpose: Check for the existence of a rtb_import table for a given workspace
                   and find out the number of records in the table
          Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL checkImportTableExists(
        INPUT  pcWorkspace  AS CHARACTER,
        OUTPUT piNumRecords AS INTEGER):

        DEFINE VARIABLE cImportAnalysisReport AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cMessage              AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hWorkspaceImportsData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf                  AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hImportBuf            AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry                  AS HANDLE    NO-UNDO.

        hWorkspaceImportsData = RTBProxyHelper:rtbGetWorkspaceImportsHandle(pcWorkspace).

        IF VALID-HANDLE(hWorkspaceImportsData) THEN
        DO:
            hImportBuf = hWorkspaceImportsData:DEFAULT-BUFFER-HANDLE.
            CREATE QUERY hQry.
            hQry:SET-BUFFERS(hImportBuf).
            hQry:QUERY-PREPARE("FOR EACH " + hWorkspaceImportsData:NAME).
            hQry:QUERY-OPEN().
            hQry:GET-FIRST().

            IF DebugOn THEN
            DO:
                ASSIGN
                    cMessage = "   ** [Debug] Debug listing of objects in import table:".
                DisplayMessage(cMessage, YES, YES).
            END.

            REPEAT WHILE NOT hQry:QUERY-OFF-END:
                IF DebugOn THEN
                    DisplayMessage(SUBSTITUTE("Object : &1 (&2)", hImportBuf::Object, hImportBuf::imp-status), YES, YES).

                ASSIGN
                    piNumRecords = piNumRecords + 1.
                hQry:GET-NEXT ().
            END.

            IF piNumRecords > 0 THEN
            DO:
                IF DebugOn THEN
                    DisplayMessage(SUBSTITUTE("   ** [Debug] Import table exists (&1)", piNumRecords), ?, YES, DebugOn).
                RETURN TRUE.
            END.
            ELSE
                RETURN FALSE.
        END.
        ELSE
        DO:
            IF DebugOn THEN
                DisplayMessage("   ** [Debug] No import table exists", ?, YES, DebugOn).
            RETURN FALSE.
        END.

        FINALLY:
            DELETE OBJECT hWorkspaceImportsData NO-ERROR.

            DELETE OBJECT hQry NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL checkImportTableExists(
        INPUT  pcWorkspace AS CHARACTER):
        /*------------------------------------------------------------------------------
              Purpose:
              Notes:
        ------------------------------------------------------------------------------*/
        DEFINE VARIABLE iNumRecords AS INTEGER NO-UNDO.

        RETURN CheckImportTableExists(
            INPUT pcWorkspace,
            OUTPUT iNumRecords).

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Check for the existence of a reports or log directory for a given workspace as a sub-directory
              to the API RtbPeportBaseDir or the current directory
     Notes:
         @param - pcLogDirname the name of the sub-directory to
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID checkLogsDir(
        INPUT  pcLogDirname      AS CHARACTER,
        INPUT  pcWorkspace       AS CHARACTER,
        INPUT  plCreateIfMissing AS LOGICAL,
        OUTPUT pcLogsDir         AS CHARACTER ):

        DEFINE VARIABLE oDirectoryHelper AS appSolutions.appSolute.Utils.DirectoryHelper NO-UNDO.

        /* Make sure we have the logs directory we need in place. */
        IF RtbReportBaseDir = "":U THEN
            ASSIGN
                RtbReportBaseDir = ".":U.

        IF NOT RtbReportBaseDir MATCHES SUBSTITUTE("*/&1*":U, pcLogDirname) THEN
            ASSIGN
                pcLogsDir = SUBSTITUTE("&1/&2/&3/":U, RtbReportBaseDir, pcLogDirname, pcWorkspace).
        ELSE
            ASSIGN
                pcLogsDir = RtbReportBaseDir.

        /* Make sure the dir for the report exists */
        /* Create if not found... */
        oDirectoryHelper = NEW appSolutions.appSolute.Utils.DirectoryHelper().
        IF NOT oDirectoryHelper:DirectoryExists(pcLogsDir) THEN
            oDirectoryHelper:makeDirectory(pcLogsDir).

        RETURN.

        CATCH Err AS Progress.Lang.Error :
            UNDO, THROW Err.
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL checkRtbSchemaUpdateTableExists(
        INPUT  pcWorkspace  AS CHARACTER,
        INPUT  pcSiteCode   AS CHARACTER):

        DEFINE VARIABLE iNumRecords AS INTEGER NO-UNDO.

        RETURN checkRtbSchemaUpdateTableExists(
            INPUT  pcWorkspace,
            INPUT  pcSiteCode,
            OUTPUT iNumRecords).

    END METHOD.

    /*------------------------------------------------------------------------------
                Purpose:
                Notes:
        ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL checkRtbSchemaUpdateTableExists(
        INPUT  pcWorkspace  AS CHARACTER,
        INPUT  pcSiteCode   AS CHARACTER,
        OUTPUT piNumRecords AS INTEGER):

        DEFINE VARIABLE dsSchupd         AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cError           AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lcJsonData       AS LONGCHAR  NO-UNDO.

        DEFINE VARIABLE hBuf             AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hSchemaUpdateBuf AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hDbHandle        AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry             AS HANDLE    NO-UNDO.

        dsSchupd = RTBProxyHelper:rtbGetSchemaUpdateList (
            INPUT pcWorkspace,
            INPUT pcSiteCode).

        /* If there is at least one Db in the dataset then set the output handle */
        IF VALID-HANDLE(dsSchupd) THEN
        DO:
            ASSIGN
                hDbHandle = dsSchupd:GET-BUFFER-HANDLE("Db"):TABLE-HANDLE.

            IF VALID-HANDLE(hDbHandle) AND hDbHandle:HAS-RECORDS THEN
            DO:
                IF DebugOn THEN
                    DisplayMessage("   ** [Debug] Schema update table exists", YES, DebugOn).

                hSchemaUpdateBuf = hDbHandle:DEFAULT-BUFFER-HANDLE.
                CREATE QUERY hQry.
                hQry:SET-BUFFERS(hSchemaUpdateBuf).
                hQry:QUERY-PREPARE("FOR EACH " + hSchemaUpdateBuf:NAME).
                hQry:QUERY-OPEN().

                REPEAT WHILE NOT hQry:QUERY-OFF-END:
                    ASSIGN
                        piNumRecords = piNumRecords + 1.
                    hQry:GET-NEXT ().
                END.

                RETURN TRUE.
            END.
        END.
        ELSE
        DO:
            IF DebugOn THEN
                DisplayMessage("   ** [Debug] No schema update table exists", YES, DebugOn).
            RETURN FALSE.
        END.

        FINALLY:
            DELETE OBJECT dsSchupd NO-ERROR.

            DELETE OBJECT hDbHandle NO-ERROR.

            DELETE OBJECT hQry NO-ERROR.
        END FINALLY.

    END METHOD.
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID checkWorkspaceIntegrity(
        INPUT  pcWorkspaceId            AS CHARACTER,
        INPUT  pcObjectFilter           AS CHARACTER,
        INPUT  plDisplayWIPObjects      AS LOGICAL,
        OUTPUT plcIntegrityCheckResults AS LONGCHAR):

        DEFINE VARIABLE cError                AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cMessage              AS CHARACTER NO-UNDO.
        DEFINE VARIABLE iObjectsChecked       AS INTEGER   NO-UNDO.
        DEFINE VARIABLE iObjectErrors         AS INTEGER   NO-UNDO.
        DEFINE VARIABLE iWIPObjects           AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cFullPathname         AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hWorkspaceObjectsData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry                  AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hWorkspaceObjectBuf   AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hVersionData          AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hVerBuf               AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cIntegrityCheckLogDir AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cRowIdent             AS CHARACTER NO-UNDO.

        IF validWorkspace(pcWorkspaceId) THEN
        DO:
            ASSIGN
                cMessage = "         ~n   Workspace object integrity check started..." + "~n":U +
                "   Changed objects: ~n".
            DisplayMessage(cMessage,YES,  YES).
            plcIntegrityCheckResults = cMessage + "~n":U.

            hWorkspaceObjectsData = RTBProxyHelper:rtbGetWorkspaceObjectsHandle(
                INPUT-OUTPUT cRowIdent,
                INPUT 0,
                INPUT pcWorkspaceId,
                INPUT "PCODE",
                INPUT pcObjectFilter).

            IF VALID-HANDLE(hWorkspaceObjectsData) THEN
            DO:
                CREATE QUERY hQry.
                hWorkspaceObjectBuf = hWorkspaceObjectsData:DEFAULT-BUFFER-HANDLE.
                hQry:SET-BUFFERS(hWorkspaceObjectBuf).
                hQry:QUERY-PREPARE("for each " + hWorkspaceObjectsData:NAME + " BY pmod BY object").
                hQry:QUERY-OPEN.
                hQry:GET-FIRST.

                objectBlock:
                REPEAT WHILE NOT hQry:QUERY-OFF-END:
                    /* If we are not to Check WIP objects, then find the version record and Check the status */
                    IF NOT plDisplayWIPObjects THEN
                    DO:
                        hVersionData = RTBProxyHelper:rtbGetVersionByRowidHandle(hWorkspaceObjectBuf::ObjVersionRowid).

                        IF VALID-HANDLE(hVersionData) THEN
                        DO:
                            hVerBuf = hVersionData:DEFAULT-BUFFER-HANDLE.
                            hVerBuf:FIND-FIRST("").

                            IF hVerBuf::obj-status NE "C":U THEN
                            DO:
                                hQry:GET-NEXT.
                                NEXT objectBlock.
                            END.
                        END.
                    END.

                    ASSIGN
                        cFullPathname   = "":U
                        cError          = "":U
                        iObjectsChecked = iObjectsChecked + 1.

                    RTBProxyHelper:rtbVerifyObjectIntegrity (hWorkspaceObjectBuf::rowIdent).

                    hQry:GET-NEXT.

                    CATCH err AS Progress.Lang.Error :
                        ASSIGN
                            cError = err:GetMessage(1).

                        RTBProxyHelper:rtbGetObjectPath (
                            INPUT hWorkspaceObjectBuf::rowIdent,
                            INPUT 1,
                            OUTPUT cFullPathname).

                        IF cError MATCHES "*object status is WIP*" THEN
                            ASSIGN
                                iWIPObjects = iWIPObjects + 1.
                        ELSE
                            ASSIGN
                                iObjectErrors = iObjectErrors + 1.

                        ASSIGN
                            cError = SUBSTITUTE("   * &1 &2 &3 : &4",
                            cFullPathname + FILL(" ", 50 - length(cFullPathname)),
                            STRING(hWorkspaceObjectBuf::VERSION, "999999":U) + FILL(" ", 10 - length(hWorkspaceObjectBuf::VERSION)),
                            hWorkspaceObjectBuf::pmod + FILL(" ", 35 - length(hWorkspaceObjectBuf::pmod)),
                            cError).

                        DisplayMessage(SUBSTITUTE("&1", cError), YES,  YES).
                        plcIntegrityCheckResults = plcIntegrityCheckResults + cError + "~n":U.
                    END CATCH.
                END.
            END.

            ASSIGN
                cMessage = "          ~n   Workspace object integrity Check completed" + "~n" +
                           SUBSTITUTE("   Objects Checked     : &1", STRING(iObjectsChecked, ">>,>>>,999")) + "~n" +
                           SUBSTITUTE("   WIP objects skipped : &1", STRING(iWIPObjects, ">>,>>>,999")) + "~n" +
                           SUBSTITUTE("   Objects with errors : &1", STRING(iObjectErrors, ">>,>>>,999")).
            DisplayMessage(cMessage,YES,  YES).
            plcIntegrityCheckResults = plcIntegrityCheckResults + cMessage + "~n":U.
        END.

        RETURN.

        FINALLY:
            DELETE OBJECT hWorkspaceObjectsData NO-ERROR.

            DELETE OBJECT hQry NO-ERROR.

            DELETE OBJECT hVersionData NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER cleanupCsvList(
        INPUT pcInputList AS CHARACTER ):

        ASSIGN
            pcInputList = TRIM(pcInputList).

        IF NUM-ENTRIES(pcInputList, " ":U) > 1 THEN
            ASSIGN
                pcInputList = REPLACE(pcInputList, " ":U, ",":U).

        IF NUM-ENTRIES(pcInputList, ";":U) > 1 THEN
            ASSIGN
                pcInputList = REPLACE(pcInputList, ";":U, ",":U).

        IF NUM-ENTRIES(pcInputList, "|":U) > 1 THEN
            ASSIGN
                pcInputList = REPLACE(pcInputList, "|":U, ",":U).

        RETURN pcInputList.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL closeStream(  ):

        OUTPUT STREAM sLogStream CLOSE.

        RETURN TRUE.   /* Function return value. */

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID compileObject(
        INPUT  pcWorkspace  AS CHARACTER,
        INPUT  piTaskNum    AS INTEGER,
        INPUT  pcObjType    AS CHARACTER,
        INPUT  pcObject     AS CHARACTER,
        INPUT  pcPmod       AS CHARACTER,
        INPUT  plXref       AS LOGICAL,
        INPUT  plListing    AS LOGICAL,
        INPUT  plVerbose    AS LOGICAL,
        OUTPUT pcCompileLog AS CHARACTER,
        OUTPUT pcError      AS CHARACTER ):

        DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cError   AS CHARACTER NO-UNDO.

        IF NOT VALID-HANDLE(RtbCompilerHandle) THEN
            startRtbCompiler().

        IF VALID-HANDLE(RtbCompilerHandle) THEN
        DO:
            IF plVerbose THEN
            DO:
                ASSIGN
                    cMessage = SUBSTITUTE("    Compiling : &1 (Product module: '&2' Workspace: '&3'...) ", pcObject, pcPmod, pcWorkspace).
                DisplayMessage(cMessage, YES, YES).
            END.

            RUN rtb_compile_object IN RtbCompilerHandle (
                INPUT pcWorkspace,
                INPUT piTaskNum,
                INPUT pcObjType,
                INPUT pcObject,
                INPUT pcPmod,
                INPUT plXref,
                INPUT plListing,
                INPUT FALSE). /* Lab */

            ASSIGN
                cError = RETURN-VALUE.

        END.
        ELSE
            DisplayMessage(SUBSTITUTE("Compiler handle not set. Cannot perform compile for workspace: &1", pcWorkspace), YES, YES).

        IF cError NE "":U THEN
            IF NUM-ENTRIES(cError,CHR(1)) > 1 THEN
            DO:
                IF ENTRY(1,cError,CHR(1)) EQ "INFO":U THEN
                    pcError = ENTRY(2,cError,CHR(1)).
                ELSE
                DO:
                    ASSIGN
                        pcCompileLog = ENTRY(2,cError,CHR(1))
                        pcError      = SUBSTITUTE("Warnings/Errors encountered. See &1 for details.",pcCompileLog).
                END.
            END.
            ELSE
                IF cError = "Cancel":U THEN
                    pcError = "Object compile cancelled.".

        IF pcError NE "":U THEN
        DO:
            IF NOT pcError BEGINS "No compiles required" THEN
                IF DebugOn THEN
                    DisplayMessage(SUBSTITUTE("   ** [Debug] Errors encountered during selective compile: &1",pcError), YES, DebugOn).
                ELSE
                    IF DebugOn THEN
                        DisplayMessage(pcError, YES, DebugOn).
        END.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Compile the objects that have been imported from the import list of the passed in workspace.
            Notes:   This does an object by object compile of the objects that have been imported in an existing import list.
                     Therefore, it is a pre-requiste that there is an import control table with objects in it.
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID compileWorkspaceImportsList(
        INPUT  pcWorkspace      AS CHARACTER,
        OUTPUT plcCompileResult AS LONGCHAR):

        DEFINE VARIABLE cCompileLog           AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE cError                AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE cMessage              AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE lcCompileLogTemp      AS LONGCHAR                                NO-UNDO.
        DEFINE VARIABLE lcCompileLogs         AS LONGCHAR                                NO-UNDO.
        DEFINE VARIABLE cCompileLogsFile      AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE cImportLogsDir        AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE hWorkspaceData        AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE hWorkspaceImportsData AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE hBuf                  AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE hTaskBuf              AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE hImportBuf            AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE hQry                  AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE oFileHelper           AS appSolutions.appSolute.Utils.FileHelper NO-UNDO.
        DEFINE VARIABLE iTaskNum              AS INTEGER                                 NO-UNDO.
        DEFINE VARIABLE cObjectType           AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE cObjectName           AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE cObjectPMod           AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE iObjectVersion        AS INTEGER                                 NO-UNDO.
        DEFINE VARIABLE lCompileErrorsExist   AS LOGICAL                                 INITIAL FALSE NO-UNDO.
        DEFINE VARIABLE cTaskList             AS CHARACTER                               NO-UNDO.

        hWorkspaceData = RTBProxyHelper:rtbGetWorkspaceHandle(pcWorkspace).

        IF VALID-HANDLE(hWorkspaceData) THEN
        DO:

            /* If the workspace has source code and rcode turned on then do the compiles*/
            hBuf = hWorkspaceData:DEFAULT-BUFFER-HANDLE.
            hBuf:FIND-FIRST(SUBSTITUTE(" WHERE wspace-id = &1", QUOTER(pcWorkspace))).

            IF hBuf:AVAILABLE THEN
                IF NOT hBuf::s-code THEN
                DO:
                    ASSIGN
                        cMessage = "Workspace not set to contain source code. Compile wil not be performed!".

                    DisplayMessage(cMessage, TRUE, TRUE).

                    RETURN.
                END.

            /* Import task data*/
            hImportTaskData = RTBProxyHelper:custRtbGetImportTableTasksHandle(pcWorkspace, OUTPUT cTaskList).

            hWorkspaceImportsData = RTBProxyHelper:rtbGetWorkspaceImportsHandle(pcWorkspace).

            IF VALID-HANDLE(hWorkspaceImportsData) THEN
            DO:
                ASSIGN
                    cMessage = SUBSTITUTE("   Compiling contents of import list for workspace '&1'", pcWorkspace).
                DisplayMessage(cMessage, TRUE, TRUE).

                hImportBuf = hWorkspaceImportsData:DEFAULT-BUFFER-HANDLE.
                CREATE QUERY hQry.
                hQry:SET-BUFFERS(hImportBuf).
                hQry:QUERY-PREPARE("FOR EACH " + hWorkspaceImportsData:NAME).
                hQry:QUERY-OPEN().
                hQry:GET-FIRST().

                REPEAT WHILE NOT hQry:QUERY-OFF-END:
                    IF hImportBuf::Done THEN
                    DO:
                        ASSIGN
                            cCompileLog    = "":U
                            cError         = "":U
                            cObjectType    = hImportBuf::obj-type
                            cObjectName    = hImportBuf::Object
                            cObjectPMod    = hImportBuf::Pmod
                            iObjectVersion = hImportBuf::VERSION.

                        compileObject(
                            INPUT  hImportBuf::wspace-id,
                            INPUT  0, /*TaskNum,*/
                            INPUT  cObjectType,
                            INPUT  cObjectName,
                            INPUT  cObjectPMod,
                            INPUT  TRUE, /* Xref */
                            INPUT  FALSE, /* Listings */
                            INPUT  TRUE, /* Verbose */
                            OUTPUT cCompileLog,
                            OUTPUT cError).

                        IF TRIM(cError) NE "":U THEN
                        DO:

                            IF ProcessImportTableTasks THEN
                            DO:
                                iTaskNum = RTBProxyHelper:custRtbGetObjectTask(
                                    INPUT cObjectType,
                                    INPUT cObjectName,
                                    INPUT cObjectPmod,
                                    INPUT iObjectVersion).

                                hTaskBuf = hImportTaskData:DEFAULT-BUFFER-HANDLE.
                                hTaskBuf:FIND-FIRST(SUBSTITUTE(" where task-num EQ &1", iTaskNum)) NO-ERROR.

                                IF hTaskBuf:AVAILABLE THEN
                                DO:
                                    IF hTaskBuf::compileFailed = FALSE THEN
                                        ASSIGN
                                            hTaskBuf::compileFailed = TRUE
                                            lCompileErrorsExist     = TRUE.
                                END.
                            END.

                            IF DebugOn THEN
                                DisplayMessage(cError, TRUE, TRUE).
                            ASSIGN
                                plcCompileResult = plcCompileResult + "~n":U + cError.
                        END.

                        /* If we get a compile log being returned, then load the contents into a longhcar that we write out at the end of the compile to a common log file */
                        IF cCompileLog NE "":U THEN
                        DO:
                            FILE-INFO:FILE-NAME = cCompileLog.
                            IF FILE-INFO:FULL-PATHNAME NE "":U AND
                                FILE-INFO:FULL-PATHNAME NE ? THEN
                            DO:
                                COPY-LOB FROM FILE FILE-INFO:FULL-PATHNAME TO lcCompileLogTemp.
                                IF lcCompileLogTemp NE "":U THEN
                                    ASSIGN
                                        lcCompileLogs = lcCompileLogs + "~n":U + lcCompileLogTemp.
                            END.

                            DisplayMessage(lcCompileLogTemp, TRUE, TRUE).
                        END.
                    END.

                    hQry:GET-NEXT().
                END.

                ASSIGN
                    plcCompileResult = TRIM(plcCompileResult, "~n":U)
                    lcCompileLogs    = TRIM(lcCompileLogs, "~n":U).

                ASSIGN
                    cMessage = "   Compilation complete.".
                DisplayMessage(cMessage, TRUE, TRUE).

                IF lcCompileLogs NE "":U THEN
                DO ON ERROR UNDO:
                    ASSIGN
                        cCompileLogsFile = buildOutputFilename (pcWorkspace, "ImportCompileLog":U).

                    checkLogsDir(
                        INPUT  "Imports",
                        INPUT  RtbWorkspaceId,
                        INPUT  TRUE,
                        OUTPUT cImportLogsDir ).

                    cCompileLogsFile = SUBSTITUTE("&1/&2":U, cImportLogsDir, cCompileLogsFile).

                    /* Unless an error has been thrown in the checkLogsDir above, then write the contents of the file to disk */
                    COPY-LOB FROM lcCompileLogs TO FILE cCompileLogsFile.

                    ASSIGN
                        plcCompileResult = lcCompileLogs
                        cMessage         = SUBSTITUTE("Import compile log written to : &1", cCompileLogsFile).

                    CATCH Err AS Progress.Lang.Error :
                        ASSIGN
                            cError = Err:GetMessage(1).
                        DisplayMessage(cError, TRUE, TRUE).
                    END CATCH.
                END.

                ASSIGN
                    cMessage = SUBSTITUTE("   Performing selective compile of workspace '&1'", pcWorkspace).
                DisplayMessage(cMessage, TRUE, FALSE).

                selectiveCompile(
                    INPUT  pcWorkspace,
                    INPUT  0,     /* piTaskNum        */
                    INPUT  "":U,  /* pcModule         */
                    INPUT  "":U,  /* pcGroup          */
                    INPUT  "":U,  /* pcObjType        */
                    INPUT  "":U,   /* pcObject         */
                    INPUT  FALSE, /* plForceCompile   */
                    INPUT  FALSE, /* plListing        */
                    INPUT  TRUE,  /* plXref           */
                    INPUT  TRUE,  /* plIgnoreWarn     */
                    INPUT  FALSE, /* plStopOnError    */
                    INPUT  FALSE, /* plTaskRcode      */
                    OUTPUT cError,
                    OUTPUT cCompileLog).

                IF cError NE "":U THEN
                    DisplayMessage(cError, TRUE, FALSE).
            END.
        END.
        ELSE
        DO:
            ASSIGN
                cError = SUBSTITUTE("*** Workspace '&1' is not valid. Import processing cannot continue!", pcWorkspace).
            DisplayMessage(cError, TRUE, FALSE).
        END.

        /* Task with failed compiles output */
        IF lCompileErrorsExist AND ProcessImportTableTasks THEN
        DO:
            hBuf = hImportTaskData:DEFAULT-BUFFER-HANDLE.
            CREATE QUERY hQry.
            hQry:SET-BUFFERS(hBuf).
            hQry:QUERY-PREPARE("FOR EACH " + hImportTaskData:NAME).
            hQry:QUERY-OPEN().
            hQry:GET-FIRST().

            ASSIGN
                cMessage = "~n    Tasks with compile failures: ~n    Task-num  Manager         Programmer                Summary".
            DisplayMessage(cMessage, TRUE, FALSE).

            ASSIGN
                cMessage = "~n    ========= =============== ========================= ==================================================".
            DisplayMessage(cMessage, TRUE, FALSE).

            REPEAT WHILE NOT hQry:QUERY-OFF-END:
                IF hBuf::compileFailed = TRUE THEN
                DO:
                    ASSIGN
                        cMessage = SUBSTITUTE("    &1 &2 &3 &4",
                    STRING(hBuf::task-num, "999999999"),
                    hBuf::manager + FILL(" ", 15 - length(hBuf::manager)),
                    hBuf::programmer + FILL(" ", 25 - length(hBuf::programmer)),
                    hBuf::summary)+ FILL(" ", 25 - length(hBuf::summary)).

                    DisplayMessage(cMessage, TRUE, FALSE).
                END.
                hQry:GET-NEXT ().
            END.
        END.

        RETURN.

        CATCH AppError AS Progress.Lang.AppError :
            DisplayMessage(SUBSTITUTE("ERROR: &1~n&2", AppError:ReturnValue, AppError:CallStack), TRUE, TRUE).
        END CATCH.

        FINALLY:
            DELETE OBJECT hWorkspaceData NO-ERROR.

            DELETE OBJECT hWorkspaceImportsData NO-ERROR.

            DELETE OBJECT hQry NO-ERROR.

            DELETE OBJECT hImportTaskData NO-ERROR.
        END FINALLY.
    END METHOD.


    METHOD PUBLIC VOID getImportTableTasks(
        INPUT pcWorkspaceId AS CHARACTER):

        DEFINE VARIABLE cRowIdent     AS CHARACTER NO-UNDO INITIAL "FIRST".
        DEFINE VARIABLE iRowsToreturn AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cMessage      AS CHARACTER NO-UNDO.

        DEFINE VARIABLE hData         AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf          AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry          AS HANDLE    NO-UNDO.

        hData = RTBProxyHelper:rtbGetImportableTasksHandle(
            INPUT-OUTPUT cRowIdent,
            INPUT        iRowsToReturn,
            INPUT        pcWorkspaceId).

        IF VALID-HANDLE(hData) THEN
        DO:
            hBuf = hData:DEFAULT-BUFFER-HANDLE.
            CREATE QUERY hQry.
            hQry:SET-BUFFERS(hBuf).
            hQry:QUERY-PREPARE("FOR EACH " + hData:NAME).
            hQry:QUERY-OPEN().
            hQry:GET-FIRST().

            REPEAT WHILE NOT hQry:QUERY-OFF-END:
                ASSIGN
                    cMessage = hBuf::task-num.
                DisplayMessage(cMessage, TRUE, TRUE).
                hQry:GET-NEXT ().
            END.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hData NO-ERROR.
        END FINALLY.

    END METHOD.


    /*------------------------------------------------------------------------------
            Purpose: Check if we have a valid AppServer connection, and if not, and we have a connection string then try and connect
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID connectAppServer(
        INPUT pcAppServerConnectString AS CHARACTER ):

        DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.

        IF pcAppServerConnectString NE "":U THEN
            ASSIGN
                AppServerConnectString = pcAppServerConnectString.

        IF AppServerConnectString NE "":U THEN
        DO:
            IF NOT VALID-HANDLE(AppServerHandle) OR
                AppServerHandle = SESSION:HANDLE THEN
                CREATE SERVER AppServerHandle.

            IF NOT RtbAppServerConnected THEN
            DO:
                IF AppServerConnectString  BEGINS "AppServer:":U THEN
                DO:
                    ASSIGN
                        AppServerConnectString = SUBSTITUTE("-URL &1", AppServerConnectString).

                    IF NOT AppServerHandle:CONNECTED() THEN
                        AppServerHandle:CONNECT(SUBSTITUTE("&1", AppServerConnectString )) NO-ERROR.

                    IF AppServerHandle:CONNECTED () THEN
                        ASSIGN
                            RtbAppServerConnected          = TRUE
                            RTBProxyHelper:AppServerHandle = AppServerHandle.
                END.
            END.

            IF NOT RtbAppServerConnected THEN
            DO:
                ASSIGN
                    AppServerHandle = ?.
                UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("AppServer '&1' not connected...", pcAppServerConnectString ), 0).
            END.
            ELSE
                IF DebugOn THEN
                DO:
                    ASSIGN
                        cMessage = SUBSTITUTE("   ** [Debug] AppServer connected: : &1", pcAppServerConnectString).
                    DisplayMessage(cMessage, TRUE, TRUE).
                END.
        END.
        ELSE
            ASSIGN
                AppServerHandle = SESSION:HANDLE.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID connectAppServer (
        INPUT pcAppServerConnectString AS CHARACTER,
        OUTPUT phAppServerHandle       AS HANDLE ):

        connectAppServer(pcAppServerConnectString).

        ASSIGN
            phAppServerHandle = AppServerHandle.

        RETURN.

    END METHOD.
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID createDeploymentRecord(
        INPUT         pcWorkspace     AS CHARACTER,
        INPUT         pcSiteCode      AS CHARACTER,
        INPUT         piRelease       AS INTEGER,
        INPUT         piSequence      AS INTEGER,
        INPUT         pcDir           AS CHARACTER):

        DEFINE VARIABLE cRowident       AS CHARACTER NO-UNDO INITIAL "First".
        DEFINE VARIABLE cMessage        AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hDeploymentData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf            AS HANDLE    NO-UNDO.

        IF piSequence = 0 THEN
            ASSIGN
                piSequence = 1.

        pcDir = CalculateDeploymentDirectory(
            INPUT pcWorkspace,
            INPUT pcSiteCode,
            INPUT pcDir,
            INPUT piSequence,
            INPUT piRelease).

        hDeploymentData = RTBProxyHelper:rtbGetDeploymentsHandle(
            INPUT-OUTPUT cRowident,
            INPUT        0,
            INPUT        pcWorkspace,
            INPUT        pcSiteCode).

        hBuf = hDeploymentData:DEFAULT-BUFFER-HANDLE.
        hBuf:FIND-FIRST (SUBSTITUTE(" where deploy-sequence = &1", QUOTER(piSequence))) NO-ERROR.



        IF NOT hBuf:AVAILABLE THEN
        DO:

            /* Get an empty rtbTask schema */
            RTBProxyHelper:rtbGetRowValues (
                INPUT-OUTPUT cRowIdent,
                INPUT 0,
                INPUT "rtbDeployment",
                INPUT-OUTPUT TABLE-HANDLE hDeploymentData,
                INPUT FALSE).

            IF VALID-HANDLE(hDeploymentData) THEN
            DO:
                hBuf = hDeploymentData:DEFAULT-BUFFER-HANDLE.
                hBuf:FIND-FIRST().


                ASSIGN
                    hBuf::RowMod          = "A":U
                    hBuf::wspace-id       = pcWorkspace
                    hBuf::site-code       = pcSiteCode
                    hBuf::release-num     = piRelease
                    hBuf::deploy-sequence = piSequence
                    hBuf::depRootDir      = pcDir
                    hBuf::deploy-status   = "WIP":U.

                RTBProxyHelper:rtbSetRowValues (
                    INPUT "rtbDeployment",
                    INPUT-OUTPUT TABLE-HANDLE hDeploymentData).

                IF VALID-HANDLE(hDeploymentData) THEN
                DO:
                    /* read the task number and other committed data */
                    hBuf = hDeploymentData:DEFAULT-BUFFER-HANDLE.
                    hBuf:FIND-FIRST().

                    IF hBuf:AVAILABLE THEN
                    DO:
                        ASSIGN
                            cMessage = SUBSTITUTE("   Deployment record created.~n    Deployment sequence number: &1~n    Root directory: &2", hBuf::deploy-sequence, hBuf::Directory).
                        DisplayMessage(cMessage, YES, YES).
                    END.
                END.
            END.

        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :

            MESSAGE "*** ERROR : " err:GetMessage(1)
                VIEW-AS ALERT-BOX.

            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hDeploymentData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID createImportAnalysisReport(
        INPUT  pcWspaceId             AS CHARACTER,
        INPUT  pcReportBaseDir        AS CHARACTER,
        INPUT  pcImportFilterSettings AS CHARACTER,
        OUTPUT pcImportReport         AS CHARACTER,
        OUTPUT pcError                AS CHARACTER):

        DEFINE VARIABLE hReport          AS HANDLE                                       NO-UNDO.
        DEFINE VARIABLE hFS              AS HANDLE                                       NO-UNDO.
        DEFINE VARIABLE iReleaseFrom     AS INTEGER                                      NO-UNDO.
        DEFINE VARIABLE cMessage         AS CHARACTER                                    NO-UNDO.
        DEFINE VARIABLE cWorkspaceString AS CHARACTER                                    NO-UNDO.
        DEFINE VARIABLE oDirectoryHelper AS appSolutions.appSolute.Utils.DirectoryHelper NO-UNDO.
        DEFINE VARIABLE cError           AS CHARACTER                                    NO-UNDO.

        IF validWorkspace(pcWspaceId) THEN
        DO ON ERROR UNDO:
            EMPTY TEMP-TABLE ttFileSpec.
            /* Create a string from the workspace without "-", "_" and spaces*/
            ASSIGN
                cWorkspaceString = REPLACE(pcWspaceId, "-":U, "":U)
                cWorkspaceString = REPLACE(cWorkspaceString, "_":U, "":U)
                cWorkspaceString = REPLACE(cWorkspaceString, " ":U, "":U).

            /* Make sure the dir for the report exists */
            oDirectoryHelper = NEW appSolutions.appSolute.Utils.DirectoryHelper().

            IF NOT oDirectoryHelper:DirectoryExists(pcReportBaseDir) THEN
                oDirectoryHelper:makeDirectory(pcReportBaseDir).

            /* We want to get the latest release from the specified workspace */
            IF NOT CheckImportTableExists(pcWspaceId) THEN
            DO:
                ASSIGN
                    pcError = SUBSTITUTE("Error: Unable to generate Import Analysis Report there is no import control table in workspace: &1", pcWspaceId).
                RETURN.
            END.

            ASSIGN
                pcImportReport = buildOutputFilename (cWorkspaceString, "ImportReport":U).

            CREATE ttFileSpec.
            ASSIGN
                ttFileSpec.SourceDir = pcReportBaseDir
                ttFileSpec.FileName  = pcImportReport
                hFS                  = TEMP-TABLE ttFileSpec:HANDLE.

            main:
            DO ON ERROR UNDO main, LEAVE main:
                ASSIGN
                    cMessage = "Creating import analysis report...":U.
                DisplayMessage(cMessage, ?, YES, DebugOn).

                RUN scm/custom/rtb/p/rtb_rpt_wspace.p PERSISTENT SET hReport NO-ERROR.

                IF NOT VALID-HANDLE(hReport) THEN
                DO:
                    ASSIGN
                        pcError = " *** Unable to start Roundtable Import Analysis Report".
                    RETURN.
                END.

                /* Next, we want to run one of the reports */
                RUN rptRtbWorkspaceImport IN hReport (
                    INPUT pcWspaceId, /* Workspace ID */
                    INPUT TABLE-HANDLE hFS).

                ASSIGN
                    FILE-INFORMATION:FILE-NAME = pcReportBaseDir + "/":U + pcImportReport
                    pcImportReport             = FILE-INFORMATION:FULL-PATHNAME.
                IF pcImportReport = ? OR pcImportReport = "":U THEN
                    ASSIGN
                        pcError = SUBSTITUTE("Import analysis report '&1' not created", pcImportReport).
                ELSE
                    /* Add the import filter criteria to the report */
                    IF pcImportFilterSettings NE "":U THEN
                    DO:
                        OUTPUT TO value(pcImportReport) APPEND.

                        PUT UNFORMATTED pcImportFilterSettings SKIP.

                        OUTPUT CLOSE.
                    END.

            END.

            CATCH Err AS Progress.Lang.Error :
                ASSIGN
                    pcError = SUBSTITUTE("Error: Invalid base directory &2 for import analysis report. Cannot create report.~n&1", Err:GetMessage(1), pcReportBaseDir).
            END CATCH.
        END.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
          Purpose: Create a release in RTB for the current workspace
          Notes:
                NB: If plWhenNeeded is set to YES, then a release will
                    not be created in the workspace if there are no new events
                    to include in the release.
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC LOGICAL createRelease(
        INPUT  pcWspaceId     AS CHARACTER,
        INPUT  pcUserId       AS CHARACTER,
        INPUT  pcReleaseNote  AS CHARACTER,
        INPUT  plWhenNeeded   AS LOGICAL,
        OUTPUT piReleaseNum   AS INTEGER,
        OUTPUT piEventCount   AS INTEGER):

        DEFINE VARIABLE lOK                  AS LOGICAL   NO-UNDO.
        DEFINE VARIABLE iWSEventCounter      AS INTEGER   NO-UNDO.
        DEFINE VARIABLE iCurrentReleaseNum   AS INTEGER   NO-UNDO.
        DEFINE VARIABLE iCurrentEventCounter AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cCurrentRelNote      AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cMessage             AS CHARACTER NO-UNDO.

        DEFINE VARIABLE cRowIdent            AS CHARACTER NO-UNDO INITIAL "First".
        DEFINE VARIABLE cError               AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hBuf                 AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hTTRelease           AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hRtbWsCount          AS HANDLE    NO-UNDO.

        IF RTBProxyHelper:rtbWorkspaceExists(pcWspaceId) THEN
        DO:

            /* Start the workspace event counter super procedure */
            RUN rtb/p/rtb_s_wscount.p ON AppServerHandle PERSISTENT SET hRtbWsCount.

            /* Get the latest event count for the workspace */
            iWSEventCounter =  DYNAMIC-FUNCTION('fnRtbGetWorkspaceEventCounter':U IN hRtbWsCount, pcWspaceId).

            /* Get the details of the latest release in the workspace */
            getLastWorkspaceRelease (
                INPUT  pcWspaceId,
                OUTPUT iCurrentReleaseNum,
                OUTPUT iCurrentEventCounter,
                OUTPUT cCurrentRelNote).

            IF iWSEventCounter = 0 AND
                plWhenNeeded THEN
            DO:
                ASSIGN
                    cMessage = "   Release not created! No events in the workspace.".
                DisplayMessage(cMessage, YES, YES).
                /*                UNDO, THROW NEW Progress.Lang.AppError(cMessage, 0).*/
                RETURN FALSE.
            END.
            ELSE
                IF iCurrentEventCounter = iWSEventCounter AND
                    plWhenNeeded THEN
                DO:
                    ASSIGN
                        cMessage = "   Release not created! No new events to include in release.".
                    DisplayMessage(cMessage, YES, YES).
                    RETURN FALSE.

                /*                    UNDO, THROW NEW Progress.Lang.AppError("   Release not created! No new events to include in release.", 0).*/
                END.
                ELSE
                DO ON ERROR UNDO:
                    /* Check if we have passed in a release note, if not, create one */
                    IF pcReleaseNote = "":U THEN
                        pcReleaseNote = SUBSTITUTE("Auto release : &1", STRING(NOW, "99/99/9999 HH:MM:SS+HH:MM":U)).

                    RTBProxyHelper:rtbCreateRelease(
                        INPUT pcWspaceId,
                        INPUT pcReleaseNote).

                    getLastWorkspaceRelease(
                        INPUT  pcWspaceId,
                        OUTPUT piReleaseNum,
                        OUTPUT piEventCount,
                        OUTPUT cCurrentRelNote).

                    ASSIGN
                        cMessage = SUBSTITUTE("   Release &1 created in &2 for event number &3 with user note '&4'", piReleaseNum, pcWspaceId, piEventCount, pcReleaseNote).
                    DisplayMessage(cMessage, TRUE, TRUE).

                    CATCH err AS Progress.Lang.Error :
                        ASSIGN
                            cError = cError + "*** Release creation failed : ":U + err:GetMessage(1).
                        DisplayMessage(cError, TRUE, TRUE).
                    END CATCH.
                END.
        END.
        ELSE
            UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("*** Workspace '&1' is not valid. Release processing cannot continue!", pcWspaceId), 0).

        RETURN TRUE.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTTRelease NO-ERROR.

            DELETE PROCEDURE hRtbWsCount NO-ERROR.
            ASSIGN
                hRtbWsCount = ?.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID createReleaseReport(
        INPUT  pcWspaceId      AS CHARACTER,
        INPUT  piReleaseFrom   AS INTEGER,
        INPUT  piReleaseTo     AS INTEGER,
        INPUT  pcReportBaseDir AS CHARACTE,
        OUTPUT pcReleaseReport AS CHARACTER):

        DEFINE VARIABLE hReport          AS HANDLE                                       NO-UNDO.
        DEFINE VARIABLE hFS              AS HANDLE                                       NO-UNDO.
        DEFINE VARIABLE iReleaseFrom     AS INTEGER                                      NO-UNDO.
        DEFINE VARIABLE cMessage         AS CHARACTER                                    NO-UNDO.
        DEFINE VARIABLE cWorkspaceString AS CHARACTER                                    NO-UNDO.
        DEFINE VARIABLE iEventCounter    AS INTEGER                                      NO-UNDO.
        DEFINE VARIABLE cReleaseNote     AS CHARACTER                                    NO-UNDO.
        DEFINE VARIABLE oDirectoryHelper AS appSolutions.appSolute.Utils.DirectoryHelper NO-UNDO.
        DEFINE VARIABLE cError           AS CHARACTER                                    NO-UNDO.
        DEFINE VARIABLE lValidDir        AS LOGICAL                                      NO-UNDO.

        EMPTY TEMP-TABLE ttFileSpec.
        /* Create a string from the workspace without "-", "_" and spaces*/
        ASSIGN
            cWorkspaceString = REPLACE(pcWspaceId, "-":U, "":U)
            cWorkspaceString = REPLACE(cWorkspaceString, "_":U, "":U)
            cWorkspaceString = REPLACE(cWorkspaceString, " ":U, "":U).

        /* Make sure the dir for the report exists */
        oDirectoryHelper = NEW appSolutions.appSolute.Utils.DirectoryHelper().

        IF NOT oDirectoryHelper:DirectoryExists(pcReportBaseDir + "/") THEN
            oDirectoryHelper:makeDirectory(pcReportBaseDir + "/"). /* Create if not found... */

        getRtbReleaseInfo (
            INPUT  pcWspaceId,
            INPUT  piReleaseTo,
            OUTPUT iEventCounter,
            OUTPUT cReleaseNote).

        IF iEventCounter = 0 THEN /* The release was not found */
            UNDO, THROW NEW Progress.Lang.AppError("Error: Unable to generate Release Report as release " + STRING(piReleaseTo) + " does not exist in workspace: " + pcWspaceId, 0).

        ASSIGN
            pcReleaseReport = cWorkspaceString + "_ReleaseReport_" + STRING(piReleaseFrom, "999":U) +  "_" + STRING(piReleaseTo, "999":U) + ".txt".

        CREATE ttFileSpec.
        ASSIGN
            ttFileSpec.SourceDir = pcReportBaseDir
            ttFileSpec.FileName  = pcReleaseReport
            hFS                  = TEMP-TABLE ttFileSpec:HANDLE.

        main:
        DO ON ERROR UNDO main, LEAVE main:
            ASSIGN
                cMessage = "   Creating release report...":U.
            DisplayMessage(cMessage, YES, DebugOn).

            RUN rtb/p/rtb_rpt_wspace.p PERSISTENT SET hReport NO-ERROR.

            IF NOT VALID-HANDLE(hReport) THEN
                UNDO, THROW NEW Progress.Lang.AppError("Unable to start Roundtable Release Workspace Report", 0).

            /* Next, we want to run one of the reports */
            RUN rptRtbWorkspaceRelease IN hReport (
                INPUT pcWspaceId, /* Workspace ID */
                INPUT piReleaseFrom, /* Beginning release # */
                INPUT piReleaseTo, /* Ending release # */
                INPUT TRUE, /* TRUE to print page headers */
                INPUT TRUE, /* TRUE to print release info */
                INPUT TRUE, /* TRUE to print object headers */
                INPUT TRUE, /* TRUE to print object details */
                INPUT TRUE, /* TRUE to print task summaries */
                INPUT TRUE, /* TRUE to print task details */
                INPUT TRUE, /* TRUE to print object variants */
                INPUT FALSE, /* FALSE to print object assignments */
                INPUT TRUE, /* TRUE to print latest object version only */
                INPUT TRUE, /* TRUE to print version info */
                INPUT TRUE, /* TRUE to print object release info */
                INPUT TRUE, /* TRUE to print object task info */
                INPUT TRUE, /* TRUE to print object version notes */
                INPUT TABLE-HANDLE hFS).

            ASSIGN
                FILE-INFORMATION:FILE-NAME = SUBSTITUTE("&1/&2":U, pcReportBaseDir, pcReleaseReport)
                pcReleaseReport            = FILE-INFORMATION:FULL-PATHNAME.
            IF pcReleaseReport = ? /*OR pcReleaseReport = "":U */ THEN
                UNDO, THROW NEW Progress.Lang.AppError("Release report " + pcReleaseReport + " not created.", 0).

            ASSIGN
                RtbReleaseReport = pcReleaseReport.
        END.

        /*            IF cError NE "":U OR                                                                                                                      */
        /*                NOT lValidDir THEN                                                                                                                    */
        /*            DO:                                                                                                                                       */
        /*                ASSIGN                                                                                                                                */
        /*                    pcError = SUBSTITUTE("Error: Invalid base directory '&1' for release report. Cannot create report.~n&2", pcReportBaseDir, cError).*/
        /*                RETURN.                                                                                                                               */
        /*            END.                                                                                                                                      */
        RETURN.

        CATCH Err AS Progress.Lang.Error :
            UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Error: Invalid base directory '&1' for release report. Cannot create report.~n&2", pcReportBaseDir, Err:GetMessage(1)), 0).
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID deleteSchemaUpdateList(
        INPUT  pcWorkspace AS CHARACTER,
        INPUT  pcSiteCode  AS CHARACTER,
        OUTPUT pcError     AS CHARACTER ):

        RTBProxyHelper:rtbDeleteSchemaUpdateList(
            INPUT  pcWorkspace,
            INPUT  pcSiteCode).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            ASSIGN
                pcError = err:GetMessage(1).
            IF pcError <> "" AND (ENTRY(1, pcError, CHR(1)) <> "WARNING" AND ENTRY(1, pcError, CHR(1)) <> "INFO") THEN
                DisplayMessage(pcError, YES, YES).
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID deleteWorkspaceImportList(
        INPUT  pcWorkspace AS CHARACTER,
        OUTPUT pcError     AS CHARACTER):

        IF validWorkspace(pcWorkspace) THEN
            RTBProxyHelper:rtbDeleteWorkspaceImportList (pcWorkspace).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            ASSIGN
                pcError = err:GetMessage(1).
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL DisplayMessage(
        PcMessage    AS CHARACTER,
        phEditor     AS HANDLE,   /* Should output go to the editor?     */
        plLogFile    AS LOGICAL,  /* Should output go to the log file ?  */
        PLMessageBox AS LOGICAL  ):

        /*------------------------------------------------------------------------------
          Purpose:  Standard display of messages & errors to the editor widget and log file.
            Notes:
        ------------------------------------------------------------------------------*/
        IF pcMessage NE "":U THEN
        DO:
            /* If we have to display a message, do so */
            IF plMessageBox THEN
                MESSAGE pcMessage VIEW-AS ALERT-BOX.

            /* If we have a valid editor handle to show the mssage in, insert the message into the editor */
            IF VALID-HANDLE(phEditor) THEN
                phEditor:INSERT-STRING(pcMessage + CHR(10)).

            /* If the log file stream is closed - open it */
            IF SEEK(sLogStream) = ?  THEN
                OpenStream(INPUT YES).

            /* If we have a valid log stream, then write the message to the log stream */
            IF SEEK(sLogStream) NE ?  THEN
            DO:
                IF plLogFile THEN
                    PUT STREAM sLogStream UNFORMATTED pcMessage SKIP.
            END.
            ELSE
                MESSAGE "Log stream closed."
                    VIEW-AS ALERT-BOX INFORMATION BUTTONS OK.
        END.

        RETURN TRUE.   /* Function return value. */

    END METHOD.

    METHOD PUBLIC VOID DisplayMessage(
        pcMessage    AS CHARACTER,
        plLogFile    AS LOGICAL,  /* Should output go to the log file ?  */
        plMessageBox AS LOGICAL  ):

        /*------------------------------------------------------------------------------
          Purpose:  Standard display of messages & errors to the editor widget and log file.
            Notes:
        ------------------------------------------------------------------------------*/
        IF pcMessage NE "":U THEN
        DO:
            /*            LogHelper:DisplayMessage(pcMessage, plLogFile, plMessageBox).*/

            /* If we have to display a message, do so */
            IF plMessageBox THEN
                MESSAGE pcMessage VIEW-AS ALERT-BOX.

            /* If the log file stream is closed - open it */
            IF SEEK(sLogStream) = ?  THEN
                OpenStream(INPUT YES).

            /* If we have a valid log stream, then write the message to the log stream */
            IF SEEK(sLogStream) NE ?  THEN
            DO:
                IF plLogFile THEN
                    PUT STREAM sLogStream UNFORMATTED pcMessage SKIP.
            END.
            ELSE
                MESSAGE "Log stream closed."
                    VIEW-AS ALERT-BOX INFORMATION BUTTONS OK.
        END.

        RETURN.

    END METHOD.

    METHOD PUBLIC VOID DisplayMessage(
        INPUT pcMessage    AS CHARACTER):

        /*------------------------------------------------------------------------------
          Purpose:  Standard display of messages & errors to the editor widget and log file.
            Notes:
        ------------------------------------------------------------------------------*/
        IF pcMessage NE "":U THEN
            MESSAGE pcMessage VIEW-AS ALERT-BOX.

        RETURN.

    END METHOD.

    METHOD PUBLIC VOID DisplayMessage(
        plcMessage    AS LONGCHAR,
        plLogFile     AS LOGICAL,  /* Should output go to the log file ?  */
        plMessageBox  AS LOGICAL):

        IF plcMessage NE "":U THEN
        DO:
            /* If we have to display a message, do so */
            IF plMessageBox THEN
                MESSAGE STRING(plcMessage) VIEW-AS ALERT-BOX.

            /* If the log file stream is closed - open it */
            IF SEEK(sLogStream) = ?  THEN
                OpenStream(INPUT YES).

            /* If we have a valid log stream, then write the message to the log stream */
            IF SEEK(sLogStream) NE ?  THEN
            DO:
                IF plLogFile THEN
                    PUT STREAM sLogStream UNFORMATTED STRING(plcMessage) SKIP.
            END.
            ELSE
                MESSAGE "Log stream closed."
                    VIEW-AS ALERT-BOX INFORMATION BUTTONS OK.
        END.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER getDefaultWorkspaceDeploymentSite(
        INPUT pcWorkspace AS CHARACTER ):

        DEFINE VARIABLE cDefaultDeploymentSiteCode AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cError                     AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hWorkspaceData             AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hDeploymentSiteData        AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hWorkspaceBuf              AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hSiteBuf                   AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry                       AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cSiteNotes                 AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cRowIdent                  AS CHARACTER NO-UNDO.

        hWorkspaceData = RTBProxyHelper:rtbGetWorkspaceHandle(pcWorkspace).

        IF VALID-HANDLE(hWorkspaceData) THEN
        DO:
            hWorkspaceBuf = hWorkspaceData:DEFAULT-BUFFER-HANDLE.
            hWorkspaceBuf:FIND-FIRST("").

            hDeploymentSiteData = RTBProxyHelper:rtbGetDeploymentSitesHandle(INPUT-OUTPUT cRowIdent, 0, pcWorkspace).

            IF VALID-HANDLE(hDeploymentSiteData) THEN
            DO:
                hSiteBuf = hDeploymentSiteData:DEFAULT-BUFFER-HANDLE.
                CREATE QUERY hQry.
                hQry:SET-BUFFERS(hSiteBuf).
                hQry:QUERY-PREPARE("FOR EACH " + hDeploymentSiteData:NAME).
                hQry:QUERY-OPEN().
                hQry:GET-FIRST().

                SiteLoop:
                REPEAT WHILE NOT hQry:QUERY-OFF-END:

                    ASSIGN
                        cSiteNotes = hSiteBuf::siteNotes.

                    IF cSiteNotes MATCHES "*DefaultSite=True*":U THEN
                    DO:
                        ASSIGN
                            cDefaultDeploymentSiteCode = hSiteBuf::site-code.
                        LEAVE SiteLoop.
                    END.

                    hQry:GET-NEXT ().
                END.
            END.

            IF cDefaultDeploymentSiteCode = "":U THEN
            DO:
                ASSIGN
                    cError = SUBSTITUTE("Unable to find 'default' deployment site for workspace '&1'", pcWorkspace).

                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
            END.
        END.

        RETURN cDefaultDeploymentSiteCode.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hDeploymentSiteData NO-ERROR.
            DELETE OBJECT hWorkspaceData NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
          Purpose:  Get the latest release for a workspace
          Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID getLastWorkspaceRelease(
        INPUT  pcWspaceId     AS CHARACTER,
        OUTPUT piReleaseNum   AS INTEGER,
        OUTPUT piEventCounter AS INTEGER,
        OUTPUT pcReleaseNote  AS CHARACTER):

        DEFINE VARIABLE hReleaseData AS HANDLE NO-UNDO.
        DEFINE VARIABLE hBuf         AS HANDLE NO-UNDO.

        hReleaseData = RTBProxyHelper:rtbGetWorkspaceReleasesHandle(pcWspaceId).

        IF VALID-HANDLE(hReleaseData) THEN
        DO:
            hBuf = hReleaseData:DEFAULT-BUFFER-HANDLE.
            hBuf:FIND-FIRST() NO-ERROR.

            IF hBuf:AVAILABLE THEN
                ASSIGN
                    piReleaseNum   = hBuf::release-num
                    piEventCounter = hBuf::event-counter
                    pcReleaseNote  = hBuf::user-note.
        END.

        FINALLY:
            IF VALID-HANDLE(hReleaseData) THEN
                DELETE OBJECT hReleaseData.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
          Purpose:  Get the latest release for a workspace
          Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC INTEGER getLastWorkspaceRelease(
        INPUT  pcWspaceId     AS CHARACTER):

        DEFINE VARIABLE iReleaseNum   AS INTEGER   NO-UNDO.
        DEFINE VARIABLE iEventCounter AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cReleaseNote  AS CHARACTER NO-UNDO.

        getLastWorkspaceRelease(
            INPUT pcWspaceId,
            OUTPUT iReleaseNum,
            OUTPUT iEventCounter,
            OUTPUT cReleaseNote).

        RETURN iReleaseNum.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID getRtbReleaseInfo(
        INPUT  pcWorkspace    AS CHARACTER,
        INPUT  piRelease      AS INTEGER,
        OUTPUT piEventCounter AS INTEGER,
        OUTPUT pcReleaseNote  AS CHARACTER):

        DEFINE VARIABLE hData     AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf      AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cRowIdent AS CHARACTER NO-UNDO.

        hData = RTBProxyHelper:rtbGetReleaseHandle(
            cRowIdent,
            0,
            INPUT        pcWorkspace,
            INPUT        piRelease).

        IF VALID-HANDLE(hData) THEN
        DO:
            hBuf = hData:DEFAULT-BUFFER-HANDLE.
            hBuf:FIND-FIRST("").

            ASSIGN
                pcReleaseNote  = hBuf::user-note
                piEventCounter = hBuf::event-counter.
        END.

        RETURN.


        FINALLY:
            IF VALID-HANDLE(hData) THEN
                DELETE OBJECT hData.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER getRtbVersion(  ):
        DEFINE VARIABLE cRtbVerFile AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cRtbVersion AS CHARACTER NO-UNDO.

        /* Check the version of RTB that we are using. This is needed due to difference in some
          of the routines to start SDOs and super procedures.
        */
        ASSIGN
            cRtbVerFile = SEARCH("rtbver").

        IF cRtbVerFile NE ? THEN
        DO:
            /* read and parse the rtbver file. This is the actual file for the RTB version we are
               using and not the one we use at compilation time - as they may be different */
            INPUT FROM VALUE(cRtbVerFile).
            IMPORT UNFORMATTED cRtbVersion.
            INPUT CLOSE.

            IF cRtbVersion NE "":U THEN
                ASSIGN
                    cRTBVersion = TRIM(REPLACE(cRTBVersion, "Roundtable TSMS release ":U, "":U)).
        END.

        RETURN cRTBVersion.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER getWorkspaceListFromFile(
        INPUT pcFilename AS CHARACTER ):

        DEFINE VARIABLE cWorkspaceList AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE oListHelper    AS appSolutions.appSolute.Utils.ListHelper NO-UNDO.
        DEFINE VARIABLE cError         AS CHARACTER                               NO-UNDO.

        IF pcFilename BEGINS "File:":U THEN
        DO:
            oListHelper = NEW appSolutions.appSolute.Utils.ListHelper().

            cWorkspaceList = oListHelper:getListFromFile(REPLACE(pcFilename, "File:":U, "":U)).

            ASSIGN
                cWorkspaceList = REPLACE(cWorkspaceList, ";":U, ",":U)
                cWorkspaceList = REPLACE(cWorkspaceList, "|":U, ",":U).
        END.

        RETURN cWorkspaceList.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:  Logical check to see if all imports in a workspace have successfully been completed
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL IsImportComplete(
        INPUT pcWorkspace AS CHARACTER ):

        DEFINE VARIABLE lImportComplete       AS LOGICAL NO-UNDO INITIAL TRUE.
        DEFINE VARIABLE hWorkspaceImportsData AS HANDLE  NO-UNDO.
        DEFINE VARIABLE hImportBuf            AS HANDLE  NO-UNDO.
        DEFINE VARIABLE hQry                  AS HANDLE  NO-UNDO.

        hWorkspaceImportsData = RTBProxyHelper:rtbGetWorkspaceImportsHandle(pcWorkspace).

        IF VALID-HANDLE(hWorkspaceImportsData) THEN
        DO:
            hImportBuf = hWorkspaceImportsData:DEFAULT-BUFFER-HANDLE.
            CREATE QUERY hQry.
            hQry:SET-BUFFERS(hImportBuf).
            hQry:QUERY-PREPARE("FOR EACH " + hWorkspaceImportsData:NAME).
            hQry:QUERY-OPEN().
            hQry:GET-FIRST().

            ImportLoop:
            REPEAT WHILE NOT hQry:QUERY-OFF-END:
                IF NOT hImportBuf::Done THEN
                DO:
                    ASSIGN
                        lImportComplete = FALSE.
                    LEAVE ImportLoop.
                END.

                hQry:GET-NEXT ().
            END.
        END.

        RETURN lImportComplete.

        FINALLY:
            DELETE OBJECT hWorkspaceImportsData NO-ERROR.

            DELETE OBJECT hQry NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID markRtbDeployment(
        INPUT  pcWorkspaceID    AS CHARACTER,
        INPUT  pcSiteCode       AS CHARACTER,
        INPUT  piDeploySequence AS INTEGER,
        INPUT  pcMarkCode       AS CHARACTER):

        DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.

        RTBProxyHelper:rtbcustRtbSetDeployStatus(
            INPUT  pcWorkspaceId,
            INPUT  pcSiteCode,
            INPUT  piDeploySequence,
            INPUT  pcMarkCode).

        ASSIGN
            cMessage = SUBSTITUTE("   Deployment '&1' for site '&2' in workspace '&3' marked as '&4'.", piDeploySequence, pcSiteCode, pcWorkspaceId, pcMarkCode).
        DisplayMessage(cMessage, YES, YES).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            IF DebugOn THEN
                DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Create client side session directory to write temp files, xref file etc to.
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbCreateClientSideTempDir(  ):
        DEFINE VARIABLE cSessionTempDir  AS CHARACTER       NO-UNDO.
        DEFINE VARIABLE oDirectoryHelper AS DirectoryHelper NO-UNDO.

        IF DebugOn THEN
            DisplayMessage(SUBSTITUTE("   ** [Debug] : &1", "Creating client-side temp-dir"), YES, YES).

        /* Create client-side temp directory*/
        ASSIGN
            cSessionTempDir = SUBSTITUTE("&1&2/&3/",
                                    REPLACE(SESSION:TEMP-DIRECTORY,"~\":U,"/":U),
                                    "rtb_temp":U,
                                    STRING(RTBSessionID,"9999":U)).

        oDirectoryHelper = NEW appSolutions.appSolute.Utils.DirectoryHelper().
        oDirectoryHelper:makeDirectory(cSessionTempDir).

        IF oDirectoryHelper:DirectoryExists(cSessionTempDir) THEN
            DYNAMIC-FUNCTION('fnRtbSetSessionProperty':U,"client-temp-dir":U,cSessionTempDir).
        ELSE
            UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Directory '&1' not created! Cannot continue", cSessionTempDir), 0).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            IF DebugOn THEN
                DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Delete the client-side session directory and the files within it.
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbDeleteClientSideTempDir(  ):
        DEFINE VARIABLE cSessionTempDir AS CHARACTER NO-UNDO.

        IF DebugOn THEN
            DisplayMessage(SUBSTITUTE("   ** [Debug] : &1", "Deleting client-side temp-dir"), YES, YES).

        ASSIGN
            cSessionTempDir = DYNAMIC-FUNCTION('fnRtbGetSessionProperty':U,"client-temp-dir":U).

        IF cSessionTempDir NE "":U THEN
            DYNAMIC-FUNCTION('fnRtbDeleteOSFile':U,cSessionTempDir).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            IF DebugOn THEN
                DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Change the import status for all objects in the import table to
                     Include.
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID rtbIncludeAllImports(
        INPUT pcWorkspace AS CHARACTER):

        DEFINE VARIABLE hRtbSImport AS HANDLE NO-UNDO.

        RUN rtb/p/rtb_s_import.p ON AppServerHandle PERSISTENT SET hRtbSImport.

        RUN rtb_set_all_import_status IN hRtbSImport (
            INPUT pcWorkspace,
            INPUT "INC":U).

        CATCH err AS Progress.Lang.Error :
            MESSAGE err:GetMessage(1)
                VIEW-AS ALERT-BOX.

            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hRtbSImport NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID init():
        THIS-OBJECT:RTBVersion = GetRTBVersion().

        IF THIS-OBJECT:AppServerHandle = ? THEN
            ASSIGN
                THIS-OBJECT:AppServerHandle = SESSION:HANDLE.

        RTBProxyHelper = NEW RTBProxyHelper(THIS-OBJECT:AppServerHandle).
        TempTableHelper = NEW TempTableHelper().
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID init(
        INPUT  hAppServerHandle AS HANDLE):

        ASSIGN
            THIS-OBJECT:AppServerHandle = hAppServerHandle.

        /* Run the standard init method */
        init().

        RETURN.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID init(

        INPUT pcAppServerConnectString AS CHARACTER ):

        ASSIGN
            AppServerConnectString = pcAppServerConnectString.

        IF AppServerHandle = ? THEN
            connectAppServer("":U).

        /* Run the standard init method */
        init().

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL openStream(
        plAppend AS LOGICAL /* PARAMETEr-DEFINItions */ ) :
        /*------------------------------------------------------------------------------
          Purpose:
            Notes:
        ------------------------------------------------------------------------------*/
        IF plAppend THEN
        DO:
            IF LogFileName NE "":U THEN
                OUTPUT STREAM sLogStream TO VALUE(LogFileName) APPEND.
            ELSE
                OUTPUT STREAM sLogStream TO VALUE(SUBSTITUTE("&1/adrtbapi_process.log", SESSION:TEMP-DIRECTORY)) APPEND.
        END.
        ELSE
        DO:
            IF LogFileName NE "":U THEN
                OUTPUT STREAM sLogStream TO VALUE(LogFileName).
            ELSE
                OUTPUT STREAM sLogStream TO VALUE(SUBSTITUTE("&1/adrtbapi_process.log", SESSION:TEMP-DIRECTORY)).
        END.

        IF SEEK(sLogStream) = ?  THEN
            RETURN FALSE.
        ELSE
            RETURN TRUE.   /* Function return value. */
    END METHOD.

    /*
     File:    processDeployment.p
     Purpose: Adds a Deployment of the given release.
     Notes:   The site "MySite" must exist or change site-code to a site that you have created.
    */
    METHOD PUBLIC VOID processDeployment(
        INPUT         pcWorkspace     AS CHARACTER,
        INPUT-OUTPUT  pcSiteCode      AS CHARACTER,
        INPUT         piRelease       AS INTEGER,
        INPUT         pcDir           AS CHARACTER,
        INPUT         plIncludeSchema AS LOGICAL,
        INPUT         plGenerateReport AS LOGICAL,
        INPUT-OUTPUT  piSequence       AS INTEGER):

        DEFINE VARIABLE cMessage                   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cError                     AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lUpdateDeploymentRelease   AS LOGICAL   NO-UNDO INITIAL FALSE.
        DEFINE VARIABLE lDeploymentRecordExists    AS LOGICAL   NO-UNDO INITIAL FALSE.
        DEFINE VARIABLE lDeploymentSiteExists      AS LOGICAL   NO-UNDO.
        DEFINE VARIABLE iCurrentDeploymentSequence AS INTEGER   NO-UNDO INITIAL 0.
        DEFINE VARIABLE iCurrentDeploymentRelease  AS INTEGER   NO-UNDO INITIAL 0.
        DEFINE VARIABLE iNewDeploymentRelease      AS INTEGER   NO-UNDO.
        DEFINE VARIABLE lDeploymentCreated         AS LOGICAL   NO-UNDO.
        DEFINE VARIABLE hDeploymentSiteData        AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hSiteBuf                   AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hDeployData                AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hDeployBuf                 AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hDeployBuf2                AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry                       AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cRowIdent                  AS CHARACTER NO-UNDO INITIAL "FIRST".

        IF ValidWorkspace(pcWorkspace) THEN
        DO ON ERROR UNDO, THROW:
            IF pcSiteCode = "Default":U THEN
                pcSiteCode = getDefaultWorkspaceDeploymentSite(pcWorkspace).

            IF pcSiteCode = "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError("*** Default deployment site does not exist. Cannot process deployment!", 0).

            IF pcSiteCode NE "":U THEN
            DO:
                ASSIGN
                    cMessage = SUBSTITUTE("   Finding deployment site '&1' in workspace '&2'...":U, pcSiteCode, pcWorkspace).
                DisplayMessage(cMessage, YES, YES).

                hDeploymentSiteData = RTBProxyHelper:rtbGetDeploymentSiteHandle(cRowIdent, 0, pcWorkspace, pcSiteCode).

                IF VALID-HANDLE(hDeploymentSiteData) THEN
                DO:
                    hSiteBuf = hDeploymentSiteData:DEFAULT-BUFFER-HANDLE.
                    hSiteBuf:FIND-FIRST().

                    IF hSiteBuf:AVAILABLE THEN
                    DO:
                        ASSIGN
                            lDeploymentSiteExists = TRUE
                            cMessage              = SUBSTITUTE("   Deployment site '&1' found.", pcSiteCode).
                        DisplayMessage(cMessage, YES, YES).
                    END.
                END.
                ELSE
                DO:
                    ASSIGN
                        lDeploymentSiteExists = FALSE.
                    UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Deployment site '&1' does not exist. Cannot process deployment!", pcSiteCode), 0).
                END.

            END.

            /* We HAVE to use the "Handle" version of the API here to get the data - as we may be updating it later - and ADM2 does not handle data that has been converted to and from JSON being passed back in */
            hDeployData = RTBProxyHelper:rtbGetDeploymentsHandle(
                cRowIdent,
                0,
                pcWorkspace,
                pcSiteCode).

            IF VALID-HANDLE(hDeployData) THEN
            DO:
                hDeployBuf = hDeployData:DEFAULT-BUFFER-HANDLE.
                hDeployBuf:FIND-FIRST() NO-ERROR.

                IF hDeployBuf:AVAILABLE THEN
                DO:

                    ASSIGN
                        iCurrentDeploymentSequence = hDeployBuf::deploy-sequence
                        iCurrentDeploymentRelease  = hDeployBuf::release-num
                        lDeploymentRecordExists    = TRUE.

                    ASSIGN
                        cMessage = SUBSTITUTE("   Deployment found. Deployment sequence : &1. Current release: &2. Deployment status: &3.", hDeployBuf::deploy-sequence, iCurrentDeploymentRelease, hDeployBuf::deploy-status).
                    DisplayMessage(cMessage, YES, YES).
                END.
            END.
            ELSE
                ASSIGN
                    lDeploymentRecordExists = FALSE.

            /* If we have not passed in a release number, then find the last release for the workspace and use this */
            IF piRelease = 0 THEN
            DO:
                ASSIGN
                    cMessage = SUBSTITUTE("   No release number passed in. Finding last release in workspace '&1'...":U, pcWorkspace).
                DisplayMessage(cMessage, YES, YES).

                piRelease = getLastWorkspaceRelease(pcWorkspace).

                IF piRelease NE 0 AND (piRelease = iCurrentDeploymentRelease) THEN
                DO:
                    ASSIGN
                        cMessage = SUBSTITUTE("   ** Release number '&1' is the latest release and is already being used in deployment number &2. Deployment will not be created!":U, iCurrentDeploymentRelease, STRING(iCurrentDeploymentSequence, "999":U)).
                    DisplayMessage(cMessage, YES, YES).

                    RETURN.
                /*                    UNDO, THROW NEW Progress.Lang.AppError(cMessage, 1).*/
                END.
                ELSE
                    IF piRelease NE 0 THEN
                    DO:
                        ASSIGN
                            cMessage = SUBSTITUTE("    Release number '&1' will be used for deployment. ":U, piRelease).
                        DisplayMessage(cMessage, YES, YES).
                    END.
                    ELSE
                    DO:
                        ASSIGN
                            cMessage = "   *** No release found in workspace. Deployment cannot be processed":U.
                        DisplayMessage(cMessage, YES, YES).
                        RETURN.
                    /*                        UNDO, THROW NEW Progress.Lang.AppError(cMessage, 0).*/
                    END.
            END.

            /* If we have set a specific sequence and the latest release is no the same as the release on the current deployment release,
               then we want to update and re-use the existing deployment record */
            IF piSequence NE 0  AND
                (piRelease NE iCurrentDeploymentRelease) THEN
            DO:
                ASSIGN
                    lUpdateDeploymentRelease = TRUE.
            END.
            ELSE
                ASSIGN
                    lUpdateDeploymentRelease = FALSE.

            /* If we have to update the existing deployment with a passed in release number, then get the deployment record and set the release number if necessary  */
            IF lUpdateDeploymentRelease THEN
            DO:
                IF hDeployBuf:AVAILABLE THEN
                DO:
                    ASSIGN
                        cMessage = SUBSTITUTE("   Updating deployment number '&1' for site '&2' to release number '&3'...":U, piSequence, pcSiteCode, piRelease).
                    DisplayMessage(cMessage, YES, YES).

                    /*
                     Create a copy of the original record we received
                    */
                    CREATE BUFFER hDeployBuf2 FOR TABLE hDeployData.
                    hDeployBuf2:BUFFER-CREATE.
                    hDeployBuf2:BUFFER-COPY(hDeployBuf).
                    ASSIGN
                        hDeployBuf2::release-num = piRelease
                        hDeployBuf2::rowMod      = "U".

                    RTBProxyHelper:rtbSetRowValues (
                        INPUT "rtbDeployment":U,
                        INPUT-OUTPUT TABLE-HANDLE hDeployData).
                END.
                ELSE
                DO ON ERROR UNDO:
                    /* Make sure the deploy-sequence is set to 1... */
                    ASSIGN
                        piSequence = 1.

                    /* Create a deployment record we can use */
                    createDeploymentRecord(
                        INPUT  pcWorkspace,
                        INPUT  pcSiteCode,
                        INPUT  piRelease,
                        INPUT  piSequence,
                        INPUT  pcDir).

                    CATCH err AS Progress.Lang.Error :
                        IF DebugOn THEN
                            DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
                        UNDO, THROW err.
                    END CATCH.
                END.
            END.
            ELSE
                /* Mark the previous deployment as Complete if it is not marked as such already */
                IF hDeployBuf:AVAILABLE THEN
                DO ON ERROR UNDO, THROW:
                    IF hDeployBuf::deploy-sequence NE piSequence AND
                        hDeployBuf::deploy-status   = "WIP":U THEN
                    DO ON ERROR UNDO, THROW:
                        ASSIGN
                            cMessage = SUBSTITUTE("   Setting deployment &1 as complete...":U, iCurrentDeploymentSequence).
                        DisplayMessage(cMessage, YES, YES).

                        markRtbDeployment(
                            INPUT  pcWorkspace,
                            INPUT  pcSiteCode,
                            INPUT  iCurrentDeploymentSequence,
                            INPUT "Complete").
                    END.
                END.

            /* If the passed in sequence number is 0, then we want to create a new deployment record
               (and increase the sequence number automatically) to create a new deployment with the latest release*/

            IF (lDeploymentSiteExists AND (piSequence = 0 AND iCurrentDeploymentSequence NE 0)) OR
                NOT lDeploymentRecordExists THEN
            DO ON ERROR UNDO:
                ASSIGN
                    piSequence = iCurrentDeploymentSequence + 1.

                createDeploymentRecord(
                    INPUT  pcWorkspace,
                    INPUT  pcSiteCode,
                    INPUT  piRelease,
                    INPUT  piSequence,
                    INPUT  pcDir).

                CATCH err AS Progress.Lang.Error :
                    UNDO, THROW err.
                END CATCH.
            END.

            IF plIncludeSchema THEN
            DO:
                /* In case there is a schema update table - delete it */
                deleteSchemaUpdateList(
                    INPUT  pcWorkspace,
                    INPUT  pcSiteCode,
                    OUTPUT cError).

                IF cError NE "":U THEN
                    UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

                ASSIGN
                    cMessage = "   Build deployment schema update list...":U.
                DisplayMessage(cMessage, YES, YES).

                buildDeploymentSchemaUpdateList(
                    INPUT  pcWorkspace,
                    INPUT  pcSiteCode,
                    INPUT  piSequence,
                    OUTPUT cError).

                IF cError NE "":U THEN
                    UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
            END.

            /* Make the Deployment */
            ASSIGN
                cMessage = SUBSTITUTE("   Making deployment &1 for site '&2'...":U, piSequence, pcSiteCode).
            DisplayMessage(cMessage, YES, YES).

            RTBProxyHelper:rtbMakeDeployment(
                INPUT  pcWorkspace,
                INPUT  pcSiteCode,
                INPUT  piSequence).
            ASSIGN
                cMessage = SUBSTITUTE("   Created deployment &1 for site '&2'.":U, piSequence, pcSiteCode).
            DisplayMessage(cMessage, YES, YES).

            /* If we have to create a release report for the deployment, do this now */
            IF plGenerateReport THEN
            DO:
                IF iCurrentDeploymentRelease NE piRelease THEN
                DO ON ERROR UNDO:
                    createReleaseReport (
                        INPUT  pcWorkspace,
                        INPUT  iCurrentDeploymentRelease,
                        INPUT  piRelease,
                        INPUT  SUBSTITUTE("&1/&2/&3", RtbReportBaseDir,"Deployments":U, pcWorkspace),
                        OUTPUT RtbReleaseReport).

                    ASSIGN
                        cMessage = "   Deployment release report created and written to file : ":U + RtbReleaseReport.
                    DisplayMessage(cMessage, YES, YES).

                    CATCH err AS Progress.Lang.Error :
                        IF DebugOn THEN
                            DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).

                        UNDO, THROW err.
                    END CATCH.
                END.

            END.

            ASSIGN
                cError   = "":U
                cMessage = SUBSTITUTE("   Processing of deployment number '&1' for workspace '&2' complete!", piSequence, pcWorkspace).
            DisplayMessage(cMessage, YES, YES).

            CATCH err AS Progress.Lang.Error:
                IF DebugOn THEN
                    DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).

                UNDO, THROW err.
            END CATCH.

        END.

        RETURN.

        CATCH err AS Progress.Lang.Error:
            IF DebugOn THEN
                DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).

            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hDeployData NO-ERROR.
            DELETE OBJECT hDeploymentSiteData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Extra deployment pocessing with the option to mark teh deployment as complete after wa successfull deployment.
            Notes:   This uses the standard processDeployment method and then does the extra processing of marking the deployment as complete afterwards.
            @param - pcWorkspace
            @param - pcSiteCode
            ...
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID processDeployment(
        INPUT         pcWorkspace              AS CHARACTER,
        INPUT         pcSiteCode               AS CHARACTER,
        INPUT         piRelease                AS INTEGER,
        INPUT         pcDir                    AS CHARACTER,
        INPUT         plIncludeSchema          AS LOGICAL,
        INPUT         plGenerateReleaseReport  AS LOGICAL,
        INPUT-OUTPUT  piSequence               AS INTEGER,
        INPUT         plMarkDeploymentComplete AS LOGICAL):

        DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cError   AS CHARACTER NO-UNDO.

        processDeployment(
            INPUT         pcWorkspace,
            INPUT-OUTPUT  pcSiteCode,
            INPUT         piRelease,
            INPUT         pcDir,
            INPUT         plIncludeSchema,
            INPUT         plGenerateReleaseReport,
            INPUT-OUTPUT  piSequence).

        IF piSequence NE 0 AND
            plMarkDeploymentComplete THEN
            markRtbDeployment(
                INPUT pcWorkspace,
                INPUT pcSiteCode,
                INPUT piSequence,
                INPUT "Complete":U).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            IF DebugOn THEN
                DisplayMessage(SUBSTITUTE("   *** [Debug &1]  &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).

            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:  Check if the workspace has task required set for assign and delete events
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL rtbCheckWorkspaceTaskRequired(
        INPUT pcWorkspace AS CHARACTER   ):

        DEFINE VARIABLE hWorkspaceData AS HANDLE   NO-UNDO.
        DEFINE VARIABLE hBuf           AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lTaskRequired  AS LOGICAL  NO-UNDO INITIAL NO.
        DEFINE VARIABLE lcJsonData     AS LONGCHAR NO-UNDO.

        hWorkspaceData = RTBProxyHelper:rtbGetWorkspaceHandle(pcWorkspace).

        IF VALID-HANDLE(hWorkspaceData) THEN
        DO:
            hBuf = hWorkspaceData:DEFAULT-BUFFER-HANDLE.
            hBuf:FIND-FIRST("").
            ASSIGN
                lTaskRequired = hBuf::task-req.
        END.

        RETURN lTaskRequired.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hWorkspaceData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Get the workspace import list
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID getWorkspaceImports(
        INPUT  pcWorkspace AS CHARACTER):

        DEFINE VARIABLE hImportsdata AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry         AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf         AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cMessage     AS CHARACTER NO-UNDO.
        DEFINE VARIABLE iNumRecords  AS INTEGER   NO-UNDO.

        hImportsdata = RTBProxyHelper:rtbGetWorkspaceImportsHandle(pcWorkspace).

        IF VALID-HANDLE(hImportsdata) THEN
        DO:
            CREATE QUERY hQry.
            hBuf = hImportsdata:DEFAULT-BUFFER-HANDLE.
            hQry:SET-BUFFERS(hBuf).
            hQry:QUERY-PREPARE("FOR EACH " + hImportsData:NAME).
            hQry:QUERY-OPEN.
            hQry:GET-FIRST.

            IF DebugOn THEN
            DO:
                ASSIGN
                    cMessage = "   ** [Debug] Debug listing of objects in import table:".
                DisplayMessage(cMessage, YES, YES).
            END.

            DO WHILE NOT hQry:QUERY-OFF-END:
                ASSIGN
                    iNumRecords = iNumRecords + 1.

                IF DebugOn THEN
                    DisplayMessage(SUBSTITUTE("Object : &1 (&2)", hBuf::object, hBuf::imp-status), YES, YES).
                hQry:GET-NEXT ().
            END.

            IF iNumRecords > 0 THEN
            DO:
                IF DebugOn THEN
                    DisplayMessage(SUBSTITUTE("   ** [Debug] Import table exists (&1)", iNumRecords), ?, YES, DebugOn).
                RETURN.
            END.
            ELSE
                RETURN.
        END.
        ELSE
        DO:
            IF DebugOn THEN
                DisplayMessage("   **[Debug] No import table exists", ?, YES, DebugOn).
            RETURN.
        END.

        RETURN.

        FINALLY:
            DELETE OBJECT hImportsData NO-ERROR.
            DELETE OBJECT hQry NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
               Purpose: Perform a selective compile of the given workspace
               Notes:
                     This code is copied from the standard RTB API as we
                     need to have the name of the compile.log file after the compile.
                     This is returned from the standard API as part of a message string.
       ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID selectiveCompile(
        INPUT  pcWorkspace      AS CHARACTER,
        INPUT  piTaskNum        AS INTEGER,
        INPUT  pcModule         AS CHARACTER,
        INPUT  pcGroup          AS CHARACTER,
        INPUT  pcObjType        AS CHARACTER,
        INPUT  pcObject         AS CHARACTER,
        INPUT  plForceCompile   AS LOGICAL,
        INPUT  plListing        AS LOGICAL,
        INPUT  plXref           AS LOGICAL,
        INPUT  plIgnoreWarn     AS LOGICAL,
        INPUT  plStopOnError    AS LOGICAL,
        INPUT  plTaskRcode      AS LOGICAL,
        OUTPUT pcError          AS CHARACTER,
        OUTPUT pcCompileLog     AS CHARACTER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF NOT VALID-HANDLE(RtbCompilerHandle) THEN
            startRtbCompiler().

        IF VALID-HANDLE(RtbCompilerHandle) THEN
        DO ON ERROR UNDO:
            DisplayMessage(SUBSTITUTE("   Performing &1selective compile for workspace: &2", IF RtbAppServerConnected THEN "AppServer ":U ELSE "":U, pcWorkspace), YES, YES).

            RUN rtb_selective_compile IN RtbCompilerHandle (
                INPUT pcWorkspace,
                INPUT piTaskNum,
                INPUT pcModule,
                INPUT pcGroup,
                INPUT pcObjType,
                INPUT pcObject,
                INPUT pcModule,     /* product module, pass a blank  */
                INPUT plXref,
                INPUT plListing,
                INPUT plForceCompile,
                INPUT NO,            /* Task completion */
                INPUT plIgnoreWarn,  /* Ignore warnings */
                INPUT plStopOnError, /* Stop on Error */
                INPUT plTaskRcode).

            pcError = RETURN-VALUE.

            CATCH err AS Progress.Lang.Error :
                DisplayMessage(err:GetMessage(1)).
            END CATCH.
        END.
        ELSE
            DisplayMessage(SUBSTITUTE("Compiler handle not set. Cannot perform compile for workspace: &1", pcWorkspace), YES, YES).

        IF pcError NE "":U THEN
            IF NUM-ENTRIES(pcError,CHR(1)) > 1 THEN
            DO:
                IF ENTRY(1,pcError,CHR(1)) EQ "INFO":U THEN
                    pcError = ENTRY(2,pcError,CHR(1)).
                ELSE
                    IF ENTRY(1,pcError,CHR(1)) BEGINS "Compile Results":U THEN
                    DO:
                        ASSIGN
                            pcCompileLog = ENTRY(2,pcError,CHR(1)).
                    END.
                    ELSE
                    DO:
                        ASSIGN
                            pcCompileLog = ENTRY(2,pcError,CHR(1))
                            pcError      = SUBSTITUTE("   Warnings/Errors encountered.~n    See &1 for details.",pcCompileLog).
                    END.
            END.
            ELSE
                IF pcError = "Cancel":U THEN
                    pcError = "Selective compile cancelled.".

        IF pcError NE "":U THEN
        DO:
            IF NOT pcError BEGINS "No compiles required" THEN
                IF DebugOn THEN
                    DisplayMessage(SUBSTITUTE("   **[Debug] Errors encountered during selective compile: &1",pcError), YES, DebugOn).
                ELSE
                    IF DebugOn THEN
                        DisplayMessage(pcError, YES, DebugOn).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            DisplayMessage(err:GetMessage(1), YES, YES).
        END CATCH.

        FINALLY:
            DELETE OBJECT RtbCompilerHandle.
            ASSIGN
                RtbCompilerHandle = ?.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Set the AppServer handle in the various places where it is used
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID setAppServerHandles(  ):
        IF VALID-HANDLE(AppServerHandle) THEN
        DO:
            IF AppServerHandle NE SESSION:HANDLE THEN
            DO:
                ASSIGN
                    RtbAppServerConnected = AppServerHandle:CONNECTED ().
            END.
            ELSE
                RtbAppServerConnected = FALSE.
        END.
        ELSE
            ASSIGN
                RtbAppServerConnected = FALSE.

        IF VALID-OBJECT(RTBProxyHelper) THEN
            ASSIGN
                RTBProxyHelper:AppServerHandle = AppServerHandle.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID setLogFile(
        /*------------------------------------------------------------------------------
          Purpose:     Set the file name for the logfile
          Parameters:  <none>
          Notes:
        ------------------------------------------------------------------------------*/
        INPUT  pcLogFileName     AS CHARACTER,
        INPUT  plAppendToLogFile AS LOGICAL):

        DEFINE VARIABLE oDirectoryHelper AS appSolutions.appSolute.Utils.DirectoryHelper NO-UNDO.
        DEFINE VARIABLE cLogDirectory    AS CHARACTER                                    NO-UNDO.
        DEFINE VARIABLE cLogFile         AS CHARACTER                                    NO-UNDO.

        /* First, close the existing stream */
        closeStream().

        oDirectoryHelper = NEW appSolutions.appSolute.Utils.DirectoryHelper().

        IF pcLogFilename NE "":U THEN
        DO:
            ASSIGN
                cLogDirectory = REPLACE(cLogDirectory, "~\":U, "/":U)
                cLogDirectory = SUBSTRING(pcLogFilename, 1, R-INDEX(pcLogFileName, "/":U) - 1)
                cLogFile      = SUBSTRING(pcLogFilename, R-INDEX(pcLogFileName, "/":U) + 1) .

            /* If the directory does not exist, we may need to create it */
            IF NOT oDirectoryHelper:DirectoryExists(cLogDirectory) THEN
            DO:
                IF NOT oDirectoryHelper:IsPath(cLogDirectory) THEN
                    ASSIGN
                        cLogDirectory = SESSION:TEMP-DIRECTORY + cLogDirectory.

                IF NOT oDirectoryHelper:DirectoryExists(cLogDirectory) THEN
                    oDirectoryHelper:makeDirectory(cLogDirectory).
            END.

            IF oDirectoryHelper:DirectoryExists(cLogDirectory) THEN
            DO:
                ASSIGN
                    cLogDirectory           = oDirectoryHelper:GetFullPathName(cLogDirectory)
                    cLogFile                = SUBSTITUTE("&1/&2", cLogDirectory, cLogFile)
                    /* Set the log file name - which we will be using... */
                    THIS-OBJECT:LogFileName = cLogFile.

                /* Then re-open the log file stream */
                OpenStream(INPUT plAppendToLogFile).
            END.
        END.

        RETURN.
        CATCH err AS Progress.Lang.Error :
            DisplayMessage(err:GetMessage(1), YES, YES ).
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID setRTBPropath(
        INPUT pcRTBInstallPath AS CHARACTER ):

        ASSIGN
        PROPATH = pcRTBInstallPath + ",":U + pcRTBInstallPath + "/rtb/proxy/p," + PROPATH.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID setRTBWorkspaceID(
        INPUT  pcWorkspaceID AS CHARACTER):

        DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.

        ASSIGN
            RtbWorkspaceId = pcWorkspaceID.

        ASSIGN
            cMessage = SUBSTITUTE("   Setting workspace in RTB session to '&1'...", pcWorkspaceID).
        DisplayMessage(cMessage, TRUE, TRUE).

        /* Also set the session property to the correct workspace */
        RtbProxyHelper:rtbSetSessionProperty("workspace",pcWorkspaceID).

        IF VALID-HANDLE(RTBAPI) THEN
            RUN set_workspace_id IN RTBAPI (pcWorkspaceID).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC VOID startAPIs(
        INPUT  pcRTBInstallPath  AS CHARACTER,
        INPUT  pcWorkspaceID     AS CHARACTER ,
        INPUT  pcUser            AS CHARACTER,
        INPUT  pcPassword        AS CHARACTER,
        INPUT  pcLogFile         AS CHARACTER,
        OUTPUT phRTBApi          AS HANDLE,
        OUTPUT piSessionID       AS INTEGER):

        DEFINE VARIABLE cError   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.

        setLogFile(pcLogFile, NO).

        /* Set basic properties for this RTB API Session */
        ASSIGN
            RTBInstallPath  = pcRTBInstallPath
            RTBUser         = pcUser
            RTBUserPassword = pcPassword.

        setRTBPropath(pcRTBInstallPath).

        startRTBAPI(
            INPUT  pcUser,
            INPUT  pcPassword,
            OUTPUT phRTBApi,
            OUTPUT piSessionID).

        ASSIGN
            RTBSessionID    = piSessionID
            RTBUser         = pcUser
            RTBUserPassword = pcPassword
            RtbWorkspaceId  = pcWorkspaceID.

        RTBproxyHelper:rtbSetSessionProperty("sid", STRING(RTBSessionID)).

        /* Add the session ID to the output so that we can see this is being set properly */
        ASSIGN
            cMessage = SUBSTITUTE("Mode: &1 (&2)...", IF RtbAppServerConnected THEN "AppServer":U ELSE "Client only", piSessionID).
        DisplayMessage(cMessage, TRUE, TRUE).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            IF DebugOn THEN
                DisplayMessage(SUBSTITUTE("   ** [Debug &1] &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).

            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID startRTBAPI(
        INPUT  pcRTBUser   AS CHARACTER,
        INPUT  pcPassword  AS CHARACTER,
        OUTPUT phRTBApi    AS HANDLE,
        OUTPUT piSessionID AS INTEGER):

        DEFINE VARIABLE cMessage    AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE iSessionID  AS INTEGER                                 NO-UNDO.
        DEFINE VARIABLE cError      AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE oFileHelper AS appSolutions.appSolute.Utils.FileHelper NO-UNDO.

        ASSIGN
            cMessage = SUBSTITUTE("Starting Roundtable API for user : &1...", pcRTBUser).
        DisplayMessage(cMessage, TRUE, TRUE).

        oFileHelper = NEW appSolutions.appSolute.Utils.FileHelper().

        IF DebugOn THEN
        DO:
            ASSIGN
                cMessage = SUBSTITUTE("PROPATH: &1", PROPATH).
            DisplayMessage(cMessage, TRUE, DebugOn).
        END.

        /*IF oFileHelper:FileExists("rtb/p/rtb_api.p") THEN
            RUN rtb/p/rtb_api.p PERSISTENT SET phRTBApi.*/

        DO ON ERROR UNDO, RETRY
            ON STOP UNDO, RETRY:
            RUN rtb/p/rtb_api.p PERSISTENT SET phRTBApi.
            CATCH e AS Progress.Lang.Error:
                UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("*** Unable to start Roundtable API! &1", e:getMessage(1)), 0).
            END CATCH.
        END.

        IF NOT VALID-HANDLE(phRTBApi) THEN
            UNDO, THROW NEW Progress.Lang.AppError("*** Unable to start Roundtable API!", 0).
        ELSE
            ASSIGN
                RTBAPI = phRTBApi.

        IF AppServerHandle NE SESSION:HANDLE THEN
        DO ON ERROR UNDO, THROW:
            ASSIGN
                cMessage = "Logging into Roundtable via AppServer...".

            piSessionID = RTBProxyHelper:rtbSessionLogin(pcRTBUser,pcPassword).

            IF piSessionID NE 0 AND
                VALID-HANDLE(RTBAPI) THEN
                RUN set_session_id IN RTBAPI (piSessionID).

            CATCH err AS Progress.Lang.Error :
                UNDO, THROW err.
            END CATCH.
        END.
        ELSE
        DO:
            ASSIGN
                cMessage = "Logging into Roundtable API...".
            DisplayMessage(cMessage, TRUE, TRUE).

            RUN login IN RTBAPI (
                INPUT  pcRTBUser,
                INPUT  pcPassword,
                OUTPUT piSessionID,
                OUTPUT cError).
        END.

        IF cError NE "":U THEN
            UNDO, THROW NEW Progress.Lang.AppError ("Roundtable session login error : " + cError, 0).
        ELSE
        DO:
            ASSIGN
                THIS-OBJECT:RTBSessionID = piSessionID.

            rtbCreateClientSideTempDir().

            /* Add the session ID to the output so that we can see this is being set properly */
            ASSIGN
                cMessage = SUBSTITUTE("Session ID : &1", piSessionID).
            DisplayMessage(cMessage, TRUE, TRUE).
        END.

        IF THIS-OBJECT:RTBInstallPath NE "":U THEN
        DO:
            ASSIGN
                cMessage = "Setting paths in RTB API...".
            DisplayMessage(cMessage, TRUE, TRUE).

            RUN set_paths IN RTBAPI (
                INPUT THIS-OBJECT:RTBInstallPath,
                INPUT PROPATH,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Unable to set path in Roundtable API: &1", cError), 0).
        END.

        IF RtbWorkspaceId NE "":U THEN
            setRTBWorkspaceID(RtbWorkspaceId).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Start the custom RTB compiler (AppServer enabled and with more output thban the standard)
                     or the standard RTB compile procedure and make this accessiable as property to this class
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID startRtbCompiler(  ):
        DEFINE VARIABLE cError      AS CHARACTER                               NO-UNDO.
        DEFINE VARIABLE hCompiler   AS HANDLE                                  NO-UNDO.
        DEFINE VARIABLE oFileHelper AS appSolutions.appSolute.Utils.FileHelper NO-UNDO.

        oFileHelper = NEW appSolutions.appSolute.Utils.FileHelper().

        /* Try and start a custom verison of the compiler. If this does not exist, then the error status will be set.  */
        IF oFileHelper:FileExists("scm/custom/rtb/p/rtbcust_compiler.p") THEN
        DO:
            DisplayMessage(SUBSTITUTE("   Starting custom selective compiler: ~n    &1", oFileHelper:GetFullPathName("scm/custom/rtb/p/rtbcust_compiler.p")), YES, YES).
            RUN scm/custom/rtb/p/rtbcust_compiler.p PERSISTENT SET hCompiler (INPUT AppServerHandle) NO-ERROR.
        END.
        ELSE
            RUN rtb/p/rtb_compiler.p PERSISTENT SET hCompiler NO-ERROR.

        IF VALID-HANDLE(hCompiler) THEN
            ASSIGN
                RTbCompilerHandle = hCompiler.

        /* Also set the session property to the correct workspace */
        RtbProxyHelper:rtbSetSessionProperty("workspace", RtbWorkspaceId).

        /* NB: Set the session ID - as this seems to have gone in the local session at this time!  */
        DYNAMIC-FUNCTION('fnRtbSetSessionProperty':U,'sid':U, STRING(RTBSessionID)).

        RETURN.

        CATCH Err AS Progress.Lang.Error :
            ASSIGN
                cError = SUBSTITUTE("*** &1", err:GetMessage(1)).
            DisplayMessage(cError, YES, YES ).

            UNDO, THROW NEW Progress.Lang.AppError(Err:GetMessage(1), 0).
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID stopAPIs(  ):
        DEFINE VARIABLE cError   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cMessage AS CHARACTER NO-UNDO.

        IF RtbAppServerConnected THEN
        DO:
            ASSIGN
                cMessage = "Logging out of RTB AppServer...".
            DisplayMessage(cMessage, TRUE, TRUE).

            IF RTBSessionID NE 0 THEN
                RTBProxyHelper:rtbSessionLogout (RTBSessionID).

            AppServerHandle:DISCONNECT().
            ASSIGN
                RtbAppServerConnected          = FALSE
                AppServerHandle                = ?
                RTBProxyHelper:AppServerHandle = ?
                RTBSessionID                   = 0.
        END.

        IF VALID-HANDLE(RTBAPI) THEN
        DO:
            ASSIGN
                cMessage = "Logging out of RTB API...".
            DisplayMessage(cMessage, TRUE, TRUE).

            IF RTBSessionID NE 0 THEN
            DO:
                RUN logout IN RTBAPI (
                    INPUT  RTBSessionID,
                    OUTPUT cError).

                IF cError NE "":U THEN
                DO:
                    DisplayMessage(cError, TRUE, TRUE).
                    RETURN.
                END.
                ELSE
                    ASSIGN
                        RTBSessionID = 0.
            END.

            /* make sure we clean up after ourselves... */
            IF NOT DebugOn THEN
                rtbDeleteClientSideTempDir().

            DELETE PROCEDURE THIS-OBJECT:RTBAPI NO-ERROR.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC CHARACTER validateCompletedTasks(
        INPUT  pcTaskList AS CHARACTER ,
        OUTPUT pcError    AS CHARACTER ):

        DEFINE VARIABLE i              AS INTEGER   NO-UNDO.
        DEFINE VARIABLE iTaskNum       AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cValidTaskList AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hTaskData      AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hTaskBuf       AS HANDLE    NO-UNDO.

        IF pcTasKLIST NE "":U THEN
        DO:
            DO i = 1 TO NUM-ENTRIES(pcTaskList):
                ASSIGN
                    iTaskNum = INTEGER(ENTRY(i, pcTaskList)).

                hTaskData = RTBProxyHelper:rtbGetTaskHandle(iTaskNum).
                hTaskBuf = hTaskData:DEFAULT-BUFFER-HANDLE.
                hTaskBuf:FIND-FIRST().

                IF hTaskBuf:AVAILABLE THEN
                DO:
                    IF hTaskBuf::task-status = "C":U THEN
                        ASSIGN
                            cValidTaskList = cValidTaskList + (IF cValidTaskList NE "":U THEN ",":U ELSE "":U) + STRING(iTaskNum).
                    ELSE
                        ASSIGN
                            pcError = pcError + (IF pcError NE "":U THEN "~n":U ELSE "":U) + SUBSTITUTE("*** Task number '&1' is not completed.", iTaskNum).
                END.
                ELSE
                    ASSIGN
                        pcError = pcError + (IF pcError NE "":U THEN "~n":U ELSE "":U) + SUBSTITUTE("*** Invalid task number '&1'!", iTaskNum).
            END.
        END.

        RETURN cValidTaskList.

        FINALLY:
            DELETE OBJECT hTaskData NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID validateParamCount(
        INPUT  pcParams          AS LONGCHAR,
        INPUT  piParamsExpected  AS INTEGER,
        INPUT  pcDelimiter       AS CHARACTER):

        IF NUM-ENTRIES(pcParams, pcDelimiter) NE piParamsExpected THEN
            UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("Invalid number of parameters passed (&1/&2). Cannot continue!", NUM-ENTRIES(pcParams, pcDelimiter), piParamsExpected), 0).
        ELSE
            RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID validateParamCount(
        INPUT  piParamsReceived  AS INTEGER,
        INPUT  piParamsExpected  AS INTEGER):

        IF piParamsReceived NE piParamsExpected THEN
            UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("Invalid number of parameters passed (&1/&2). Cannot continue!", piParamsReceived, piParamsExpected), 0).
        ELSE
            RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL ValidWorkspace(
        INPUT pcWorkspace AS CHARACTER ):

        RETURN RTBProxyHelper:rtbWorkspaceExists(pcWorkspace).

    END METHOD.

    DESTRUCTOR PUBLIC RTBAutomationApi ( ):
        IF SEEK(sLogStream) <> ? THEN
            closeStream().

        IF VALID-HANDLE(RtbCompilerHandle) THEN
        DO:
            IF RtbAppServerConnected THEN
            DO:
                DELETE OBJECT RtbCompilerHandle.
                ASSIGN
                    RtbCompilerHandle = ?.
            END.
            ELSE
                DYNAMIC-FUNCTION("fnRtbEndSuperProc", SESSION, RtbCompilerHandle, YES) NO-ERROR.
        END.

        IF RtbAppServerConnected AND
            VALID-HANDLE(AppServerHandle) THEN
        DO:
            IF AppServerHandle:CONNECTED() THEN
                AppServerHandle:DISCONNECT ().
            ASSIGN
                AppServerHandle = ?.
        END.

        IF VALID-HANDLE(RtbLoadWriteHandle) THEN
        DO:
            DYNAMIC-FUNCTION("fnRtbEndSuperProc", SESSION, RtbLoadWriteHandle, YES).

            ASSIGN
                RtbLoadWriteHandle = ?.
        END.

        DELETE OBJECT RTBProxyHelper NO-ERROR.
    END DESTRUCTOR.

END CLASS.
