/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : appSolutions/appSolute/Utils/DirectoryHelper.cls
    Purpose     :
    Syntax      :
    Description :
    Author(s)   : Thomas Hansen / appSolutions
    Created     : Fri Aug 16 10:27:44 CEST 2013
    Notes       :

   History     :
                Version 01.00.00    08-2013    Thomas Hansen
                Initial version for toolkit created
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS appSolutions.appSolute.Utils.DirectoryHelper:

    /*------------------------------------------------------------------------------
     Purpose: Check if the passe din directory should be considered as a full path
     Notes: This checks to see if the passed in directory (which does not HAVE to exist) contains ":", of begins with "//" or "\\".
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL IsPath(
        INPUT pcDirectoryName AS CHARACTER ):

        ASSIGN
            pcDirectoryName = REPLACE(pcDirectoryName, "~\":U, "/").

        IF (INDEX(pcDirectoryName, ":":U) NE 0 OR
            pcDirectoryName BEGINS "//":U OR
            pcDirectoryName BEGINS "/":U OR
            pcDirectoryName BEGINS "./":U OR
            pcDirectoryName BEGINS "../":U ) AND
                    INDEX(pcDirectoryName, ".":U) = 0 THEN
            RETURN TRUE.
        ELSE
            RETURN FALSE.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:  Check if the passed in directory exists or not.
                      If the directory does not exist, create it if this option is selected.
            Notes:
            @param  - Input of the directory to check
            @param  - Option logical parameter to create the directory if it does not exist
            @param  - Output of errors thta may occur during processing
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL validateAndCreateIfMissing(
        INPUT  pcDirname   AS CHARACTER,
        INPUT  plCreateDir AS LOGICAL):

        DEFINE VARIABLE lDirStatus AS LOGICAL   NO-UNDO INITIAL FALSE.
        DEFINE VARIABLE iStat      AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cError     AS CHARACTER NO-UNDO.

        IF pcDirname = "":U OR
            pcDirname = ? THEN
        DO:
            ASSIGN
                cError = "You must supply a directory name! Try again.".
            UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("Directory validation error: '&1'", cError), 0).
        END.

        FILE-INFO:FILE-NAME = pcDirname.

        IF FILE-INFO:FULL-PATHNAME = ? THEN
        DO:
            IF plCreateDir THEN
            DO ON ERROR UNDO:
                makeDirectory(pcDirname).

                ASSIGN
                    lDirStatus = TRUE.

                CATCH err AS Progress.Lang.Error :
                    ASSIGN
                        cError     = SUBSTITUTE("Directory not created. System Error #: &1", iStat)
                        lDirStatus = FALSE.
                    UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("Directory validation error: '&1'", cError), 0).
                END CATCH.
            END.
            ELSE
                lDirStatus = FALSE.
        END.
        ELSE
            lDirStatus = TRUE.

        RETURN lDirStatus.

    END METHOD.

    /*------------------------------------------------------------------------------
        Purpose: Verifies that the directory exists
        Notes:
        @param pcFileName The file name to verify
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL DirectoryExists(
        INPUT pcDirectoryName AS CHARACTER ):

        DEFINE VARIABLE lExists AS LOGICAL NO-UNDO.

        FILE-INFO:FILE-NAME = pcDirectoryName .
        IF FILE-INFO:FULL-PATHNAME NE ? AND
            FILE-INFO:FILE-TYPE BEGINS "D":U THEN
            ASSIGN
                lExists = TRUE.
        ELSE
            lExists = FALSE.

        RETURN lExists.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Returns the absolute file name for the given file
            Notes:
            @param pcFileName The file name to search
            @return The absolute path to the file or ? when the file is not found
        ------------------------------------------------------------------------------*/
    METHOD PUBLIC CHARACTER GetFullPathName (pcDirectoryName AS CHARACTER):

        FILE-INFO:FILE-NAME = pcDirectoryName .
        IF FILE-INFO:FULL-PATHNAME NE ? AND
            FILE-INFO:FILE-TYPE BEGINS "D":U THEN
            RETURN FILE-INFO:FULL-PATHNAME .
        ELSE
            RETURN "":U.

    END METHOD.

    /* -----------------------------------------------------------
      Purpose: To create a directory or series of directories.
      Parameters:  <none>
      Notes:   Code taken from _update.w from Roundtable.

              Thomas 15/04/2014: Code has been chanbged to throw an error now instead of returning pcError.

      @param  - The full path of the directory to create

    -------------------------------------------------------------*/
    METHOD PUBLIC VOID makeDirectory(
        INPUT  pcDirectory AS CHARACTER):

        DEFINE VARIABLE lRrelative   AS LOGICAL   NO-UNDO.
        DEFINE VARIABLE Mi           AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cPath        AS CHARACTER NO-UNDO.
        DEFINE VARIABLE iErrorStatus AS INTEGER   NO-UNDO.

        ASSIGN
            pcDirectory = REPLACE(pcDirectory,"~\","~/")
            /* Make sure we don't have // at the end */
            pcDirectory = RIGHT-TRIM(pcDirectory,"//":U).

        IF pcDirectory = ? OR
            pcDirectory = "" THEN
            UNDO, THROW NEW AppError("Directory cannot be blank or unknown",0).

        FILE-INFO:FILE-NAME = pcDirectory.
        IF FILE-INFO:FULL-PATHNAME NE ? THEN
            IF NOT FILE-INFO:FILE-TYPE BEGINS "D" THEN
                UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Bad directory name: &1", pcDirectory), 0).

        IF SUBSTRING(pcDirectory,2,1) = ":" THEN
            ASSIGN
                cPath = ENTRY(1,pcDirectory,"~/")
                Mi    = 2.
        ELSE
            ASSIGN
                cPath = "/"
                Mi    = 1.

        DO WHILE Mi <= NUM-ENTRIES(pcDirectory,"~/"):
            ASSIGN
                cPath = cPath +
              (IF cPath = "" OR cPath = "/" THEN
                  ""
               ELSE
                  "~/") +
              ENTRY(Mi,pcDirectory,"~/").

            FILE-INFO:FILE-NAME = cPath.
            IF FILE-INFO:FULL-PATHNAME = ? THEN
            DO:
                OS-CREATE-DIR VALUE(cPath) NO-ERROR.
                iErrorStatus = OS-ERROR.
                IF iErrorStatus <> 0 THEN
                    UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Error! OS error # &1 while creating directory", iErrorStatus), 0).
            END.
            ELSE
            DO:
                IF NOT FILE-INFO:FILE-TYPE BEGINS "D" THEN
                    UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("Error! directory name: &1", cPath), 0).
            END.

            Mi = Mi + 1.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /* --------------------------------------------------------------------
      Purpose:     Simulate the xcopy function
      Parameters:  Source Directory, Target Directory
      Notes:
      @param  - The full path of the source directory to copy
      @param  - The full path of the target directory to create
      @param  - Options to use when processing.
                "S" is similar to the Windows OS-Command option to make the process recursive.

       -------------------------------------------------------------------- */
    METHOD PUBLIC VOID Xcopy(
        INPUT pcSource AS CHARACTER,
        INPUT pcTarget AS CHARACTER,
        INPUT pcOption AS CHARACTER):

        DEFINE VARIABLE cSubDirName AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cSubDirPath AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cFilename   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cFullpath   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cFileType   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE i           AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cError      AS CHARACTER NO-UNDO.

        IF pcSource = pcTarget THEN
            RETURN.

        IF pcTarget MATCHES "*~/":U THEN
            makeDirectory(pcTarget).
        ELSE
            makeDirectory(pcTarget + "~/":U).

        INPUT FROM OS-DIR (pcSource) NO-ECHO.
        REPEAT:
            IMPORT cFilename cFullPath cFileType.
            IF NOT cFilename BEGINS ".":U THEN
            DO:
                IF cFileType BEGINS "D":U THEN
                    ASSIGN
                        cSubDirName = cSubDirName + cFilename + ",":U
                        cSubDirPath = cSubDirPath + cFullPath + ",":U.
                ELSE
                DO:
                    OS-COPY VALUE(cFullPath) VALUE(pcTarget + "~/":U + cFilename).
                END.
            END.
        END.

        INPUT CLOSE.

        IF pcOption = "S":U THEN
        DO i = 1 TO NUM-ENTRIES(cSubDirName) - 1 :
            Xcopy(
                INPUT ENTRY(i, cSubDirPath),
                INPUT pcTarget + "~/":U + ENTRY(i, cSubDirName),
                INPUT "S":U).
        END.

        RETURN.

        CATCH err AS System.Exception :
            UNDO, THROW NEW AppError(err:GetMessage(1),0).
        END CATCH.
    END METHOD.

END CLASS.
