/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : appSolute/RTB/Fileutil.cls
    Purpose     :
    Syntax      :
    Description :
    Author(s)   : Thomas Hansen / appSolutions
    Created     : Fri Aug 16 10:27:44 CEST 2013
    Notes       :

   History     :
                Version 01.00.00    08-2013    Thomas Hansen
                Initial version for toolkit created
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS appSolutions.appSolute.Utils.FileHelper:

    /*------------------------------------------------------------------------------
     Purpose: Determine the file type by parsing the AppBuilder generated procedure code
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID DetermineABFileType(
        INPUT  pcFileName      AS CHARACTER,
        INPUT  pcSearchString  AS CHARACTER,
        OUTPUT pcFileType      AS CHARACTER):

        DEFINE VARIABLE EdReadFile AS CHARACTER VIEW-AS EDITOR SCROLLBAR-VERTICAL LARGE SIZE 78 BY 11 NO-UNDO.
        DEFINE VARIABLE lcFile     AS LONGCHAR  NO-UNDO.

        IF THIS-OBJECT:FileExists(pcFileName) THEN
        DO WITH FRAME a:
            ENABLE EdReadFile.
            EdReadFile:HIDDEN = TRUE.

            EdReadFile:READ-FILE(pcFileName ).
            IF edReadFile:SEARCH("Settings for THIS-PROCEDURE", 32) THEN
                IF edReadFile:SEARCH(pcSearchString, 32) THEN
                DO:
                    edReadFile:SET-SELECTION(edReadFile:SELECTION-START, edReadFile:SELECTION-END + 15).
                    ASSIGN
                        pcFileType = TRIM(ENTRY(2, edReadFile:SELECTION-TEXT, " ":U)).

                    /***DIAG***/
                    MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
                        edReadFile:SELECTION-TEXT
                        VIEW-AS ALERT-BOX INFO BUTTONS OK.
                END.
        END.
        ELSE
            UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("File '&1' does not exist. Cannot determine AB file type.", pcFileName), 0).

        RETURN.
    END METHOD.

    /*------------------------------------------------------------------------------
        Purpose: Verifies that the file exists
        Notes:
        @param pcFileName The file name to verify
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL FileExists(
        INPUT pcFileName AS CHARACTER ):

        DEFINE VARIABLE lFileExists AS LOGICAL NO-UNDO.

        FILE-INFO:FILE-NAME = pcFileName .
        IF FILE-INFO:FULL-PATHNAME = ? THEN
            ASSIGN
                lFileExists = FALSE.
        ELSE
            lFileExists = TRUE.

        RETURN lFileExists.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Get the base file name from a passed in file name
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER getBaseFilename(
        INPUT pcFullPathName AS CHARACTER  ):

        DEFINE VARIABLE cFilename     AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cBaseFilename AS CHARACTER NO-UNDO.

        IF FileExists(pcFullPathName) THEN
        DO:
            ASSIGN
                cFilename = GetFullPathName(cFilename)
                cFilename = REPLACE(pcFullPathName, "~\":U, "/":U).
            IF NUM-ENTRIES(cFilename, "/":U) NE 0 THEN
            DO:
                ASSIGN
                    cBaseFilename = SUBSTRING(cFilename, R-INDEX(cFilename, "/":U) + 1).
            END.
        END.

        RETURN cBaseFilename.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Read the contents of a file from the file system into a longchar return value.
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR getFileContents(
        INPUT pcFileName AS CHARACTER ):

        DEFINE VARIABLE lcFileContents AS LONGCHAR NO-UNDO.

        IF pcFileName NE "":U THEN
        DO:
            IF FileExists(pcFileName) THEN
            DO:
                pcFileName = GetFullPathName(pcFileName).
                COPY-LOB FROM FILE pcFileName TO lcFileContents.
            END.
        END.

        RETURN lcFileContents.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Get the directory name for the passed in file
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER getFileDirectory(
        INPUT pcFullPathName AS CHARACTER  ):

        DEFINE VARIABLE cFileDirectory AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cFilename      AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cBaseFilename  AS CHARACTER NO-UNDO.

        IF FileExists(pcFullPathName) THEN
        DO:
            ASSIGN
                cFilename = GetFullPathName(cFilename)
                cFilename = REPLACE(pcFullPathName, "~\":U, "/":U).
            IF NUM-ENTRIES(cFilename, "/":U) NE 0 THEN
            DO:
                ASSIGN
                    cBaseFilename  = SUBSTRING(cFilename, R-INDEX(cFilename, "/":U) + 1)
                    cFileDirectory = SUBSTRING(cFilename, 1, R-INDEX(cFilename, "/":U)).
            END.
        END.

        RETURN cFileDirectory.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Returns the absolute file name for the given file
            Notes:
            @param pcFileName The file name to search
            @return The absolute path to the file or ? when the file is not found
        ------------------------------------------------------------------------------*/
    METHOD PUBLIC CHARACTER getFullPathName (pcFileName AS CHARACTER):

        FILE-INFO:FILE-NAME = pcFileName .
        RETURN FILE-INFO:FULL-PATHNAME .

    END METHOD.

    /*------------------------------------------------------------------------------
        Purpose: Returns the extension of a given file
        Notes:
        @param pcFileName The file name to search
        @return The file extension
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC CHARACTER GetExtension(pcFileName AS CHARACTER  ):
        DEFINE VARIABLE cExtension AS CHARACTER NO-UNDO.

        IF INDEX(pcFileName, ".":U) > 0 THEN
            cExtension = SUBSTRING(pcFileName, R-INDEX(pcFileName, ".") + 1).

        RETURN cExtension.

    END METHOD.

END CLASS.
