/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : appSolute/RTB/ListUtil.cls
    Purpose     :
    Syntax      :
    Description :
    Author(s)   : Thomas Hansen / appSolutions
    Created     : Fri Aug 16 10:27:44 CEST 2013
    Notes       :

   History     :
                Version 01.00.00    08-2013    Thomas Hansen
                Initial version for toolkit created
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS appSolutions.appSolute.Utils.ListHelper:

    /*------------------------------------------------------------------------------
            Purpose: Get a line separated list from a file and return it as a comma separated list
            Notes:
            @param   pcFileName - The name of the input file containing a list with each item on a separate line.
            @param   pcError    - Output of errors that may occur during processing.
                                  (This should really be replaced with a thrown error instead.
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER getListFromFile(
        INPUT  pcFileName AS CHARACTER):

        DEFINE VARIABLE cFullPathName AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cList         AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cLine         AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cError        AS CHARACTER NO-UNDO.

        ASSIGN
            pcFileName = REPLACE(pcFileName, '"':U, "":U).

        IF pcFileName = "":U OR
            pcFileName = ? THEN
        DO:
            ASSIGN
                cError = "You must supply a directory name! Try again.".
            UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("Directory validation error: '&1'", cError), 0).
        END.

        FILE-INFO:FILE-NAME = pcFileName.
        IF FILE-INFO:FULL-PATHNAME NE ? THEN
        DO:
            ASSIGN
                cFullPathName = FILE-INFO:FULL-PATHNAME.
            INPUT FROM value(cFullPathName).

            REPEAT TRANSACTION ON ERROR UNDO :
                ASSIGN
                    cLine = "":U.
                IMPORT cLine.
                IF TRIM(cLine) NE "":U AND
                    NOT TRIM(cLine) BEGINS "#":U THEN
                    ASSIGN
                        cList = cList + ",":U + TRIM(cLine).
            END.
            INPUT CLOSE.

            /* make sure we do not have starting or ending commas in the list */
            ASSIGN
                cList = TRIM(cList, ",":U).
        END.
        ELSE
            UNDO, THROW NEW Progress.Lang.AppError (SUBSTITUTE("Error: File '&1' not found", pcFileName), 0).

        RETURN cList.

    END METHOD.

END CLASS.
