/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
   File        : LogHelper
   Purpose     :
   Syntax      :
   Description :
   Author(s)   : thomas
   Created     : Fri Apr 19 07:04:50 CEST 2013
   Notes       :
 ----------------------------------------------------------------------*/

USING appSolutions.appSolute.LoadNGo.Utils.* FROM PROPATH.
USING Progress.Lang.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS appSolutions.appSolute.Utils.LogHelper:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE hCallingProcedure AS HANDLE NO-UNDO.

    DEFINE STREAM sLogStream.

    DEFINE PUBLIC STATIC PROPERTY CallingProcedure AS HANDLE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC STATIC PROPERTY LogFileName AS CHARACTER NO-UNDO
        GET():
            RETURN LOG-MANAGER:LOGFILE-NAME .

            CATCH e AS Progress.Lang.Error :
                RETURN ? .
            END CATCH.
        END GET.

        SET(INPUT arg AS CHARACTER):

            LOG-MANAGER:LOGFILE-NAME = REPLACE(arg, "/":U, "\":U) .
            LOG-MANAGER:LOG-ENTRY-TYPES = ? .

            CATCH err AS Progress.Lang.Error :
                /***DIAG***/
                MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
                    err:GetMessage(1)
                    VIEW-AS ALERT-BOX INFO BUTTONS OK.

            END CATCH.

        /* no catch block here. Error should be catched by caller */
        END SET.

    CONSTRUCTOR PROTECTED LogHelper (  ):
        SUPER ().

    END CONSTRUCTOR.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL ClearLogFile () :
        IF LOG-MANAGER:LOGFILE-NAME <> ? THEN
        DO:
            LOG-MANAGER:CLEAR-LOG () .
            RETURN TRUE.
        END.

        RETURN FALSE.

        CATCH e AS Progress.Lang.Error :

            RETURN FALSE.
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC STATIC VOID CloseLogStream(  ):
        IF SEEK(sLogStream) <> ?  THEN
            OUTPUT STREAM sLogStream CLOSE.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC STATIC VOID DeleteLogFile(  ):

        DEFINE VARIABLE result AS LOGICAL NO-UNDO.

        IF LogFileName NE "":U THEN
            OS-DELETE VALUE(LogFileName).

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC STATIC VOID DisplayEditorMessage(
        pcMessage   AS CHARACTER,
        phEditor    AS HANDLE,   /* Should output go to the editor ? */
        plLogFile   AS LOGICAL,  /* Should output go to the log file ?  */
        plTimeStamp AS LOGICAL ):

        /*------------------------------------------------------------------------------
          Purpose:  Standard display of messages & errors to the editor widget and log file.
            Notes:
        ------------------------------------------------------------------------------*/
        IF pcMessage NE "":U THEN
        DO:
            IF plTimeStamp THEN
                ASSIGN
                    pcMessage = SUBSTITUTE("&1 &2: &3", STRING(TODAY), STRING(TIME, "HH:MM:SS":U), pcMessage).

            IF VALID-HANDLE(phEditor) THEN
                phEditor:INSERT-STRING(pcMessage + CHR(10)).

            IF plLogFile THEN
                writeMessage(pcMessage).
        /*            IF SEEK(sLogStream) = ?  THEN                            */
        /*                OpenStream(YES).                                     */
        /*                                                                     */
        /*            IF SEEK(sLogStream) NE ?  THEN                           */
        /*            DO:                                                      */
        /*                IF plLogFile THEN                                    */
        /*                    PUT STREAM sLogStream UNFORMATTED pcMessage SKIP.*/
        /*            END.                                                     */
        /*            ELSE                                                     */
        /*                MESSAGE "Log stream closed."                         */
        /*                    VIEW-AS ALERT-BOX INFORMATION BUTTONS OK.        */
        END.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC STATIC VOID DisplayMessage(
        pcMessage    AS CHARACTER,
        plLogFile    AS LOGICAL  /* Should output go to the log file ?  */,
        plMessageBox AS LOGICAL,
        plTimeStamp  AS LOGICAL ) :
        /*------------------------------------------------------------------------------
          Purpose:  Standard display of messages & errors to the editor widget and log file.
            Notes:
        ------------------------------------------------------------------------------*/
        IF pcMessage NE "":U THEN
        DO:
            IF plTimeStamp THEN
                ASSIGN
                    pcMessage = SUBSTITUTE("&1 &2: &3", STRING(TODAY), STRING(TIME, "HH:MM:SS":U), pcMessage).

            IF plLogFile THEN
                WriteMessage(pcMessage).

            IF plMessageBox THEN
                MESSAGE
                    pcMessage
                    VIEW-AS ALERT-BOX.
        END.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
                Purpose:
                Notes:
        ------------------------------------------------------------------------------*/

    METHOD PUBLIC STATIC VOID DisplayMessage(
        pcMessage    AS CHARACTER,
        plLogFile    AS LOGICAL  /* Should output go to the log file ?  */,
        plMessageBox AS LOGICAL) :

        /***DIAG***/
        MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
            "plMessageBox : " plMessageBox SKIP
            "SESSION:BATCH " SESSION:BATCH
            VIEW-AS ALERT-BOX INFO BUTTONS OK.

        IF pcMessage NE "":U THEN
        DO:
            /* If we ARE in a batch sesson of some sort, then display the message.
               On a non-batch session, we end up with a huge number of displayed messages that we don't want */
            IF plMessageBox AND SESSION:BATCH THEN
            DO:
                /***DIAG***/
                MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
                    "Here in log... "
                    VIEW-AS ALERT-BOX INFO BUTTONS OK.

                MESSAGE
                    pcMessage
                    VIEW-AS ALERT-BOX.
            END.

            IF plLogFile THEN
                WriteMessage(pcMessage).
        END.

        RETURN.

    END METHOD.
    /*------------------------------------------------------------------------------
        Purpose: Returns if currently a logfile is open.
        Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL LogFileIsOpen () :
        RETURN LOG-MANAGER:LOGFILE-NAME <> ? .

        CATCH e AS Progress.Lang.Error :
            RETURN FALSE .
        END CATCH.
    END.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC STATIC LOGICAL OpenStream(
        plAppend AS LOGICAL /* parameter-definitions */ ) :
        /*------------------------------------------------------------------------------
          Purpose:
            Notes:
        ------------------------------------------------------------------------------*/
        IF plAppend THEN
        DO:
            IF LogFileName NE "":U THEN
                OUTPUT STREAM sLogStream TO VALUE(LogFileName) APPEND.
            ELSE
            DO:
                ASSIGN
                    LogFileName = "ARAT_Log.log".
                OUTPUT STREAM sLogStream TO VALUE(LogFileName) APPEND.
            END.
        END.
        ELSE
        DO:
            IF LogFileName NE "":U THEN
                OUTPUT STREAM sLogStream TO VALUE(LogFileName).
            ELSE
                OUTPUT STREAM sLogStream TO VALUE("ARAT_Log.log").
        END.

        IF SEEK(sLogStream) = ?  THEN
            RETURN FALSE.
        ELSE
            RETURN TRUE.   /* Function return value. */

    END METHOD.

    /*------------------------------------------------------------------------------
        Purpose: Writes a character message to the client logile and the default
                 logging sub system.
        Notes:
        @param pcMessage A character message to write to the log file
        @retun Logical value indicating if the message has been written to the log file
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL WriteMessage (pcMessage AS CHARACTER):

        IF LOG-MANAGER:LOGFILE-NAME <> ? OR SESSION:REMOTE OR SESSION:CLIENT-TYPE = "WEBSPEED":U THEN
            RETURN LOG-MANAGER:WRITE-MESSAGE (pcMessage) .

        RETURN FALSE .
    END.

    /*------------------------------------------------------------------------------
        Purpose: Writes an date message to the client logile and the default
                 logging sub system.
        Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL WriteMessage (pdaMessage AS DATE):

        IF LOG-MANAGER:LOGFILE-NAME <> ? OR SESSION:REMOTE OR SESSION:CLIENT-TYPE = "WEBSPEED":U THEN
            RETURN LOG-MANAGER:WRITE-MESSAGE (STRING (pdaMessage)) .

        RETURN FALSE .
    END.

    /*------------------------------------------------------------------------------
        Purpose: Writes an datetime message to the client logile and the default
                 logging sub system.
        Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL WriteMessage (pdaMessage AS DATETIME):

        IF LOG-MANAGER:LOGFILE-NAME <> ? OR SESSION:REMOTE OR SESSION:CLIENT-TYPE = "WEBSPEED":U THEN
            RETURN LOG-MANAGER:WRITE-MESSAGE (STRING (pdaMessage)) .

        RETURN FALSE .
    END.

    /*------------------------------------------------------------------------------
        Purpose: Writes an decimal message to the client logile and the default
                 logging sub system.
        Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL WriteMessage (pdeMessage AS DECIMAL):

        IF LOG-MANAGER:LOGFILE-NAME <> ? OR SESSION:REMOTE OR SESSION:CLIENT-TYPE = "WEBSPEED":U THEN
            RETURN LOG-MANAGER:WRITE-MESSAGE (STRING (pdeMessage)) .

        RETURN FALSE .
    END.

    /*------------------------------------------------------------------------------
        Purpose: Writes an integer message to the client logile and the default
                 logging sub system.
        Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL WriteMessage (piMessage AS INTEGER):

        IF LOG-MANAGER:LOGFILE-NAME <> ? OR SESSION:REMOTE OR SESSION:CLIENT-TYPE = "WEBSPEED":U THEN
            RETURN LOG-MANAGER:WRITE-MESSAGE (STRING (piMessage)) .

        RETURN FALSE .
    END.

    /*------------------------------------------------------------------------------
        Purpose: Writes an logical message to the client logile and the default
                 logging sub system.
        Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC STATIC LOGICAL WriteMessage (plMessage AS LOGICAL):

        IF LOG-MANAGER:LOGFILE-NAME <> ? OR SESSION:REMOTE OR SESSION:CLIENT-TYPE = "WEBSPEED":U THEN
            RETURN LOG-MANAGER:WRITE-MESSAGE (STRING (plMessage)) .

        RETURN FALSE .
    END.

    DESTRUCTOR PUBLIC LogHelper ( ):

        /* Make sure the log stream has been closed */
        CloseLogStream().
    END DESTRUCTOR.

END CLASS.