
/*------------------------------------------------------------------------
    File        : arat_create_task_sample.p
    Purpose     : Sample procedure using ARAT APIs to create task in RTB

    Syntax      :

    Description : Sample procedure using the ARAT APIs to create a task in Roundtable

    Author(s)   : Thomas Hansen, appSolutions a/s
    Created     : Wed Jul 09 16:42:46 CEST 2014
    Notes       :
  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */

ROUTINE-LEVEL ON ERROR UNDO, THROW.

{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

/* Task creation properties */
DEFINE VARIABLE gcRtbTaskSummary     AS CHARACTER NO-UNDO.
DEFINE VARIABLE gcRtbTaskDescription AS CHARACTER NO-UNDO.
DEFINE VARIABLE gcRtbTaskProgrammer  AS CHARACTER NO-UNDO.
DEFINE VARIABLE gcRtbTaskManager     AS CHARACTER NO-UNDO.
DEFINE VARIABLE gcRtbTaskUserRef     AS CHARACTER NO-UNDO.
 
/* ***************************  Main Block  *************************** */
oFileHelper = NEW appSolutions.appSolute.Utils.FileHelper(). 

ASSIGN 
    giParamsExpected = 13
    cPropsFile       = "examples/properties.json"
    gcLogFile        = "logs/arat_task_create.log". 

IF oFileHelper:FileExists(cPropsFile) THEN 
    COPY-LOB FROM FILE oFileHelper:GetFullPathName(cPropsFile) TO glcSessionProperties.

RUN initialize. 

RUN createTasks. 

CATCH Err AS Progress.Lang.Error :
    ASSIGN 
        cErrors = " ***":U + Err:GetMessage(1).
     
    IF VALID-OBJECT(oRTBAutomationApi) THEN 
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE 
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    RUN shutdown. 
    
    RETURN "1". /* Failure */

END CATCH.

FINALLY:
    RUN shutdown.           
END FINALLY.
/* **********************  Internal Procedures  *********************** */

PROCEDURE createTasks:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cWorkspace     AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cError         AS CHARACTER NO-UNDO.
    DEFINE VARIABLE i              AS INTEGER   NO-UNDO.
    DEFINE VARIABLE cReleaseReport AS CHARACTER NO-UNDO.
    DEFINE VARIABLE iTaskNum       AS INTEGER   NO-UNDO.
    
    DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, NEXT:
        ASSIGN 
            cWorkspace = ENTRY(i, gcWorkspaceList). 
        IF NUM-ENTRIES(gcWorkspaceList) > 1 THEN 
            ASSIGN
                cMessage = "  /*****************************************/".  
        
        IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN 
        DO ON ERROR UNDO: 
            ASSIGN 
                cMessage = cMessage + SUBSTITUTE("   Creating task in workspace: &1", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).    

            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).         
            iTaskNum = oRTBAutomationApi:RTBProxyHelper:custRtbCreateTask(
                INPUT  cWorkspace, 
                INPUT  gcRtbTaskSummary, 
                INPUT  gcRtbTaskDescription, 
                INPUT  gcRtbTaskUserRef,  
                INPUT  gcRtbTaskProgrammer, 
                INPUT  gcRtbTaskManager).
                
            IF iTaskNum NE 0 THEN 
            DO:
                ASSIGN 
                    cMessage = SUBSTITUTE("   Created task number : &1", iTaskNum). 
                oRTBAutomationApi:DisplayMessage(cMessage, TRUE, TRUE ). 

                /***DIAG***/
                MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
                    "Created task number : " iTaskNum   
                    VIEW-AS ALERT-BOX INFO BUTTONS OK.  
            END.
            ELSE 
            DO:
                ASSIGN 
                    cMessage = SUBSTITUTE("Something went wrong, task was not created for workspace '&1'", cWorkspace).
                oRTBAutomationApi:DisplayMessage(cMessage ).  
            END.
                    
            CATCH err AS Progress.Lang.Error :
                IF oRTBAutomationApi:DebugOn THEN
                    oRTBAutomationApi:DisplayMessage(SUBSTITUTE("   ** &1 ", err:GetMessage(1)), YES, YES).
                
                UNDO, THROW err.
            END CATCH.
        END.
        
        CATCH err AS Progress.Lang.Error :
            IF oRTBAutomationApi:DebugOn THEN
                oRTBAutomationApi:DisplayMessage(SUBSTITUTE("   ** &1 ", err:GetMessage(1)), YES, YES).
                
            UNDO, THROW err.
        END CATCH.
    END.

    CATCH err AS Progress.Lang.Error :
        IF oRTBAutomationApi:DebugOn THEN
            oRTBAutomationApi:DisplayMessage(SUBSTITUTE("   ** &1 ", err:GetMessage(1)), YES, YES).
                
        UNDO, THROW err.
    END CATCH.

END PROCEDURE.
 
 
PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:                                                                      
            Notes:                                                                        
    ------------------------------------------------------------------------------*/
    
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}
    
    RETURN.

    CATCH err AS Progress.Lang.Error:
        UNDO, THROW err.
    END CATCH.
    
    FINALLY:
        DELETE OBJECT hPropertiesTT NO-ERROR.   
    END FINALLY.
END PROCEDURE.

PROCEDURE readParams:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    IF ghBuf:AVAILABLE THEN 
    DO: 
        ASSIGN 
            gcRTBInstallPath         = ghBuf::RtbInstallPath
            gcUser                   = ghBuf::RtbUser
            gcPassword               = ghBuf::RtbPassword
            gcWorkspaceList          = ghBuf::RtbWspaceId
            gcAppServerConnectString = ghBuf::RtbAppServerConnection
            glUseAppServer           = ghBuf::RtbUseAppServer
            gcReportBaseDir          = ghBuf::RtbReportBaseDir
            glDebug                  = ghBuf::BuildDebug
            gcRtbTaskSummary         = ghBuf::RtbTaskSummary
            gcRtbTaskDescription     = ghBuf::RtbTaskDescription
            gcRtbTaskProgrammer      = ghBuf::RtbTaskProgrammer
            gcRtbTaskManager         = ghBuf::RtbTaskManager
            gcRtbTaskUserRef         = ghBuf::RtbTaskUserRef. 
    END.   

    RETURN. 

END PROCEDURE.

PROCEDURE shutDown:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    IF VALID-OBJECT(oRTBAutomationApi) THEN 
        oRTBAutomationApi:stopAPIs(). 

    DELETE OBJECT oRTBAutomationApi NO-ERROR.
    
    RETURN. 

END PROCEDURE.

