
/*------------------------------------------------------------------------
    File        : testRtbProxies.p
    Purpose     : Test RTBProxyHelper procedures

    Syntax      :

    Description :

    Author(s)   : Thomas Hansen
    Created     : Mon Mar 07 16:20:54 CET 2016
    Notes       :
  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */

ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING appSolutions.appSolute.Utils.* FROM PROPATH.
USING scm.custom.rtb.Utils.*         FROM PROPATH.
USING Progress.Lang.AppError         FROM PROPATH.

/* ********************  Preprocessor Definitions  ******************** */


/* ***************************  Main Block  *************************** */

DEFINE VARIABLE RtbProxyHelper         AS RTBProxyHelper  NO-UNDO.
DEFINE VARIABLE TempTableHelper        AS TempTableHelper NO-UNDO.

DEFINE VARIABLE lcResult               AS LONGCHAR        NO-UNDO.
DEFINE VARIABLE appServerHandle        AS HANDLE          NO-UNDO.
DEFINE VARIABLE appServerConnectString AS CHARACTER       NO-UNDO.
DEFINE VARIABLE RtbSessionID           AS INTEGER         NO-UNDO.
DEFINE VARIABLE RtbUser                AS CHARACTER       NO-UNDO.
DEFINE VARIABLE RtbUserPassword        AS CHARACTER       NO-UNDO.
DEFINE VARIABLE cError                 AS CHARACTER       NO-UNDO.

DEFINE VARIABLE hImportTaskData        AS HANDLE          NO-UNDO.

DEFINE TEMP-TABLE ttRtbImportTask LIKE rtb_task
    FIELD compileFailed AS LOGICAL INITIAL FALSE.

ASSIGN
    /* appServerConnectString = "AppServer://win-dev.nodeable.local:5162/as_rtbprps"*/
    RtbUser         = "sysop":U
    RtbUserPassword = "".

RtbProxyHelper         = NEW RTBProxyHelper().
TempTableHelper = NEW scm.custom.rtb.Utils.TempTableHelper().

RUN connectAppServer.

IF RtbSessionID = 0 THEN
    RtbSessionID = RtbProxyHelper:rtbSessionLogin(RtbUser, RtbUserPassword).

/*RUN testRtbGetProducts.
RUN testRtbGetProductsHandle.
RUN testRtbGetImportableReleases.
RUN testRtbGetImportableTasks.
RUN testRtbGetDeploymentSites.
RUN testRtbGetDeployments.
RUN testRtbGetProductsPmods.*/

/*RUN testRtbGetWorkspaces.
RUN testRtbGetModuleDefs.
RUN testRtbGetWorkspaceObjects.
RUN testRtbGetVersion.
RUN testRtbGetWorkspaceModules.
RUN testRtbGetWorkspaceReleases.
RUN testRtbGetWorkspaceSourcePmods.*/

/*RUN testCustRtbGetObjectTask.
RUN testRtbGetImportableTasksHandle.
RUN testRtbGetImportableTasksHandleStatic.*/

/*RUN testCustRtbGetImportTableTasks.
RUN testCustRtbGetImportTableTasksHandle.
RUN testRtbGetWorkspaceImports.
RUN testRtbGetWorkspaceImportsHandle.
RUN testRtbGetTaskHandle.*/

CATCH err AS Progress.Lang.Error :
    MESSAGE err:GetMessage(1)
        VIEW-AS ALERT-BOX.

END CATCH.

FINALLY:
    RtbProxyHelper:rtbSessionLogout(RtbSessionID).

    IF VALID-HANDLE(appServerHandle) AND appServerHandle:CONNECTED () THEN
        appServerHandle:DISCONNECT () NO-ERROR.

    DELETE OBJECT RtbProxyHelper NO-ERROR.
    DELETE OBJECT TempTableHelper NO-ERROR.
END FINALLY.



/* **********************  Internal Procedures  *********************** */

PROCEDURE connectAppServer:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    IF NOT VALID-HANDLE(AppServerHandle) OR
        AppServerHandle = SESSION:HANDLE THEN
        CREATE SERVER AppServerHandle.

    IF AppServerConnectString  BEGINS "AppServer:":U THEN
    DO:
        ASSIGN
            AppServerConnectString = SUBSTITUTE("-URL &1", AppServerConnectString).

        IF NOT AppServerHandle:CONNECTED() THEN
            AppServerHandle:CONNECT(SUBSTITUTE("&1", AppServerConnectString )) /*NO-ERROR*/.

        IF AppServerHandle:CONNECTED () THEN
        DO:
            ASSIGN
                RTBProxyHelper:AppServerHandle = AppServerHandle.

            IF AppServerHandle NE SESSION:HANDLE THEN
            DO ON ERROR UNDO, THROW:
                RtbSessionID = RTBProxyHelper:rtbSessionLogin(RtbUser,RtbUserPassword).

                CATCH err AS Progress.Lang.Error :
                    UNDO, THROW err.
                END CATCH.
            END.

        END.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

END PROCEDURE.

PROCEDURE testRtbGetProducts:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult AS LONGCHAR NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetProducts().

    MESSAGE
        "lcResult : " STRING(lcResult)
        VIEW-AS ALERT-BOX.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

END PROCEDURE.

PROCEDURE testRtbGetProductsHandle:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cRowIdent     AS CHARACTER NO-UNDO INITIAL "FIRST".
    DEFINE VARIABLE iRowsToreturn AS INTEGER   NO-UNDO.
    DEFINE VARIABLE lcJsonResult  AS LONGCHAR  NO-UNDO.

    DEFINE VARIABLE hData         AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hBuf          AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hQry          AS HANDLE    NO-UNDO.

    hData = RtbProxyHelper:rtbGetProductsHandle(
        INPUT-OUTPUT cRowIdent,
        INPUT        iRowsToReturn).

    MESSAGE "hData : " hData SKIP
        VALID-HANDLE(hData)
        VIEW-AS ALERT-BOX.

    IF VALID-HANDLE(hData) THEN
    DO:
        hBuf = hData:DEFAULT-BUFFER-HANDLE.
        CREATE QUERY hQry.
        hQry:SET-BUFFERS(hBuf).
        hQry:QUERY-PREPARE("FOR EACH " + hData:NAME).
        hQry:QUERY-OPEN().
        hQry:GET-FIRST().

        REPEAT WHILE NOT hQry:QUERY-OFF-END:
            MESSAGE hBuf::product-id
                VIEW-AS ALERT-BOX.
            hQry:GET-NEXT ().
        END.
    END.

    RETURN.

    CATCH err AS Progress.Lang.Error :
        UNDO, THROW err.
    END CATCH.

    FINALLY:
        DELETE OBJECT hData NO-ERROR.
    END FINALLY.


END PROCEDURE.

PROCEDURE testRtbGetImportableTasksHandle:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cRowIdent     AS CHARACTER NO-UNDO INITIAL "FIRST".
    DEFINE VARIABLE iRowsToreturn AS INTEGER   NO-UNDO.
    DEFINE VARIABLE lcJsonResult  AS LONGCHAR  NO-UNDO.

    DEFINE VARIABLE hData         AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hBuf          AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hQry          AS HANDLE    NO-UNDO.

    hData = RtbProxyHelper:rtbGetImportableTasksHandle(
        INPUT-OUTPUT cRowIdent,
        INPUT        iRowsToReturn,
        INPUT        "devel").

    MESSAGE "hData : " hData SKIP
        VALID-HANDLE(hData)
        VIEW-AS ALERT-BOX.

    IF VALID-HANDLE(hData) THEN
    DO:
        hBuf = hData:DEFAULT-BUFFER-HANDLE.
        CREATE QUERY hQry.
        hQry:SET-BUFFERS(hBuf).
        hQry:QUERY-PREPARE("FOR EACH " + hData:NAME).
        hQry:QUERY-OPEN().
        hQry:GET-FIRST().

        REPEAT WHILE NOT hQry:QUERY-OFF-END:
            MESSAGE hBuf::task-num
                VIEW-AS ALERT-BOX.
            hQry:GET-NEXT ().
        END.
    END.

    RETURN.

    CATCH err AS Progress.Lang.Error :
        UNDO, THROW err.
    END CATCH.

    FINALLY:
        DELETE OBJECT hData NO-ERROR.
    END FINALLY.


END PROCEDURE.

PROCEDURE testRtbGetImportableTasksHandleStatic:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cRowIdent     AS CHARACTER NO-UNDO INITIAL "FIRST".
    DEFINE VARIABLE iRowsToreturn AS INTEGER   NO-UNDO.
    DEFINE VARIABLE lcJsonResult  AS LONGCHAR  NO-UNDO.

    DEFINE VARIABLE hData         AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hBuf          AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hQry          AS HANDLE    NO-UNDO.

    hData = RtbProxyHelper:rtbGetImportableTasksHandle(
        INPUT-OUTPUT cRowIdent,
        INPUT        iRowsToReturn,
        INPUT        "devel").

    IF VALID-HANDLE(hData) THEN
    DO:
        hBuf = hData:DEFAULT-BUFFER-HANDLE.
        CREATE QUERY hQry.
        hQry:SET-BUFFERS(hBuf).
        hQry:QUERY-PREPARE("FOR EACH " + hData:NAME).
        hQry:QUERY-OPEN().
        hQry:GET-FIRST().

        REPEAT WHILE NOT hQry:QUERY-OFF-END:
            CREATE ttRtbImportTask.
            ASSIGN
                ttRtbImportTask.task-num    = hBuf::task-num
                ttRtbImportTask.task-status = hBuf::task-status
                ttRtbImportTask.wspace-id   = hBuf::wspace-id
                ttRtbImportTask.manager     = hBuf::manager
                ttRtbImportTask.programmer  = hBuf::programmer
                ttRtbImportTask.task-guid   = hBuf::task-guid.

            hQry:GET-NEXT ().
        END.
    END.

    FOR EACH ttRtbImportTask:
        DISPLAY ttRtbImportTask.
    END.

    RETURN.

    CATCH err AS Progress.Lang.Error :
        UNDO, THROW err.
    END CATCH.

    FINALLY:
        DELETE OBJECT hData NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetWorkspaces:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetWorkspaces(TRUE).
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetWorkspaces.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    /*    MESSAGE                           */
    /*        "lcResult : " STRING(lcResult)*/
    /*        VIEW-AS ALERT-BOX.            */

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetImportableReleases:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetImportableReleases("Test").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetImportableReleases.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetImportableTasks:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetImportableTasks("devel").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetImportableTasks.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetDeploymentSites:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetDeploymentSites("Test").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetDeploymentSites.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetDeployments:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetDeployments("Test", "Kaunas Sports2k").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetDeployments.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetProductsPmods:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetProductsPmods("014AD").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetProductsPmods.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetModuleDefs:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetModuleDefs().
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetModuleDefs.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetWorkspaceObjects:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetWorkspaceObjects(10, "devel", "PCODE", "*").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetWorkspaceObjects.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetVersion:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetVersion("PCODE", "arwdwn.gif", "Images", 10000).
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetVersion.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetWorkspaceModules:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetWorkspaceModules("test").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetWorkspaceModules.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetWorkspaceReleases:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetWorkspaceReleases("test").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetWorkspaceReleases.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetWorkspaceImports:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetWorkspaceImports("test2").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetWorkspaceImports.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testRtbGetWorkspaceImportsHandle:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE hData AS HANDLE NO-UNDO.
    DEFINE VARIABLE hBuf  AS HANDLE NO-UNDO.
    DEFINE VARIABLE hQry  AS HANDLE NO-UNDO.

    hData = RtbProxyHelper:rtbGetWorkspaceImportsHandle(
        INPUT        "test2").

    MESSAGE "hData : " hData SKIP
        VALID-HANDLE(hData)
        VIEW-AS ALERT-BOX.

    IF VALID-HANDLE(hData) THEN
    DO:
        hBuf = hData:DEFAULT-BUFFER-HANDLE.
        CREATE QUERY hQry.
        hQry:SET-BUFFERS(hBuf).
        hQry:QUERY-PREPARE("FOR EACH " + hData:NAME).
        hQry:QUERY-OPEN().
        hQry:GET-FIRST().

        REPEAT WHILE NOT hQry:QUERY-OFF-END:
            MESSAGE hBuf::object
                VIEW-AS ALERT-BOX.
            hQry:GET-NEXT ().
        END.
    END.

    RETURN.

    CATCH err AS Progress.Lang.Error :
        UNDO, THROW err.
    END CATCH.

    FINALLY:
        DELETE OBJECT hData NO-ERROR.
    END FINALLY.


END PROCEDURE.

PROCEDURE testRtbGetWorkspaceSourcePmods:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:rtbGetWorkspaceSourcePmods("test", "", "", "", TRUE, TRUE).
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/testRtbGetWorkspaceSourcePmods.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testCustRtbGetImportTableTasks:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult    AS LONGCHAR   NO-UNDO.
    DEFINE VARIABLE oFileHelper AS FileHelper NO-UNDO.
    DEFINE VARIABLE cFilename   AS CHARACTER  NO-UNDO.

    lcResult = RtbProxyHelper:custRtbGetImportTableTasks("test2").
    oFileHelper = NEW FileHelper().

    IF lcResult NE ? THEN
    DO:
        ASSIGN
            cFilename = "tests/custRtbGetImportTableTasks.json".
        COPY-LOB FROM lcResult TO FILE cFilename.
        cFilename = oFileHelper:getFullPathName(cFilename).

        MESSAGE
            SUBSTITUTE("Output located in: &1", cFilename)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

    FINALLY:
        DELETE OBJECT oFileHelper NO-ERROR.
    END FINALLY.

END PROCEDURE.

PROCEDURE testCustRtbGetImportTableTasksHandle:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult  AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE cWhere    AS CHARACTER NO-UNDO.
    DEFINE VARIABLE hBuf      AS HANDLE    NO-UNDO.
    DEFINE VARIABLE cTaskList AS CHARACTER NO-UNDO.

    hImportTaskData = RTBProxyHelper:custRtbGetImportTableTasksHandle("test2", OUTPUT cTaskList).

    ASSIGN
        cWhere = " where task-num eq 1".

    hBuf = hImportTaskData:DEFAULT-BUFFER-HANDLE.
    hBuf:FIND-FIRST(cWhere) NO-ERROR.

    IF hBuf:AVAILABLE THEN
    DO:
        MESSAGE hBuf::compileFailed
            VIEW-AS ALERT-BOX.

        ASSIGN
            hBuf::compileFailed = TRUE.

        MESSAGE hBuf::compileFailed
            VIEW-AS ALERT-BOX.
    END.

    MESSAGE hImportTaskData
        VIEW-AS ALERT-BOX.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

END PROCEDURE.

PROCEDURE testCustRtbGetObjectTask:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcResult AS INTEGER NO-UNDO.

    lcResult = RtbProxyHelper:custRtbGetObjectTask("PCODE", "subpub.p", "GUI", 10000).

    IF lcResult NE ? THEN
    DO:
        MESSAGE
            SUBSTITUTE("Output : &1", lcResult)
            VIEW-AS ALERT-BOX.
    END.

    CATCH err AS Progress.Lang.Error :
        MESSAGE err:GetMessage(1)
            VIEW-AS ALERT-BOX.
    END CATCH.

END PROCEDURE.

PROCEDURE testRtbGetTaskHandle:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE hData AS HANDLE NO-UNDO.
    DEFINE VARIABLE hBuf  AS HANDLE NO-UNDO.
    DEFINE VARIABLE hQry  AS HANDLE NO-UNDO.

    hData = RtbProxyHelper:rtbGetTaskHandle(
        INPUT        1).

    MESSAGE "hData : " hData SKIP
        VALID-HANDLE(hData)
        VIEW-AS ALERT-BOX.

    IF VALID-HANDLE(hData) THEN
    DO:
        hBuf = hData:DEFAULT-BUFFER-HANDLE.
        CREATE QUERY hQry.
        hQry:SET-BUFFERS(hBuf).
        hQry:QUERY-PREPARE("FOR EACH " + hData:NAME).
        hQry:QUERY-OPEN().
        hQry:GET-FIRST().

        REPEAT WHILE NOT hQry:QUERY-OFF-END:
            MESSAGE hBuf::task-num SKIP hBuf::task-guid
                VIEW-AS ALERT-BOX.
            hQry:GET-NEXT ().
        END.
    END.

    RETURN.

    CATCH err AS Progress.Lang.Error :
        UNDO, THROW err.
    END CATCH.

    FINALLY:
        DELETE OBJECT hData NO-ERROR.
    END FINALLY.


END PROCEDURE.
